
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: font.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "font.h"
#include "surface.h"
#include "main.h"

font::font() {
  _height = 0;
  for ( Uint16 c = 0 ; c < 256 ; ++c ) {
    metrics[c].x = 0; metrics[c].w = 0; metrics[c].s = 0; metrics[c].k = 0;
  }

  _image = NULL;
}

font::~font() {
  delete _image;
}

bool font::load(const char *name) {
  bool success = false;

  char buf[256];
  sprintf(buf, "fnt/%s.fnt", name);
  FILE *desc = fopen(buf, "r");
  if ( desc ) {
    char image_path[256] = "";
    char metric_path[256] = "";

    while ( fgets(buf, sizeof(buf), desc) ) {
      for ( char *s = buf + strlen(buf) - 1 ; (s >= buf) && isspace(*s) ; *s-- = 0 );

      if ( (strlen(buf) > 0) && (buf[0] != '#') ) {
	if ( !strncmp(buf, "IMAGE=", 6) ) 
	  strncpy(image_path, buf + 6, sizeof(image_path));
	else if ( !strncmp(buf, "METRIC=", 7) ) 
	  strncpy(metric_path, buf + 7, sizeof(metric_path));
	else
	  show_message("Malformed line in font description file '%s':\n%s\n", name, buf);
      }
    }

    if ( (strlen(image_path) > 0) && (strlen(metric_path) > 0) ) {
      sprintf(buf, "fnt/%s", image_path);
      _image = surface::load_image(buf);
      if ( _image ) {
	sprintf(buf, "fnt/%s", metric_path);
	FILE *met = fopen(buf, "rb");
	if ( met ) {
	  short height = 0;
	  if ( fread(&height, sizeof(short), 1, met) == 1 ) {
	    _height = height;

	    success = true;

	    unsigned char c;
	    while ( success && (fread(&c, sizeof(unsigned char), 1, met) == 1) ) {
	      short x, w, s, k;
	      if ( (fread(&x, sizeof(short), 1, met) == 1) &&
		   (fread(&w, sizeof(short), 1, met) == 1) &&
		   (fread(&s, sizeof(short), 1, met) == 1) &&
		   (fread(&k, sizeof(short), 1, met) == 1) ) {
		metrics[c].x = x;
		metrics[c].w = w;
		metrics[c].s = s;
		metrics[c].k = k;
	      }
	      else {
		show_message("I/O error while reading 'fnt/%s'.\n", metric_path);
		success = false;
	      }
	    }
	  }
	  else
	    show_message("I/O error while reading 'fnt/%s'.\n", metric_path);

	  fclose(met);
	}
	else
	  show_message("Unable to load font metrics 'fnt/%s'.\n", metric_path);
      }
      else
	show_message("Unable to load font image 'fnt/%s'.\n", image_path);
    }
    else
      show_message("Malformed font description '%s'.\n", name);
   
    fclose(desc);
  }
  else
    show_message("Unable to open font '%s'.\n", name);

  return( success );
}

Uint16 font::strwidth(const Uint8 *str) const {
  Uint16 w = 0;

  while ( *str ) {
    w += width(*str);
    str++;
  }

  return( w );
}

rectangle font::get_rect(Uint8 ch) const {
  rectangle r;

  if ( metrics[ch].w > 0 ) {
    r.x(metrics[ch].x);
    r.y(0);
    r.w(metrics[ch].w);
    r.h(_height);
  }

  return( r );
}

