
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: game.h,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#ifndef _GAME_H_
#define _GAME_H_

#include "main.h"
#include "font.h"
#include "surface_video.h"
#include "sarray.h"
#include "object.h"
#include "group.h"
#include "level.h"

#define PANEL_WIDTH 160
#define PANEL_HEIGHT SCREEN_HEIGHT
#define PANEL_XPOS (SCREEN_WIDTH - PANEL_WIDTH)
#define PANEL_YPOS 0

#define MAIN_WIDTH (SCREEN_WIDTH - PANEL_WIDTH)
#define MAIN_HEIGHT SCREEN_HEIGHT
#define MAIN_XPOS 0
#define MAIN_YPOS 0

#define MESSAGE_COUNT 16

#define MAX_LOADED_IMAGES 64
#define MAX_LOADED_BOBS 128
#define MAX_LOADED_SONGS 16
#define MAX_LOADED_SOUNDS 64
#define MAX_LOADED_FONTS 16

#define HIGHSCORE_COUNT 10

#define BONUS_NONE     0
#define BONUS_ENERGY   1
#define BONUS_SHIELD   2
#define BONUS_LIFE     3
#define BONUS_LASER    4
#define BONUS_PLASMA   5
#define BONUS_BALL     6
#define BONUS_SEEKER   7

#define SOUND_EXPL_1     0
#define SOUND_EXPL_2     1
#define SOUND_EXPL_3     2
#define SOUND_EXPL_4     3
#define SOUND_EXPL_COUNT 4

#define SOUND_LAYER_ENEMY   0
#define SOUND_LAYER_PLAYER  1
#define SOUND_LAYER_MISC    2

#define INPUT_KEYBOARD  0
#define INPUT_JOYSTICK  1

class game : public surface_video {
 protected:
  /* settings */
  bool _enable_audio;
  bool _enable_music;
  bool _enable_sound;
  bool _enable_fullscreen;
  bool _enable_joystick;

  Uint16 _input_device;
  Uint16 _joystick_number;

  /* ... */
  SDL_Joystick *_joystick_handle;

  Uint16 _panel_font_id;
  Uint16 _tiny_font_id;
  Uint16 _flow_font_id;

  Uint32 _signal_mask;
  Sint32 _xpos, _ypos;
  Sint32 _yvel;
  Sint32 _current_time;


  bool _quit_flag;
  bool _skip_intro;

  bool _enable_fps;
  Uint32 _frames;
  Uint32 _fps_start_ticks;
  Uint32 _fps;

  Uint32 _old_energy;
  Uint32 _old_score;
  Uint32 _old_lifes;

  struct message { char *text; Sint32 x, y, s; Uint16 phase; Sint32 time; };
  struct message _messages[MESSAGE_COUNT];

  sarray<group> _groups;
  typedef sarray<group>::iterator _group_iterator;
  typedef sarray<group>::const_iterator _const_group_iterator;
  sarray<object> _objects;
  sarray<object> _added_objects;
  typedef sarray<object>::iterator _object_iterator;
  typedef sarray<object>::const_iterator _const_object_iterator;

  struct loaded_bob { char *path; surface *s; Uint16 cnt; Uint32 time; };
  struct loaded_bob _loaded_bobs[MAX_LOADED_BOBS];
  Uint32 _bobs_memory;
  void flush_bobs();

  struct loaded_image { char *path; surface *s; Uint16 cnt; Uint32 time; };
  struct loaded_image _loaded_images[MAX_LOADED_IMAGES];
  Uint32 _images_memory;
  void flush_images();

  struct loaded_song { char *path; Mix_Music *s; Uint16 cnt; Uint32 time; };
  struct loaded_song _loaded_songs[MAX_LOADED_SONGS];
  Uint32 _songs_memory;
  void flush_songs();

  struct loaded_sound { char *path; Mix_Chunk *s; Uint16 cnt; Uint32 time; };
  struct loaded_sound _loaded_sounds[MAX_LOADED_SOUNDS];
  Uint32 _sounds_memory;
  void flush_sounds();

  struct loaded_font { char *path; font *s; Uint16 cnt; Uint32 time; };
  struct loaded_font _loaded_fonts[MAX_LOADED_FONTS];
  Uint32 _fonts_memory;
  void flush_fonts();

  Uint16 _panel_id;
  Uint16 _slot_upper_id[4];
  Uint16 _slot_lower_id[4];
  Uint16 _slot_status[4];
  Sint32 _slot_time[4];

  Uint16 _expl_id[SOUND_EXPL_COUNT];



  bool _paused;

  Uint16 _bigenemy_state;
  Uint32 _bigenemy_max;
  Uint32 _bigenemy_current;
  surface *_bigenemy_surface;
  Sint32 _bigenemy_last_time;

  struct highscore { char name[16]; Uint32 score; Uint16 level; };
  struct highscore _highscores[HIGHSCORE_COUNT];
  bool read_highscores();
  bool write_highscores();

  void intro();
  void menu();
  void start();
  bool enter_code();
  void enter_game(Uint16);
  void complete();
  void gameover();
  void options_callback(Sint16, Uint16);
  void options();
  void info();
  void help();
  bool play();

#if ( 0 ) 
  struct scroll_screen_spec {
    char *back_pic;
    char **text;
    Sint32 speed;
  };
  void scroll_screen(scroll_screen_spec &);
#endif

  struct menu_screen_entry {
    Sint16 type;
    Sint16 xpos, ypos;
    char *text;
    char *fnt;
    Uint16 option;
    char **options;
    void (game::*callback)(Sint16, Uint16); 
  };
  struct menu_screen_spec {
    char *back_pic;
    menu_screen_entry *entries;
    Uint32 timeout;
  };
  Sint16 menu_screen(menu_screen_spec &);

  void eat_input();

  sarray<class level> _levels;
  class level *_level;
  class object_player *_player;

  void next_frame();
  void update_panel();

  Uint16 _title_song;
  Uint16 _complete_song;
  Uint16 _gameover_song;
  Uint16 _song_current;

 public:
  game();
  ~game();

  Sint32 fxpos() const { return( _xpos ); }
  Sint32 xpos() const { return( _xpos >> 8 ); }
  Sint32 fypos() const { return( _ypos ); }
  Sint32 ypos() const { return( _ypos >> 8 ); }
  Sint32 fyvel() const { return( _yvel ); }
  Sint32 yvel() const { return( _yvel >> 8 ); }
  Sint32 map_width() const { return( _level->width() ); }
  Sint32 map_height() const { return( _level->height() ); }
  class level *level() const { return( _level ); }

  Uint32 signal_mask() { return( _signal_mask ); }
  void signal_mask(Uint32 s) { _signal_mask = s; }
  void add_signal(Uint32 s) { _signal_mask |= s; }
  void rem_signal(Uint32 s) { _signal_mask &= ~s; }

  void read_settings();
  void write_settings();

  bool audio_enabled() const { return( _enable_audio ); }
  void enable_audio(bool b) { _enable_audio = b; }

  bool music_enabled() const { return( _enable_music ); }
  void enable_music(bool b) { _enable_music = b; }

  bool sound_enabled() const { return( _enable_sound ); }
  void enable_sound(bool b) { _enable_sound = b; }

  bool fullscreen_enabled() const { return( _enable_fullscreen ); }
  void enable_fullscreen(bool b) { _enable_fullscreen = b; }

  bool joystick_enabled() const { return( _enable_joystick ); }
  void enable_joystick(bool b) { _enable_joystick = b; }

  bool initialize();
  void go(int);

  class object_player *player() const { return( _player ); }

  Uint16 input_device() const { return( _input_device ); }
  Uint16 joystick_number() const { return( _joystick_number ); }
  SDL_Joystick *joystick_handle() const { return( _joystick_handle ); }

  class group *obtain_group(const char *, Uint32);

  void add_object(class object *obj) {
    _added_objects += obj;
  }

  object **first_object();
  object **next_object(object **);

  Uint16 obtain_bob(const char *);
  void release_bob(Uint16);
  surface *get_bob(Uint16 id) { return( _loaded_bobs[id].s ); }

  Uint16 obtain_image(const char *);
  void release_image(Uint16);
  surface *get_image(Uint16 id) { return( _loaded_images[id].s ); }

  Uint16 obtain_song(const char *);
  void release_song(Uint16);
  Mix_Music *get_song(Uint16 id) { return( _loaded_songs[id].s ); }

  void play_song(Uint16);
  void stop_song();

  Uint16 obtain_sound(const char *);
  void release_sound(Uint16);
  Mix_Chunk *get_sound(Uint16 id) { return( _loaded_sounds[id].s ); }

  void play_sound(Uint16, Uint16, Uint16 = 128);
  void play_expl_sound(Uint16, Uint16, Uint16 = 128);
  void stop_sounds();

  Uint16 obtain_font(const char *);
  void release_font(Uint16);
  font *get_font(Uint16 id) { return( _loaded_fonts[id].s ); }

  Uint32 current_time() const { return( _current_time ); }

  char *level_code();

  void bigenemy_init(Uint32);
  void bigenemy_update(Uint32);
  void bigenemy_cleanup();

  void add_message(const char *, Sint32 = 350);
};


#endif
