
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: game1.cc,v 1.1 2000/10/12 14:35:03 pink Exp pink $ */

#include <stdlib.h>
#include <memory.h>
#include <ctype.h>
#include <smpeg/smpeg.h>
#include "main.h"
#include "game.h"
#include "group.h"
#include "group_table.h"
#include "object.h"
#include "object_player.h"
#include "object_bonus.h"
#include "font.h"
#include "surface.h"

#include "leveldemo.h"

#define MAX_IMAGES_MEMORY 5000000
#define MAX_BOBS_MEMORY   2000000
#define MAX_SONGS_MEMORY  1000000
#define MAX_SOUNDS_MEMORY 1000000
#define MAX_FONTS_MEMORY  1000000

#define SOUND_LAYER_COUNT 5

#define SOUND_LAYER_ENEMY_MIN 0
#define SOUND_LAYER_ENEMY_MAX 2
#define SOUND_LAYER_PLAYER_MIN 3
#define SOUND_LAYER_PLAYER_MAX 3
#define SOUND_LAYER_MISC_MIN 4
#define SOUND_LAYER_MISC_MAX 4


game::game() : surface_video(SCREEN_WIDTH, SCREEN_HEIGHT) {
  _enable_audio = true;
  _enable_music = true;
  _enable_sound = true;
  _enable_fullscreen = false;
  _enable_joystick = true;
  _input_device = INPUT_KEYBOARD;
  _joystick_number = 0;

  _quit_flag = false;
  


  _skip_intro = false;

  _joystick_handle = NULL;

  _levels += new leveldemo();

  for ( Uint16 i = 0 ; i < MAX_LOADED_FONTS ; ++i ) {
    _loaded_fonts[i].path = NULL;
    _loaded_fonts[i].s = NULL;
    _loaded_fonts[i].cnt = 0;
  }
  _fonts_memory = 0;

  for ( Uint16 i = 0 ; i < MAX_LOADED_SOUNDS ; ++i ) {
    _loaded_sounds[i].path = NULL;
    _loaded_sounds[i].s = NULL;
    _loaded_sounds[i].cnt = 0;
  }
  _sounds_memory = 0;

  for ( Uint16 i = 0 ; i < MAX_LOADED_SONGS ; ++i ) {
    _loaded_songs[i].path = NULL;
    _loaded_songs[i].s = NULL;
    _loaded_songs[i].cnt = 0;
  }
  _songs_memory = 0;

  for ( Uint16 i = 0 ; i < MAX_LOADED_IMAGES ; ++i ) {
    _loaded_images[i].path = NULL;
    _loaded_images[i].s = NULL;
    _loaded_images[i].cnt = 0;
  }
  _images_memory = 0;

  for ( Uint16 i = 0 ; i < MAX_LOADED_BOBS ; ++i ) {
    _loaded_bobs[i].path = NULL;
    _loaded_bobs[i].s = NULL;
    _loaded_bobs[i].cnt = 0;
  }
  _bobs_memory = 0;

  for ( Uint16 i = 0 ; i < MESSAGE_COUNT ; ++i ) {
    _messages[i].text = NULL;
  }

  _title_song = 0xffff;
  _complete_song = 0xffff;
  _gameover_song = 0xffff;
  _song_current = 0xffff;
}

game::~game() {
  dprintf("~game():");
  denter();

  release_song(_title_song);
  release_song(_complete_song);
  release_song(_gameover_song);

  for ( Uint16 i = 0 ; i < MESSAGE_COUNT ; ++i ) {
    delete _messages[i].text;
  }

  dprintf("flushing bobs...");
  for ( int i = 0 ; i < MAX_LOADED_BOBS ; ++i ) {
    delete _loaded_bobs[i].path;
    delete _loaded_bobs[i].s;
  }

  dprintf("flushing images...");
  for ( int i = 0 ; i < MAX_LOADED_IMAGES ; ++i ) {
    delete _loaded_images[i].path;
    delete _loaded_images[i].s;
  }

  dprintf("flushing songs...");
  for ( int i = 0 ; i < MAX_LOADED_SONGS ; ++i ) {
    delete _loaded_songs[i].path;
    if ( _loaded_songs[i].s )
      Mix_FreeMusic(_loaded_songs[i].s);
  }

  dprintf("flushing fonts...");
  for ( int i = 0 ; i < MAX_LOADED_FONTS ; ++i ) {
    delete _loaded_fonts[i].path;
    if ( _loaded_fonts[i].s )
      delete _loaded_fonts[i].s;
  }

  dprintf("flushing sounds...");
  for ( int i = 0 ; i < MAX_LOADED_SOUNDS ; ++i ) {
    delete _loaded_sounds[i].path;
    if ( _loaded_sounds[i].s )
      Mix_FreeChunk(_loaded_sounds[i].s);
  }

  dprintf("flushing objects...");
  for ( _object_iterator i = _objects.begin() ; i != _objects.end() ; ++i ) {
    delete *i;
    *i = NULL;
  }

  for ( _object_iterator i = _added_objects.begin() ; i != _added_objects.end() ; ++i ) {
    delete *i;
    *i = NULL;
  }

  dprintf("flushing groups...");
  for ( _group_iterator i = _groups.begin() ; i != _groups.end() ; ++i ) {
    delete *i;
    *i = NULL;
  }

  dprintf("flushing levels...");
  for ( sarray<class level>::iterator l = _levels.begin() ; l != _levels.end() ; ++l ) {
    delete *l;
    *l = NULL;
  }

  dprintf("close audio...");
  Mix_CloseAudio();
  
  dprintf("done.");
  dleave();
}

void game::read_settings() {
  /* set defaults */
  _enable_audio = true;
  _enable_music = true;
  _enable_sound = true;
  _enable_fullscreen = false;
  _enable_joystick = true;
  _input_device = INPUT_KEYBOARD;
  _joystick_number = 0;

  FILE *fp;
  char path[1024];
  char *home = getenv("HOME");
  if ( !home )
    home = ".";

  sprintf(path, "%s/.gammapatrol", home);
  fp = fopen(path, "r");
  if ( fp ) {
    char buf[256];

    int lineno = 1;
    while ( fgets(buf, sizeof(buf), fp) ) {
      for ( char *e = buf + strlen(buf) - 1 ; (e >= buf) && isspace(*e) ; *e-- = 0);
      char *s = buf;
      while ( *s && isspace(*s) )
	s++;

      if ( (*s == '#') || (*s == 0) )
	continue;

      char *key = s;

      while ( *s && !isspace(*s) && (*s != '=') )
	s++;

      int key_len = s - key;

      while ( *s && (isspace(*s) || (*s == '=')) )
	s++;

      char *val = s;

      while ( *s && !isspace(*s) )
	s++;

      int val_len = s - val;

      if ( (key_len == strlen("enable_music")) && !strncmp("enable_music", key, key_len) ) {
	if ( (val_len == strlen("true")) && !strncmp("true", val, val_len) )
	  _enable_music = true;
	else
	  _enable_music = false;
      }
      else if ( (key_len == strlen("enable_sound")) && !strncmp("enable_sound", key, key_len) ) {
	if ( (val_len == strlen("true")) && !strncmp("true", val, val_len) )
	  _enable_sound = true;
	else
	  _enable_sound = false;
      }
      else if ( (key_len == strlen("audio_quality")) && !strncmp("audio_quality", key, key_len) ) {
	/* ... */
      }
      else if ( (key_len == strlen("enable_fullscreen")) && !strncmp("enable_fullscreen", key, key_len) ) {
	if ( (val_len == strlen("true")) && !strncmp("true", val, val_len) )
	  _enable_fullscreen = true;
	else
	  _enable_fullscreen = false;
      }
      else if ( (key_len == strlen("input_device")) && !strncmp("input_device", key, key_len) ) {
	if ( (val_len == strlen("joystick")) && !strncmp("joystick", val, val_len) )
	  _input_device = INPUT_JOYSTICK;
	else
	  _input_device = INPUT_KEYBOARD;
      }
      else if ( (key_len == strlen("joystick_number")) && !strncmp("joystick_number", key, key_len) ) {
	_joystick_number = atoi(val);
      }
      else
	printf("%s: error in line %d.\n", path, lineno);

      lineno++;
    }

    fclose(fp);
  }
}

void game::write_settings() {
  FILE *fp;
  char path[1024];
  char *home = getenv("HOME");
  if ( !home )
    home = ".";

  sprintf(path, "%s/.gammapatrol", home);
  fp = fopen(path, "w");
  if ( fp ) {
    fprintf(fp, "# audio settings\n");
    fprintf(fp, "enable_music = %s\n", _enable_audio ? "true" : "false");
    fprintf(fp, "enable_sound = %s\n", _enable_sound ? "true" : "false");
    fprintf(fp, "audio_quality = medium\n");
    fprintf(fp, "# video settings\n");
    fprintf(fp, "enable_fullscreen = %s\n", _enable_fullscreen ? "true" : "false");
    fprintf(fp, "# input settings\n");
    fprintf(fp, "input_device = %s\n", (_input_device == INPUT_KEYBOARD) ? "keyboard" : "joystick");
    fprintf(fp, "joystick_number = %d\n", _joystick_number);

    fclose(fp);
  }
}

bool game::initialize() {
  bool success = true;

  if ( success ) {
    Uint32 sdl_flags = SDL_INIT_VIDEO;
    if ( audio_enabled() )
      sdl_flags |= SDL_INIT_AUDIO;
    if ( joystick_enabled() )
      sdl_flags |= SDL_INIT_JOYSTICK;

    if ( SDL_Init(sdl_flags) < 0 ) {
      show_message("Couldn't initialize SDL: %s\n", SDL_GetError());
      success = false;
    }
  }

  if ( success ) {
    SDL_WM_SetCaption("Gamma Patrol (Preview release)", NULL);
  }

  if ( success ) {
    if ( audio_enabled() ) {
      int freq = 11025;
      int chan = 2;
      int form = AUDIO_S8;
      int buf = (freq / 25 + 255) & -256;

      if ( Mix_OpenAudio(freq, form, chan, buf) < 0 ) {
	show_message("Couldn't open audio: %s\n", SDL_GetError());
	success = false;
      }
    }
  }

  if ( success && audio_enabled() ) {
    if ( Mix_ReserveChannels(SOUND_LAYER_COUNT) != SOUND_LAYER_COUNT ) {
      show_message("Unable to reserve %d sound channels.\n", SOUND_LAYER_COUNT);
      success = false;
    }

    if ( success && !Mix_GroupChannels(SOUND_LAYER_ENEMY_MIN, SOUND_LAYER_ENEMY_MAX, SOUND_LAYER_ENEMY) ) {
      show_message("Unable to set channel group %d to %d..%d.\n", SOUND_LAYER_ENEMY, SOUND_LAYER_ENEMY_MIN, SOUND_LAYER_ENEMY_MAX);
      success = false;
    }
    
    if ( success && !Mix_GroupChannels(SOUND_LAYER_PLAYER_MIN, SOUND_LAYER_PLAYER_MAX, SOUND_LAYER_PLAYER) ) {
      show_message("Unable to set channel group %d to %d..%d.\n", SOUND_LAYER_PLAYER, SOUND_LAYER_PLAYER_MIN, SOUND_LAYER_PLAYER_MAX);
      success = false;
    }
    
    if ( success && !Mix_GroupChannels(SOUND_LAYER_MISC_MIN, SOUND_LAYER_MISC_MAX, SOUND_LAYER_MISC) ) {
      show_message("Unable to set channel group %d to %d..%d.\n", SOUND_LAYER_MISC, SOUND_LAYER_MISC_MIN, SOUND_LAYER_MISC_MAX);
      success = false;
    }
  }

  if ( success ) {
    Uint32 flags = 0;

    if ( fullscreen_enabled() ) {
      flags |= SDL_FULLSCREEN;
      dprintf("Requesting fullscreen mode\n");
    }
       
    success = surface_video::initialize(flags);
  }

  SDL_ShowCursor(0);

  return( success );
}

bool game::read_highscores() {
  bool success = false;

  for ( int i = 0 ; i < HIGHSCORE_COUNT ; ++i ) {
    strcpy(_highscores[i].name, "");
    _highscores[i].score = 0;
    _highscores[i].level = 0;
  }

  FILE *fp = fopen("highscore.dat", "rb");
  if ( fp ) {
    for ( int i = 0 ; i < HIGHSCORE_COUNT ; ++i ) {
      char buf[sizeof(struct highscore)];
      if ( fread(buf, 1, sizeof(struct highscore), fp) != 1 ) {
        show_message("Unable to read highscore entry #%d.\n", i + 1);
        goto done;
      }

      for ( unsigned int j = 0 ; j < sizeof(struct highscore) ; ++j )
        buf[j] ^= ((j + 1) << 4) + 7 * j;

      memcpy(&_highscores[i], buf, sizeof(struct highscore));
    }

    success = true;

   done:
    fclose(fp);
  }

  return( success );
}

bool game::write_highscores() {
  bool success = false;

  FILE *fp = fopen("highscore.dat", "wb");
  if ( fp ) {
    for ( int i = 0 ; i < HIGHSCORE_COUNT ; ++i ) {
      char buf[sizeof(struct highscore)];

      memcpy(buf, &_highscores[i], sizeof(struct highscore));

      for ( unsigned int j = 0 ; j < sizeof(struct highscore) ; ++j )
        buf[j] ^= ((j + 1) << 4) + 7 * j;

      if ( fwrite(buf, 1, sizeof(struct highscore), fp) !=  1 ) {
        show_message("Unable to read highscore entry #%d.\n", i + 1);
        goto done;
      }
    }

    success = true;

   done:
    fclose(fp);
  }

  return( success );
}

void game::eat_input() {
  SDL_Event event;
  while(SDL_PollEvent(&event)) {
    switch (event.type) {
    case SDL_QUIT:
      _quit_flag = true;
      break;
    }
  }
}

#if ( 0 )
void game::scroll_screen(scroll_screen_spec &spec) {
  Uint16 _bg_id = obtain_image(spec.back_pic);
  if ( _bg_id != 0xffff ) {
    struct row {
      Sint32 y1, y2;
      char *text;
    };
    Uint16 row_count = 0;
    
    for ( char **r = spec.text ; *r ; ++r )
      ++row_count;

    row *rows = new row[row_count];

    row *rscan = rows;
    Sint32 yoffset = 480;
    for ( char **r = spec.text ; *r ; ++r ) {
      rscan->y1 = yoffset;
      rscan->y2 = yoffset + _font32->height() - 1;
      rscan->text = *r;

      yoffset += _font32->height() + 2;
      ++rscan;
    }
    Sint32 ymax = yoffset << 8;

    yoffset = 0;
    Sint32 last_time = SDL_GetTicks();

    eat_input();
    bool running = true;

    while ( running && !_quit_flag ) {
      Sint32 dt = SDL_GetTicks() - last_time;
      if ( dt > 0 ) {
	yoffset += dt * (spec.speed << 8) / 1000;
	if ( yoffset >= ymax )
	  yoffset = 0;

	copy(0, 0, get_image(_bg_id), rectangle(0, 0, 640, 480));

	Sint32 _yoffset = yoffset >> 8;

	rscan = rows;
	for ( Uint16 i = row_count ; i > 0 ; --i ) {
	  if ( (rscan->y2 - _yoffset >= 0) && (rscan->y1 - _yoffset <= 479) ) {
	    draw_text((640 - _font32->strwidth((Uint8 *)rscan->text)) / 2, rscan->y1 - _yoffset, false, _font32, rscan->text);
	  }

	  ++rscan;
	}

	update();

	last_time = SDL_GetTicks();
      }

      if ( dt < 1000 / 50 )
	SDL_Delay(1000 / 50 - dt / 4);

      SDL_Event event;
      while(SDL_PollEvent(&event)) {
	switch (event.type) {
	case SDL_KEYDOWN:
	  switch ( event.key.keysym.sym ) {

	  default :
	    running = false;
	    break;
	  }
	  break;

	case SDL_QUIT:
	  _quit_flag = true;
	  break;
	}
      }
    }

    delete rows;

    release_image(_bg_id);
  }
}
#endif

Sint16 game::menu_screen(menu_screen_spec &spec) {
  Sint16 selection = 0;

  Uint16 star_id = obtain_bob("avop");
  if ( star_id != 0xffff ) {
    surface *star = get_bob(star_id);

    Uint16 blip_id = obtain_sound("blip");
    if ( blip_id != 0xffff ) {
      Uint16 click_id = obtain_sound("click");
      if ( click_id != 0xffff ) {
	Uint16 bg_id = obtain_image(spec.back_pic);
	if ( bg_id != 0xffff ) {
	  struct entry {
	    Sint16 type;
	    Sint16 x, y;
	    char *text;
	    Uint16 *option;
	    char **options;
	    Uint16 fnt_id;
	    font *fnt;
	    void (game::*callback)(Sint16, Uint16);
	  };
	  Uint16 entry_count = 0;

	  Sint16 min_selection = 1000;
	  Sint16 max_selection = 0;

	  for ( menu_screen_entry *e = spec.entries ; e->text ; ++e ) {
	    if ( e->type != 0 ){
	      if ( e->type < min_selection )
		min_selection = e->type;
	      if ( e->type > max_selection )
		max_selection = e->type;
	    }

	    ++entry_count;
	  }

	  selection = min_selection;

	  entry *entries = new entry[entry_count];

	  entry *escan = entries;
	  for ( menu_screen_entry *e = spec.entries ; e->text ; ++e ) {
	    escan->type = e->type;
	    escan->text = e->text;
	    escan->fnt_id = obtain_font(e->fnt);
	    if ( escan->fnt_id != 0xffff )
	      escan->fnt = get_font(escan->fnt_id);
	    escan->x = e->xpos + ((escan->type > 0) ? 70 : 0);
	    escan->y = e->ypos - (star->height() - (escan->fnt ? escan->fnt->height() : 0)) / 2;
	    escan->option = &e->option;
	    escan->options = e->options;
	    escan->callback = e->callback;
	    ++escan;
	  }

	  Sint32 phase_start = SDL_GetTicks();
	  Uint16 phase = 0;
	  Sint32 star_start = SDL_GetTicks();
	  Sint32 time_start = SDL_GetTicks();

	  eat_input();
	  bool running = true;

	  while ( running && !_quit_flag ) {
	    Sint32 dt = SDL_GetTicks() - phase_start;

	    copy(0, 0, get_image(bg_id), rectangle(0, 0, 640, 480));
	    escan = entries;
	    for ( Uint16 i = entry_count ; i > 0 ; --i ) {
	      Sint16 xoffset = 0;
	      Sint16 yoffset = 0;

	      switch ( phase ) {
	      case 0 :
		xoffset = 0;
		yoffset = (480 + 2*escan->y) * (1000 - dt) / 1000;
		if ( yoffset < 0 )
		  yoffset = 0;
		break;

	      case 1 :
		xoffset = 0;
		yoffset = 0;
		break;

	      case 2 :
		xoffset = 0;
		yoffset = (480 + 2*escan->y) * dt / 1000;
		if ( yoffset < 0 )
		  yoffset = 0;
		break;
	      }


	      char buf[256];
	      strcpy(buf, escan->text);
	      if ( escan->options ) 
		strcat(buf, escan->options[*escan->option]);

	      Sint16 x1 = escan->x + xoffset;
	      Sint16 y1 = escan->y + yoffset;

	      if ( escan->fnt ) {
		Sint16 x2 = x1 + escan->fnt->strwidth((Uint8 *)buf) - 1;
		Sint16 y2 = y1 + star->height() - 1;

		draw_text(x1, y1 + (star->height() - escan->fnt->height()) / 2, false, escan->fnt, buf);
	      }

	      if ( escan->type == selection ) {
		draw_bob(x1 - 70, y1, (25 * (SDL_GetTicks() - star_start) / 1000) % star->frames(), star);
	      }

	      ++escan;
	    }

	    update();

	    switch ( phase ) {
	    case 0 :
	      if ( dt > 1000 ) {
		eat_input();
		phase = 1;
		phase_start = SDL_GetTicks();
	      }
	      break;

	    case 1 :
	      SDL_Event event;
	      while(SDL_PollEvent(&event)) {
		switch (event.type) {
		case SDL_KEYDOWN:
		  time_start = SDL_GetTicks();

		  switch ( event.key.keysym.sym ) {

		  case SDLK_UP :
		    play_sound(click_id, SOUND_LAYER_MISC);

		    if ( selection > min_selection ) {
		      selection--;
		    }
		    break;

		  case SDLK_DOWN :
		    play_sound(click_id, SOUND_LAYER_MISC);

		    if ( selection < max_selection ) {
		      selection++;
		    }
		    break;
		    
		  case SDLK_RETURN :
		  case SDLK_KP_ENTER :
		  case SDLK_SPACE : {
		    escan = entries;
		    for ( Uint16 i = entry_count ; i > 0 ; --i ) {
		      if ( escan->type == selection ) {
			play_sound(blip_id, SOUND_LAYER_MISC);

			if ( escan->options ) {
			  ++(*escan->option);
			  if ( !escan->options[*escan->option] )
			    *escan->option = 0;
			  if ( escan->callback )
			    (this->*(escan->callback))(escan->type, *escan->option);
			}
			else {
			  phase = 2;
			  phase_start = SDL_GetTicks();
			}
			break;
		      }

		      ++escan;
		    }
		    break;
		  }

		  default :
		    break;
		  }
		  break;

		case SDL_QUIT:
		  _quit_flag = true;
		  break;
		}
	      }

	      if ( SDL_GetTicks() - time_start > spec.timeout * 1000 ) {
		selection = -1;
		phase = 2;
		phase_start = SDL_GetTicks();
	      } 
	      break;

	    case 2 :
	      if ( dt > 1000 ) {
		running = false;
	      }
	      break;
	    }
	  }

	  escan = entries;
	  for ( Uint16 i = entry_count ; i > 0 ; --i ) {
	    if ( escan->fnt_id != 0xffff )
	      release_font(escan->fnt_id);

	    escan++;
	  }

	  delete entries;

	  release_image(bg_id);
	}

	release_sound(click_id);
      }

      release_sound(blip_id);
    }

    release_bob(star_id);
  }

  return( selection );
}

void game::go(int start_level) {
  if ( start_level != -1 )
    _skip_intro = true;

  if ( !_quit_flag ) {
    if ( initialize() ) {
      if ( !_skip_intro )
	intro();
      _skip_intro = false;

      if ( object_player::setup() ) {
	if ( start_level != -1 ) {
	  delete _player;
	  _player = new object_player();

	  enter_game(start_level);
	}

	while ( !_quit_flag ) {
	  if ( _title_song == 0xffff )
	    _title_song = obtain_song("mus/title.mod");
	  play_song(_title_song);

	  menu();
	}

	object_player::cleanup();
      }
      else
	show_message("Unable to setup player.\n");
    }
  }
}

void game::intro() {
    draw_rect(rectangle(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT), 0, 0, 0);

    Uint16 fnt_id = obtain_font("flow");
    if ( fnt_id != 0xffff ) {
      font *fnt = get_font(fnt_id);

      char *text = 
	"Thank you for testing this preview release of Gamma Patrol.\n"
	"\n"
	"You can download the latest version at\n"
	"  www.odahoda.de/avop/gammapatrol/\n"
	"\n"
	"This preview version is MAILWARE. If you like it and you are interested in the final version, "
	"please send me an e-mail or postcard. See Info/Contact for the address.\n"
	"Getting positive feedback is one of the biggest motivations for a coder and has great influence "
	"on the progress and quality of this product.\n";
      char *s = text;

      Sint16 x = 32;
      Sint16 y = 16;

      while ( *s ) {
	if ( *s == '\n' ) {
	  x = 32;
	  y += fnt->height() + 2;
	  SDL_Delay(1000/10);
	  ++s;
	}
	else {
	  Sint32 tx = x;
	  char *ts = s;

	  while ( *ts && (*ts != ' ') && (*ts != '\n') )
	    tx += fnt->width(*ts++);

	  if ( tx >= 640 - 32 ) {
	    x = 32;
	    y += fnt->height() + 2;
	    SDL_Delay(1000/10);
	  }

	  char t[2] = { *s, 0 };
	      
	  draw_text(x, y, true, fnt, t);
        
	  x += fnt->width(*s);
	  ++s;
	}
      }

      update();

      bool wait = true;
      while ( wait && !_quit_flag ) {
	SDL_Event event;
	while(SDL_PollEvent(&event)) {
	  switch (event.type) {
	  case SDL_KEYDOWN:
	    switch ( event.key.keysym.sym ) {
	    default :
	      wait = false;
	      break;
	    }
	    break;
	    
	  case SDL_QUIT:
	    _quit_flag = true;
	    break;
	  }
	}
      }

      release_font(fnt_id);
    }

  draw_rect(rectangle(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT), 0, 0, 0);
  update();
}

void game::menu() {
#define MENU_ID_START   1
#define MENU_ID_OPTIONS 2
#define MENU_ID_HELP    3
#define MENU_ID_INFO    4
#define MENU_ID_QUIT    5

  struct menu_screen_entry entries[] = {
    { MENU_ID_START, 50,  50, "Start Game", "menu", 0, NULL, NULL },
    { MENU_ID_OPTIONS, 50, 180, "Options", "menu", 0, NULL, NULL },
    { MENU_ID_HELP, 50, 230, "Help", "menu", 0, NULL, NULL },
    { MENU_ID_INFO, 50, 280, "Info", "menu", 0, NULL, NULL },
    { MENU_ID_QUIT, 50, 340, "Quit", "menu", 0, NULL, NULL },
    { 0, 0, 0, NULL },
  };

  struct menu_screen_spec spec;

  spec.entries = entries;
 spec.back_pic = "gfx/menu.png";
  spec.timeout = 10000;

  bool running = true;
  while ( running && !_quit_flag ) {
    switch ( menu_screen(spec) ) {
    case MENU_ID_START :
      start();
      running = false;
      break;


    case MENU_ID_OPTIONS :
      options();
      break;

    case MENU_ID_HELP :
      help();
      break;

    case MENU_ID_INFO :
      info();
      break;

    case MENU_ID_QUIT :
      _quit_flag = true;
      break;

    case -1 :
      running = false;
      break;

    default :
      show_message("Huh!\n");
      _quit_flag = true;
      break;
    }
  }
}

void game::start() {
  delete _player;
  _player = new object_player();

  enter_game(0);
}

bool game::enter_code() {
  return( true );
}

void game::enter_game(Uint16 level) {
  while ( (level < _levels.size()) && !_quit_flag ) {
    _level = _levels[level];
    if ( play() )
      level++;
    else
      break;
  }

  if ( level == _levels.size() )
    complete();
  else
    gameover();
}

char *game::level_code() {
  return("4OKHJIZRV2LN32AD");
}

void game::complete() {
  draw_rect(rectangle(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT), 0, 0, 0);

  Uint16 fnt_id = obtain_font("flow");
  if ( fnt_id != 0xffff ) {
    font *fnt = get_font(fnt_id);

    char *text = 
      "The preview release stops here...\n"
      "\n"
      "If you like the game, you might be interested in the complete version. "
      "Unfortunately it is still work in progress, so you need some more patience, "
      "before it is available.\n"
      "\n"
      "Visit www.odahoda.de/avop/gammapatrol/ for information about the development progress.\n"
      "\n"
      "\n"
      "Thank you,\n"
      "  P!\\K";

    char *s = text;

    Sint16 x = 32;
    Sint16 y = 16;

    while ( *s ) {
      if ( *s == '\n' ) {
	x = 32;
	y += fnt->height() + 2;
	SDL_Delay(1000/10);
	++s;
      }
      else {
	Sint32 tx = x;
	char *ts = s;

	while ( *ts && (*ts != ' ') && (*ts != '\n') )
	  tx += fnt->width(*ts++);

	if ( tx >= 640 - 32 ) {
	  x = 32;
	  y += fnt->height() + 2;
	  SDL_Delay(1000/10);
	}

	char t[2] = { *s, 0 };
	      
	draw_text(x, y, true, fnt, t);
        
	x += fnt->width(*s);
	++s;
      }
    }

    update();

    bool wait = true;
    while ( wait && !_quit_flag ) {
      SDL_Event event;
      while(SDL_PollEvent(&event)) {
	switch (event.type) {
	case SDL_KEYDOWN:
	  switch ( event.key.keysym.sym ) {
	  default :
	    wait = false;
	    break;
	  }
	  break;
	    
	case SDL_QUIT:
	  _quit_flag = true;
	  break;
	}
      }
    }

    release_font(fnt_id);
  }
}

void game::gameover() {
  if ( _gameover_song == 0xffff )
    _gameover_song = obtain_song("mus/gameover.mod");
  play_song(_gameover_song);

  eat_input();

  bool redraw = true;
  bool running = true;

  while ( running && !_quit_flag ) {
    if ( redraw ) {
      Uint16 _bg_id = obtain_image("gfx/gameover.png");
      if ( _bg_id != 0xffff ) {
	copy(0, 0, get_image(_bg_id), rectangle(0, 0, 640, 480));

	release_image(_bg_id);
      }
      else
	draw_rect(rectangle(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT), 0, 0, 0);

      Uint16 font_id = obtain_font("flow");
      if ( font_id != 0xffff ) {
	font *fnt = get_font(font_id);

	draw_text((640 - fnt->strwidth((Uint8 *)"GAME OVER")) / 2, (480 - fnt->height()) / 2, false, fnt, "GAME OVER");

	release_font(font_id);
      }

      update();
      redraw = false;
    }

    SDL_Delay(1000 / 25);
    SDL_Event event;
    while(SDL_PollEvent(&event)) {
      switch (event.type) {
      case SDL_KEYDOWN:
	switch ( event.key.keysym.sym ) {

	case SDLK_RETURN :
	case SDLK_KP_ENTER :
	case SDLK_SPACE :
	  running = false;
	}
        break;

      case SDL_QUIT:
        _quit_flag = true;
        break;
      }
    }
  }
}

void game::options_callback(Sint16 entry, Uint16 value) {
  switch ( entry ) {
  case 1 :
    switch ( value ) {
    case 0 :
      _input_device = INPUT_KEYBOARD;
      break;
    default :
      _input_device = INPUT_JOYSTICK;
      _joystick_number = value - 1;
      break;
    }
    break;

  case 2 :
    switch ( value ) {
    case 0 :
      surface_video::enable_fullscreen(false);
      enable_fullscreen(surface_video::fullscreen_enabled());
      break;

    case 1 :
      surface_video::enable_fullscreen(true);
      enable_fullscreen(surface_video::fullscreen_enabled());
      break;
    }
    break;
  }
}

void game::options() {
  char **control_options = NULL;
  Uint16 control_count = 0;
  control_count = 1;
  if ( joystick_enabled() )
    control_count += SDL_NumJoysticks();

  control_options = new char *[control_count + 1];
  char **co_scan = control_options;

  *co_scan = new char[9];
  strcpy(*co_scan, "Keyboard");
  co_scan++;

  if ( joystick_enabled() ) {
    for ( Uint16 i = 0 ; i < SDL_NumJoysticks() ; ++i ) {
      const char *n;
      char buf[256];

      n = SDL_JoystickName(i);
      if ( !n )
	n = "Unknown Joystick";

      sprintf(buf, "Joystick %u (%s)", i + 1, n);

      *co_scan = new char[strlen(buf) + 1];
      strcpy(*co_scan, buf);
      co_scan++;
    }
  }

  *co_scan = NULL;

  char *display_options[] = {
    "Window",
    "Fullscreen",
    NULL
  };

  char *sound_options[] = {
    "On",
    "Off",
    NULL
  };

  char *music_options[] = {
    "On",
    "Off",
    NULL
  };

  char *audio_options[] = {
    "Low",
    "Medium",
    "High",
    NULL
  };

  struct menu_screen_entry entries[] = {
    { 0, 50, 10, "Change options", "menu", 0, NULL, NULL },
    { 1, 50, 60, "Control : ", "flow", (_input_device == INPUT_KEYBOARD) ? 0 : ((_joystick_number < SDL_NumJoysticks()) ? _joystick_number : 0), control_options, &game::options_callback },
    { 2, 50, 100, "Display : ", "flow", fullscreen_enabled() ? 1 : 0, display_options, &game::options_callback },
    { 3, 50, 440, "Back to menu", "flow", 0, NULL, NULL },
    { 0, 0, 0, NULL, NULL },
  };

  struct menu_screen_spec spec;

  spec.entries = entries;
  spec.back_pic = "gfx/options.png";
  spec.timeout = 10000;

  menu_screen(spec);

  for ( char **s = control_options ; *s ; ++s )
    delete *s;
  delete control_options;
}

void game::help() {
  struct menu_screen_entry info_entries[] = {
    { 1, 50, 50,  "Intro", "menu", 0, NULL, NULL },
    { 2, 50, 100, "Keyboard", "menu", 0, NULL, NULL },
    { 3, 50, 440, "Back to menu", "menu", 0, NULL, NULL },
    { 0, 0, 0, NULL, NULL },
  };

  struct menu_screen_spec info_spec;

  info_spec.entries = info_entries;
  info_spec.back_pic = "gfx/help.png";
  info_spec.timeout = 10000;

  bool running = true;
  while ( running && !_quit_flag ) {
    switch ( menu_screen(info_spec) ) {
    case 1 : { /* Intro */
      struct menu_screen_entry entries[] = {
	{ 0, 50, 50,  "Not written yet...", "flow", 0, NULL, NULL },
	{ 1, 50, 440, "Back", "menu", 0, NULL, NULL },
	{ 0, 0, 0, NULL, NULL },
      };

      struct menu_screen_spec spec;

      spec.entries = entries;
      spec.back_pic = "gfx/help.png";
      spec.timeout = 10000;

      menu_screen(spec);
      break;
    }

    case 2 : { /* keyboard */
      struct menu_screen_entry entries[] = {
	{ 0, 60, 70,  "Cursor keys", "flow", 0, NULL, NULL },
	{ 0, 250, 70,  "- Navigate ship", "flow", 0, NULL, NULL },
	{ 0, 60, 88,  "Space", "flow", 0, NULL, NULL },
	{ 0, 250, 88,  "- Fire", "flow", 0, NULL, NULL },
	{ 0, 60, 106,  "NumPad 1-3", "flow", 0, NULL, NULL },
	{ 0, 60, 124,  "PgUp/PgDown", "flow", 0, NULL, NULL },
	{ 0, 250, 114,  "- Select weapon", "flow", 0, NULL, NULL },
	{ 0, 60, 142, "Esc", "flow", 0, NULL, NULL },
	{ 0, 250, 142, "- Quit", "flow", 0, NULL, NULL },
	{ 0, 60, 160, "P", "flow", 0, NULL, NULL },
	{ 0, 250, 160, "- Pause", "flow", 0, NULL, NULL },
	{ 0, 60, 178, "S", "flow", 0, NULL, NULL },
	{ 0, 250, 178, "- Turn sound FX on/off", "flow", 0, NULL, NULL },
	{ 0, 60, 196, "M", "flow", 0, NULL, NULL },
	{ 0, 250, 196, "- Turn music on/off", "flow", 0, NULL, NULL },
	{ 0, 60, 214, "F", "flow", 0, NULL, NULL },
	{ 0, 250, 214, "- Toggle fps display", "flow", 0, NULL, NULL },
	{ 1, 50, 440, "Back to menu", "menu", 0, NULL, NULL },
	{ 0, 0, 0, NULL, NULL },
      };

      struct menu_screen_spec spec;

      spec.entries = entries;
      spec.back_pic = "gfx/help.png";
      spec.timeout = 10000;

      menu_screen(spec);
      break;
    }

    case 3 :
      running = false;
      break;

    default :
      show_message("Huh!\n");
      _quit_flag = true;
      break;
    }
  }
}

void game::info() {
  struct menu_screen_entry info_entries[] = {
    { 1, 50, 50,  "Author", "menu", 0, NULL, NULL },
    { 2, 50, 100, "Shareware info", "menu", 0, NULL, NULL },
    { 3, 50, 150, "Contact", "menu", 0, NULL, NULL },
    { 4, 50, 440, "Back to menu", "menu", 0, NULL, NULL },
    { 0, 0, 0, NULL, NULL },
  };

  struct menu_screen_spec info_spec;

  info_spec.entries = info_entries;
  info_spec.back_pic = "gfx/info.png";
  info_spec.timeout = 10000;

  bool running = true;
  while ( running && !_quit_flag ) {
    switch ( menu_screen(info_spec) ) {
    case 1 : {
      struct menu_screen_entry entries[] = {
	{ 0, 320, 50,  "Design, Coding &", "flow", 0, NULL, NULL },
	{ 0, 320, 70,  "Graphics by", "flow", 0, NULL, NULL },
	{ 0, 320, 90,  "Pink", "flow", 0, NULL, NULL },
	{ 1, 320, 440, "Back", "menu", 0, NULL, NULL },
	{ 0, 0, 0, NULL, NULL },
      };

      struct menu_screen_spec spec;

      spec.entries = entries;
      spec.back_pic = "gfx/info-author.png";
      spec.timeout = 10000;

      menu_screen(spec);
      break;
    }

    case 2 : {
      struct menu_screen_entry entries[] = {
	{ 0, 50, 50,  "The final version is supposed to be", "flow", 0, NULL, NULL },
	{ 0, 50, 70,  "shareware.", "flow", 0, NULL, NULL },
	{ 0, 50, 95,  "Until mentioned differently this", "flow", 0, NULL, NULL },
	{ 0, 50, 115,  "preview version is MAILWARE.", "flow", 0, NULL, NULL },
	{ 0, 50, 135,  "Please send me an e-mail or postcard,", "flow", 0, NULL, NULL },
	{ 0, 50, 155,  "if you like the game.", "flow", 0, NULL, NULL },
	{ 1, 50, 440, "Back", "menu", 0, NULL, NULL },
	{ 0, 0, 0, NULL, NULL },
      };
      struct menu_screen_spec spec;

      spec.entries = entries;
      spec.back_pic = "gfx/info.png";
      spec.timeout = 10000;

      menu_screen(spec);
      break;
    }

    case 3 : {
      struct menu_screen_entry entries[] = {
	{ 0, 50, 50,  "Benjamin Niemann (Pink)", "flow", 0, NULL, NULL },
	{ 0, 50, 70,  "Scheffelstr. 14", "flow", 0, NULL, NULL },
	{ 0, 50, 90,  "69214 Eppelheim", "flow", 0, NULL, NULL },
	{ 0, 50, 110,  "Germany", "flow", 0, NULL, NULL },
	{ 0, 50, 135,  "pink@odahoda.de", "flow", 0, NULL, NULL }, 
	{ 0, 50, 155,  "http://www.odahoda.de/avop", "flow", 0, NULL, NULL }, 
	{ 0, 50, 190,  "Comments, suggestion and critics", "flow", 0, NULL, NULL }, 
	{ 0, 50, 210,  "are welcome!", "flow", 0, NULL, NULL }, 
	{ 1, 50, 440, "Back", "menu", 0, NULL, NULL },
	{ 0, 0, 0, NULL, NULL },
      };

      struct menu_screen_spec spec;

      spec.entries = entries;
      spec.back_pic = "gfx/info.png";
      spec.timeout = 10000;

      menu_screen(spec);
      break;
    }

    case 4 :
      running = false;
      break;

    default :
      show_message("Huh!\n");
      _quit_flag = true;
      break;
    }
  }
}

Uint16 game::obtain_bob(const char *path) {
  Uint16 id = 0;
  for ( loaded_bob *iend = _loaded_bobs + MAX_LOADED_BOBS, *i = _loaded_bobs ; i < iend ; ++i, ++id ) {
    if ( (i->path == NULL) || !strcmp(i->path, path) ) {
      if ( !i->path ) {
	flush_bobs();
	
	char full_path[256];
	sprintf(full_path, "bob/%s.img", path);
	surface *b = surface::load_image(full_path);
	if ( b ) {
	  i->s = b;
	  i->cnt = 1;
	  i->path = new char[strlen(path) + 1];
	  strcpy(i->path, path);

	  _bobs_memory += b->memory();

	  return( id );
	}
	else {
	  show_message("Couldn't load bob '%s'.\n", path);
	  return( 0xffff );
	}
      }
      else {
	i->cnt++;

	return(id);
      }
    }
  }

  show_message("Fatal error: out of free bob slots.\n");
  exit(0);
}

void game::release_bob(Uint16 id) {
  if ( id < MAX_LOADED_BOBS ) {
    loaded_bob *i = &_loaded_bobs[id];

    if ( i->s && (i->cnt > 0) ) {
      i->cnt--;
      if ( i->cnt == 0 ) {
	i->time = SDL_GetTicks();
      }
    }
  }
}

void game::flush_bobs() {
  while ( _bobs_memory > MAX_BOBS_MEMORY ) {
    loaded_bob *f = NULL;

    for ( loaded_bob *iend = _loaded_bobs + MAX_LOADED_BOBS, *i = _loaded_bobs ; i < iend ; ++i ) {
      if ( i->path && (i->cnt == 0) ) {
	if ( !f || (i->time < f->time) )
	  f = i;
      }
    }

    if ( f ) {
      _bobs_memory -= f->s->memory();

      delete f->path;
      f->path = NULL;
      delete f->s;
      f->s = NULL;
    }
    else
      break;
  }
}

Uint16 game::obtain_image(const char *path) {
  Uint16 id = 0;

  for ( loaded_image *iend = _loaded_images + MAX_LOADED_IMAGES, *i = _loaded_images ; i < iend ; ++i, ++id ) {
    if ( (i->path == NULL) || !strcmp(i->path, path) ) {
      if ( !i->path ) {
	flush_images();

	surface *s = load_image(path);
	if ( s ) {
	  s->display_format();

	  i->s = s;
	  i->cnt = 1;
	  i->path = new char[strlen(path) + 1];
	  strcpy(i->path, path);

	  _images_memory += s->memory();
	  return(id);
	}
	else {
	  show_message("Couldn't load image '%s'.\n", path);
	  return( 0xffff );
	}
      }
      else {
	i->cnt++;
	return(id);
      }
    }
  }

  show_message("Fatal error: out of free image slots.\n");
  exit(0);
}

void game::release_image(Uint16 id) {
  if ( id < MAX_LOADED_IMAGES ) {
    loaded_image *i = &_loaded_images[id];

    if ( i->s && (i->cnt > 0) ) {
      i->cnt--;
      if ( i->cnt == 0 ) {
	i->time = SDL_GetTicks();
      }
    }
  }
}

void game::flush_images() {
  while ( _images_memory > MAX_IMAGES_MEMORY ) {
    loaded_image *f = NULL;

    for ( loaded_image *iend = _loaded_images + MAX_LOADED_IMAGES, *i = _loaded_images ; i < iend ; ++i ) {
      if ( i->path && (i->cnt == 0) ) {
	if ( !f || (i->time < f->time) )
	  f = i;
      }
    }

    if ( f ) {
      _images_memory -= f->s->memory();

      delete f->path;
      f->path = NULL;
      delete f->s;
      f->s = NULL;
    }
    else
      break;
  }
}

void game::play_song(Uint16 id) {
  if ( id != _song_current ) {
    if ( _enable_audio ) {
      if( Mix_PlayingMusic() ) {
	Mix_FadeOutMusic(1500);
	SDL_Delay(1500);
	Mix_HaltMusic();
      }
    
      if ( _enable_music ) 
	Mix_PlayMusic(get_song(id), -1);
    }

    _song_current = id;
  }
}

void game::stop_song() {
  if ( _enable_audio ) {
    if( Mix_PlayingMusic() ) {
      Mix_FadeOutMusic(1500);
      SDL_Delay(1500);
      Mix_HaltMusic();
      _song_current = 0xffff;
    }
  }
}

Uint16 game::obtain_song(const char *path) {
  Uint16 id = 0;

  for ( loaded_song *iend = _loaded_songs + MAX_LOADED_SONGS, *i = _loaded_songs ; i < iend ; ++i, ++id ) {
    if ( (i->path == NULL) || !strcmp(i->path, path) ) {
      if ( !i->path ) {
	flush_songs();

	if ( _enable_audio ) {
	  Mix_Music *s = Mix_LoadMUS(path);
	  if ( s ) {
	    i->s = s;
	    i->cnt = 1;
	    i->path = new char[strlen(path) + 1];
	    strcpy(i->path, path);

	    //_songs_memory += s->memory();
	    return(id);
	  }
	  else {
	    show_message("Couldn't load song '%s'.\n", path);
	    return( 0xffff );
	  }
	}
	else {
	  i->s = NULL;
	  i->cnt = 1;
	  i->path = new char[strlen(path) + 1];
	  strcpy(i->path, path);

	  //_songs_memory += s->memory();

	  return(id);
	}
      }
      else {
	i->cnt++;
	return(id);
      }
    }
  }

  show_message("Fatal error: out of free song slots.\n");
  exit(0);
}

void game::release_song(Uint16 id) {
  if ( id < MAX_LOADED_SONGS ) {
    loaded_song *i = &_loaded_songs[id];

    if ( i->s && (i->cnt > 0) ) {
      i->cnt--;
      if ( i->cnt == 0 ) {
	i->time = SDL_GetTicks();
      }
    }
  }
}

void game::flush_songs() {
  while ( _songs_memory > MAX_SONGS_MEMORY ) {
    loaded_song *f = NULL;

    for ( loaded_song *iend = _loaded_songs + MAX_LOADED_SONGS, *i = _loaded_songs ; i < iend ; ++i ) {
      if ( i->path && (i->cnt == 0) ) {
	if ( !f || (i->time < f->time) )
	  f = i;
      }
    }

    if ( f ) {
      //_songs_memory -= f->s->memory();

      delete f->path;
      f->path = NULL;
      if ( f->s )
	Mix_FreeMusic(f->s);
      f->s = NULL;
    }
    else
      break;
  }
}

void game::play_sound(Uint16 id, Uint16 layer, Uint16 vol = 128) {
  if ( _enable_audio && _enable_sound ) {
    int channel = Mix_GroupAvailable(layer);
    if ( channel == -1 )
      channel = Mix_GroupOldest(layer);

    if ( channel != -1 ) {
      Mix_Volume(channel, vol);
      Mix_PlayChannel(channel, get_sound(id), 0);
    }
  }
}

void game::play_expl_sound(Uint16 id, Uint16 layer, Uint16 vol = 128) {
  play_sound(_expl_id[id], layer, vol);
}

void game::stop_sounds() {
  if( _enable_audio && Mix_Playing(-1) ) {
    Mix_HaltGroup(-1);
  }
}

Uint16 game::obtain_sound(const char *path) {
  Uint16 id = 0;

  for ( loaded_sound *iend = _loaded_sounds + MAX_LOADED_SOUNDS, *i = _loaded_sounds ; i < iend ; ++i, ++id ) {
    if ( (i->path == NULL) || !strcmp(i->path, path) ) {
      if ( !i->path ) {
	flush_sounds();

	if ( _enable_audio ) {
	  char buf[256];
	  sprintf(buf, "sfx/%s.wav", path);
	  Mix_Chunk *s = Mix_LoadWAV(buf);
	  if ( s ) {
	    i->s = s;
	    i->cnt = 1;
	    i->path = new char[strlen(path) + 1];
	    strcpy(i->path, path);
	    
	    //_sounds_memory += s->memory();

	    return(id);
	  }
	  else {
	    show_message("Couldn't load sound '%s'.\n", path);
	    return( 0xffff );
	  }
	}
	else {
	  i->s = NULL;
	  i->cnt = 1;
	  i->path = new char[strlen(path) + 1];
	  strcpy(i->path, path);
	    
	  //_sounds_memory += s->memory();

	  return(id);
	}
      }
      else {
	i->cnt++;
	return(id);
      }
    }
  }

  show_message("Fatal error: out of free sound slots.\n");
  exit(0);
}

void game::release_sound(Uint16 id) {
  if ( id < MAX_LOADED_SOUNDS ) {
    loaded_sound *i = &_loaded_sounds[id];

    if ( i->s && (i->cnt > 0) ) {
      i->cnt--;
      if ( i->cnt == 0 ) {
	i->time = SDL_GetTicks();
      }
    }
  }
}

void game::flush_sounds() {
  while ( _sounds_memory > MAX_SOUNDS_MEMORY ) {
    loaded_sound *f = NULL;

    for ( loaded_sound *iend = _loaded_sounds + MAX_LOADED_SOUNDS, *i = _loaded_sounds ; i < iend ; ++i ) {
      if ( i->path && (i->cnt == 0) ) {
	if ( !f || (i->time < f->time) )
	  f = i;
      }
    }

    if ( f ) {
      //_sounds_memory -= f->s->memory();

      delete f->path;
      f->path = NULL;
      if ( f->s )
	Mix_FreeChunk(f->s);
      f->s = NULL;
    }
    else
      break;
  }
}

Uint16 game::obtain_font(const char *path) {
  Uint16 id = 0;

  for ( loaded_font *iend = _loaded_fonts + MAX_LOADED_FONTS, *i = _loaded_fonts ; i < iend ; ++i, ++id ) {
    if ( (i->path == NULL) || !strcmp(i->path, path) ) {
      if ( !i->path ) {
	flush_fonts();

	font *s = new font();
	if ( s->load(path) ) {
	  i->s = s;
	  i->cnt = 1;
	  i->path = new char[strlen(path) + 1];
	  strcpy(i->path, path);
	    
	  //_fonts_memory += s->memory();
	  
	  return(id);
	}
	else {
	  delete s;
	  show_message("Couldn't load font '%s'.\n", path);
	  return( 0xffff );
	}
      }
      else {
	i->cnt++;
	return(id);
      }
    }
  }

  show_message("Fatal error: out of free font slots.\n");
  exit(0);
}

void game::release_font(Uint16 id) {
  if ( id < MAX_LOADED_FONTS ) {
    loaded_font *i = &_loaded_fonts[id];

    if ( i->s && (i->cnt > 0) ) {
      i->cnt--;
      if ( i->cnt == 0 ) {
	i->time = SDL_GetTicks();
      }
    }
  }
}

void game::flush_fonts() {
  while ( _fonts_memory > MAX_FONTS_MEMORY ) {
    loaded_font *f = NULL;

    for ( loaded_font *iend = _loaded_fonts + MAX_LOADED_FONTS, *i = _loaded_fonts ; i < iend ; ++i ) {
      if ( i->path && (i->cnt == 0) ) {
	if ( !f || (i->time < f->time) )
	  f = i;
      }
    }

    if ( f ) {
      //_fonts_memory -= f->s->memory();

      delete f->path;
      f->path = NULL;
      if ( f->s )
	delete f->s;
      f->s = NULL;
    }
    else
      break;
  }
}

