
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: game2.cc,v 1.1 2000/10/12 14:35:06 pink Exp pink $ */

#include <stdlib.h>
#include <memory.h>
#include <ctype.h>
#include "main.h"
#include "game.h"
#include "group.h"
#include "group_table.h"
#include "object.h"
#include "object_player.h"
#include "object_bonus.h"
#include "font.h"
#include "surface.h"

bool game::play() {
  bool success = false;
  bool ok = true;

  bool _level_cleanup = false;
  if ( _level->setup() ) {
    _level_cleanup = true;
    play_song(_level->song());
  }
  else {
    show_message("Unable to setup level.\n");
    ok = false;
  }

  _tiny_font_id = 0xffff;
  if ( ok ) {
    _tiny_font_id = obtain_font("tiny");
    if ( _tiny_font_id != 0xffff ) {
    }
    else {
      show_message("Unable to load 'tiny'.\n");
      ok = false;
    }
  }

  _panel_font_id = 0xffff;
  if ( ok ) {
    _panel_font_id = obtain_font("panel");
    if ( _panel_font_id != 0xffff ) {
    }
    else {
      show_message("Unable to load 'panel'.\n");
      ok = false;
    }
  }

  _flow_font_id = 0xffff;
  if ( ok ) {
    _flow_font_id = obtain_font("flow");
    if ( _flow_font_id != 0xffff ) {
    }
    else {
      show_message("Unable to load 'flow'.\n");
      ok = false;
    }
  }

  for ( int i = 0 ; i < SOUND_EXPL_COUNT ; ++i )
    _expl_id[i] = 0xffff;

  if ( ok ) {
    char *s[SOUND_EXPL_COUNT] = {
      "expl1", "expl2", "expl3", "expl4"
    };

    for ( int i = 0 ; ok && (i < SOUND_EXPL_COUNT) ; ++i ) {
      _expl_id[i] = obtain_sound(s[i]);
      if ( _expl_id[i] == 0xffff ) {
	ok = false;
      }
    }
  }

  _bigenemy_surface = NULL;
  if ( ok ) {
    _bigenemy_surface = surface::load_image("gfx/benemy-energy.img");
    if ( _bigenemy_surface )
      _bigenemy_surface->display_format();
    else {
      show_message("Unable to load 'gfx/benemy-energy.img'.\n");
      ok = false;
    }
  }

  _panel_id = 0xffff;
  if ( ok ) {
    _panel_id = obtain_image("gfx/panel.img");
    if ( _panel_id == 0xffff ) {
      show_message("Unable to load 'gfx/panel.img'.\n");
      ok = false;
    }
  }

  _slot_upper_id[0] = 0xffff;
  if ( ok ) {
    _slot_upper_id[0] = obtain_image("gfx/slot1a.img");
    if ( _slot_upper_id[0] != 0xffff ) {
    }
    else {
      show_message("Unable to load 'gfx/slot1a.img'.\n");
      ok = false;
    }
  }

  _slot_lower_id[0] = 0xffff;
  if ( ok ) {
    _slot_lower_id[0] = obtain_image("gfx/slot1b.img");
    if ( _slot_lower_id[0] != 0xffff ) {
    }
    else {
      show_message("Unable to load 'gfx/slot1b.img'.\n");
      ok = false;
    }
  }

  _slot_upper_id[1] = 0xffff;
  if ( ok ) {
    _slot_upper_id[1] = obtain_image("gfx/slot2a.img");
    if ( _slot_upper_id[1] != 0xffff ) {
    }
    else {
      show_message("Unable to load 'gfx/slot2a.img'.\n");
      ok = false;
    }
  }

  _slot_lower_id[1] = 0xffff;
  if ( ok ) {
    _slot_lower_id[1] = obtain_image("gfx/slot2b.img");
    if ( _slot_lower_id[1] != 0xffff ) {
    }
    else {
      show_message("Unable to load 'gfx/slot2b.img'.\n");
      ok = false;
    }
  }

  _slot_upper_id[2] = 0xffff;
  if ( ok ) {
    _slot_upper_id[2] = obtain_image("gfx/slot3a.img");
    if ( _slot_upper_id[2] != 0xffff ) {
    }
    else {
      show_message("Unable to load 'gfx/slot3a.img'.\n");
      ok = false;
    }
  }

  _slot_lower_id[2] = 0xffff;
  if ( ok ) {
    _slot_lower_id[2] = obtain_image("gfx/slot3b.img");
    if ( _slot_lower_id[2] != 0xffff ) {
    }
    else {
      show_message("Unable to load 'gfx/slot3b.img'.\n");
      ok = false;
    }
  }

  _slot_upper_id[3] = 0xffff;
  if ( ok ) {
    _slot_upper_id[3] = obtain_image("gfx/slot4a.img");
    if ( _slot_upper_id[3] != 0xffff ) {
    }
    else {
      show_message("Unable to load 'gfx/slot4a.img'.\n");
      ok = false;
    }
  }

  _slot_lower_id[3] = 0xffff;
  if ( ok ) {
    _slot_lower_id[3] = obtain_image("gfx/slot4b.img");
    if ( _slot_lower_id[3] != 0xffff ) {
    }
    else {
      show_message("Unable to load 'gfx/slot4b.img'.\n");
      ok = false;
    }
  }

  if ( ok ) {
    if ( _input_device == INPUT_JOYSTICK ) {
      _joystick_handle = SDL_JoystickOpen(_joystick_number);
      if ( !_joystick_handle ) {
	show_message("Unable to open joystick device.\n");
	ok = false;
      }
    }
  }

  if ( ok ) {
    draw_rect(rectangle(0, 0, 640, 480), 0, 0, 0);
    copy(PANEL_XPOS, PANEL_YPOS, get_image(_panel_id), rectangle(0, 0, PANEL_WIDTH, PANEL_HEIGHT));
    update();

    eat_input();

    _current_time = 0;
    Uint32 _offset_time = SDL_GetTicks();
    Uint32 _last_frame_time = 0;
    Uint32 _step_delay = 0;
    _paused = false;

    for ( Uint16 i = 0 ; i < MESSAGE_COUNT ; ++i ) {
      delete _messages[i].text;
      _messages[i].text = NULL;
    }

    _xpos = ((_level->width() - MAIN_WIDTH) / 2) << 8;
    _ypos = (_level->height() - MAIN_HEIGHT) << 8;
    _yvel = _level->yvel();
    _signal_mask = 0;

    _player->add_energy(PLAYER_MAX_ENERGY);
    _player->set_invul();
    _player->xpos(xpos() + MAIN_WIDTH / 2);
    _player->xvel(0);
    _player->xacc(0);
    _player->ypos(ypos() + MAIN_HEIGHT - 40);
    _player->yvel(0);
    _player->yacc(0);
    _player->flags(_player->flags() & ~(OBJECT_INVISIBLE | OBJECT_DELETED));
    _player->set_gun(PLAYER_GUN_PLASMA);
    _player->set_gun(PLAYER_GUN_BALL);
    _player->set_gun(PLAYER_GUN_SEEKER);
    _player->select_gun(PLAYER_GUN_LASER);
    add_object(_player);

#ifdef OLD
    _stars_last_time = current_time();
#endif

    _frames = 0;
    _fps_start_ticks = SDL_GetTicks();
    _fps = 0;
    _enable_fps = false;

    _old_energy = 0xffffffff;
    _old_score = 0xffffffff;
    _old_lifes = 0xffffffff;

    _slot_status[0] = 0;
    _slot_status[1] = 0;
    _slot_status[2] = 0;
    _slot_status[3] = 0;

    _bigenemy_state = 0;

    Sint32 _bg_last_time = current_time();

    add_message("Get ready!", 150);

    bool finished = false;
    Uint16 finished_phase = 0;

    bool running = true;
    while( running && !_quit_flag ) {
      _frames++;
	  
      SDL_Event event;
      while(SDL_PollEvent(&event)) {
	switch (event.type) {
	case SDL_MOUSEBUTTONDOWN:
	case SDL_MOUSEBUTTONUP:
	  break;
	      
	case SDL_MOUSEMOTION:
	  break;
	    
	case SDL_KEYDOWN: 
	  if ( event.key.keysym.mod & KMOD_SHIFT ) {
	    switch ( event.key.keysym.sym ) {
	    default :
	      break;
	    }
	  }
	  else {
	    switch ( event.key.keysym.sym ) {
	    case SDLK_ESCAPE :
	      running = false;
	      break;
	      
	    case SDLK_m :
	      _enable_music = !_enable_music;
	      if ( _enable_audio ) {
		if ( _enable_music ) {
		  if ( _song_current != 0xffff )
		    Mix_PlayMusic(get_song(_song_current), -1);
		}
		else
		  Mix_HaltMusic();
	      }
	      break;

	    case SDLK_s :
	      _enable_sound = !_enable_sound;
	      if ( _enable_audio ) {
		if ( !_enable_sound ) 
		  Mix_HaltGroup(-1);
	      }
	      break;

	    case SDLK_p :
	      _paused = !_paused;	   
	      break;

	    case SDLK_PAGEUP : {
	      static Uint16 guns[] = { PLAYER_GUN_LASER, PLAYER_GUN_PLASMA, PLAYER_GUN_BALL };
	      Sint16 g;
	      switch ( _player->gun_type() ) {
	      default :
	      case PLAYER_GUN_LASER : g = 2; break;
	      case PLAYER_GUN_PLASMA : g = 0; break;
	      case PLAYER_GUN_BALL : g = 1; break;
	      }

	      while ( _player->gun_power(g) == 0 ) {
		if ( g == 0 )
		  g = 2;
		else
		  --g;
	      }

	      _player->select_gun(g);
	      break;
	    }

	    case SDLK_PAGEDOWN : {
	      static Uint16 guns[] = { PLAYER_GUN_LASER, PLAYER_GUN_PLASMA, PLAYER_GUN_BALL };
	      Sint16 g;
	      switch ( _player->gun_type() ) {
	      default :
	      case PLAYER_GUN_LASER : g = 1; break;
	      case PLAYER_GUN_PLASMA : g = 2; break;
	      case PLAYER_GUN_BALL : g = 0; break;
	      }

	      while ( _player->gun_power(g) == 0 ) {
		if ( g == 2 )
		  g = 0;
		else
		  ++g;
	      }

	      _player->select_gun(g);
	      break;
	    }

	    case SDLK_KP1 :
	      if ( _player->gun_power(PLAYER_GUN_LASER) > 0 )
		_player->select_gun(PLAYER_GUN_LASER);
	      break;

	    case SDLK_KP2 :
	      if ( _player->gun_power(PLAYER_GUN_PLASMA) > 0 )
		_player->select_gun(PLAYER_GUN_PLASMA);
	      break;

	    case SDLK_KP3 :
	      if ( _player->gun_power(PLAYER_GUN_BALL) > 0 )
		_player->select_gun(PLAYER_GUN_BALL);
	      break;

	    case SDLK_KP4 :
	      break;

	    case SDLK_KP5 :
	      break;

	    case SDLK_KP6 :
	      break;

	    case SDLK_f :
	      _enable_fps = !_enable_fps;
	      break;


	    case SDLK_SPACE :
	      if ( _input_device == INPUT_KEYBOARD )
		_player->set_fire_flag();
	      break;

	    default :
	      break;
	    }
	  }
	  break;
	    
	case SDL_JOYBUTTONDOWN:
	  if ( _input_device == INPUT_JOYSTICK ) {
	    switch ( event.jbutton.button ) {
	    case 0 :
	      _player->set_fire_flag();
	      break;
	    }
	  }
	  break;

	case SDL_QUIT:
	  _quit_flag = true;
	  break;
	    
	default:
	  break;
	}
      }

      Uint32 _now_time = SDL_GetTicks() - _offset_time;

      if ( _paused )
	_step_delay += _now_time - _last_frame_time;

      _current_time = _now_time - _step_delay;

      _last_frame_time = _now_time;

#ifdef OLD
      while ( (!_wait_mask || (_wait_mask & _signal_mask)) && _current_time >= _next_event ) {
	if ( _signal_mask & _wait_mask ) {
	  Uint32 sigs_got = _signal_mask & _wait_mask;
	  _wait_mask &= ~sigs_got;
	  _signal_mask &= ~sigs_got;
	}
	
	_next_event = _level->next_event();
	
	if ( _next_event == 0xffffffff ) {
	  finished = true;
	}
      }
#endif

      if ( _player->visible() && !finished )
	_yvel = _level->yvel() << 8;
      else
	_yvel = 0;

      _level->render_background();
      _level->check_triggers();
      next_frame();
      update_panel();

      update_flip();

      if ( _yvel > 0 ) {
	Sint32 dy = _yvel * (current_time() - _bg_last_time) / 1000;

	if ( _ypos > dy )
	  _ypos -= dy;
	else {
	  _ypos = 0;
	  finished = true;
	}
      }

      if ( _player->xpos() < xpos() + 40 ) {
	_xpos = (_player->xpos() - 40) << 8;
	if ( _xpos < 0 )
	  _xpos = 0;
      }
      else if ( _player->xpos() > xpos() + MAIN_WIDTH - 40 ) {
	_xpos = (_player->xpos() - MAIN_WIDTH + 40) << 8;
	if ( _xpos > (_level->width() - MAIN_WIDTH) << 8 )
	  _xpos =  (_level->width() - MAIN_WIDTH) << 8;
      }

      _bg_last_time = current_time();

      if ( finished ) {
	switch ( finished_phase ) {
	case 0 :
	  _player->disable_steering();
	  _player->xacc(0);
	  _player->xvel(0);
	  _player->yvel(-50);
	  _player->yacc(-250);

	  ++finished_phase;
	  break;

	case 1 :
	  if ( _player->ypos() + _player->bbound() < 0 )
	    ++finished_phase;
	  break;

	default :
	  running = false;
	  break;
	}
      }

      if ( _player->flags() & PLAYER_KILLED_DONE ) {
	_player->flags(_player->flags() & ~(PLAYER_KILLED_DONE | PLAYER_KILLED) );

	if ( _player->lifes() > 0 ) {
	  _player->sub_life();
	  _player->startup();
	  add_message("Get ready!", 150);
	}
	else
	  running = false;
      }
    }

    if ( finished )
      success = true;

    for ( int i = 255 ; i >= 0 ; i -= 16 ) {
      draw_rect(rectangle(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT), i, i, i);
      update();
    }

    stop_song();
    stop_sounds();
  }

  if ( _joystick_handle ) {
    SDL_JoystickClose(_joystick_handle);
    _joystick_handle = NULL;
  }

  for ( sarray<group>::iterator i = _groups.begin() ; i != _groups.end() ; ++i ) {
    delete *i;
    *i = NULL;
  }

  for ( sarray<object>::iterator i = _objects.begin() ; i != _objects.end() ; ++i ) {
    *i = NULL;
  }

  for ( sarray<object>::iterator i = _added_objects.begin() ; i != _added_objects.end() ; ++i ) {
    if ( *i != _player )
      delete *i;

    *i = NULL;
  }

  if ( _tiny_font_id != 0xffff ) {
    release_font(_tiny_font_id);
    _tiny_font_id = 0xffff;
  }

  if ( _panel_font_id != 0xffff ) {
    release_font(_panel_font_id);
    _panel_font_id = 0xffff;
  }

  if ( _flow_font_id != 0xffff ) {
    release_font(_flow_font_id);
    _flow_font_id = 0xffff;
  }

  for ( int i = 0 ; i < 4 ; ++i ) {
    if ( _slot_upper_id[i] != 0xffff ) {
      release_image(_slot_upper_id[i]);
      _slot_upper_id[i] = 0xffff;
    }
    if ( _slot_lower_id[i] != 0xffff ) {
      release_image(_slot_lower_id[i]);
      _slot_lower_id[i] = 0xffff;
    }
  }

  if ( _panel_id != 0xffff ) {
    release_image(_panel_id);
    _panel_id = 0xffff;
  }

  if ( _bigenemy_surface ) {
    delete _bigenemy_surface;
    _bigenemy_surface = NULL;
  }

  for ( int i = 0 ; i < SOUND_EXPL_COUNT ; ++i ) {
    if ( _expl_id[i] != 0xffff ) {
      release_sound(_expl_id[i]);
      _expl_id[i] = 0xffff;
    }
  }

  if ( _level_cleanup )
    _level->cleanup();

  return( success );
}

void game::next_frame() {
  // update main area

  rectangle old_clip = clip_box();
  clip_box(rectangle(MAIN_XPOS, MAIN_YPOS, MAIN_WIDTH, MAIN_HEIGHT));

  // insert previously added objects into object list
  for ( _object_iterator o = _added_objects.begin() ; o != _added_objects.end() ; ++o ) {
    if ( *o ) {
      (*o)->initialize();
      _objects += *o;
      *o = NULL;
    }
  }

  // move groups
  for ( _group_iterator g = _groups.begin() ; g != _groups.end() ; ++g ) {
    if ( *g ) 
      (*g)->move();
  }

  // move objects
  for ( _object_iterator o = _objects.begin() ; o != _objects.end() ; ++o ) {
    if ( *o ) 
      (*o)->move();
  }

  // do collision check
  for ( _object_iterator o1 = _objects.begin() ; o1 != _objects.end() ; ++o1 ) {
    if ( *o1 && (*o1)->visible() ) {
      for ( _object_iterator o2 = o1 + 1 ; o2 != _objects.end() ; ++o2 ) {
	// check if o2 could be hitten by o1
	if ( *o2 &&
	     (*o2)->visible() &&
	     ((*o1)->type_mask() & (*o2)->hit_mask()) &&
	     ((*o2)->type_mask() & (*o1)->hit_mask()) ) {
	  (*o2)->check_collision(*o1);
	}
      }
    }
  }

  // remove 'out-of-view' objects & find layers
  Uint16 min_layer = 64;
  Uint16 max_layer = 0;

  for ( _object_iterator o = _objects.begin() ; o != _objects.end() ; ++o ) {
    if ( *o ) {
      if ( (*o)->auto_delete() ) {
	if ( (*o)->type_mask() & (OBJECT_ENEMY_PROJECTILE | OBJECT_PLAYER_PROJECTILE | OBJECT_NEUTRAL) ) {
	  if ( ((*o)->ypos() + (*o)->tbound() > ypos() + MAIN_HEIGHT + 100) ||
	       ((*o)->ypos() + (*o)->bbound() < ypos() - 100) ||
	       ((*o)->xpos() + (*o)->lbound() > map_width()) ||
	       ((*o)->xpos() + (*o)->rbound() < 0) )
	    (*o)->flags((*o)->flags() | OBJECT_DELETED);
	}
	else {
	  if ( (((*o)->yvel() >= 0) && ((*o)->ypos() + (*o)->tbound() > ypos() + 2*MAIN_HEIGHT)) ||
	       (((*o)->yvel() <= 0) && ((*o)->ypos() + (*o)->bbound() < 0)) ||
	       (((*o)->xvel() >= 0) && ((*o)->xpos() + (*o)->lbound() > map_width())) ||
	       (((*o)->xvel() <= 0) && ((*o)->xpos() + (*o)->rbound() < 0)) )
	    (*o)->flags((*o)->flags() | OBJECT_DELETED);
	}
      }

      if ( (*o)->visible() ) {
	if ( (*o)->layer() < min_layer )
	  min_layer = (*o)->layer();

	if ( (*o)->layer() > max_layer )
	  max_layer = (*o)->layer();
      }
    }
  }

  // really delete removed objects
  for ( _object_iterator o = _objects.begin() ; o != _objects.end() ; ++o ) {
    if ( *o ) {
      if ( ((*o)->flags() & OBJECT_DELETED) && (*o != _player) ) {
	delete *o;
	*o = NULL;
      }
    }
  }
  
  // really delete removed groups
  for ( _group_iterator g = _groups.begin() ; g != _groups.end() ; ++g ) {
    if ( *g && ((*g)->flags() & GROUP_DELETED) ) {
      delete *g;
      *g = NULL;
    }
  }
  
  // paint objects
  for ( Uint16 l = min_layer ; l <= max_layer ; ++l ) {
    for ( _object_iterator o = _objects.begin() ; o != _objects.end() ; ++o ) {
      if ( *o && (*o)->visible() && ((*o)->layer() == l) ) {
	(*o)->paint();
      }
    }
  }
  
  if ( SDL_GetTicks() - _fps_start_ticks > 1000 ) {
    _fps = 1000 * _frames / (SDL_GetTicks() - _fps_start_ticks);
    _fps_start_ticks = SDL_GetTicks();
    _frames = 0;
  }

  if ( _enable_fps ) {
    char buf[64];
    sprintf(buf, "%u FPS", _fps);
    draw_text(MAIN_XPOS + 16, MAIN_YPOS + 16, false, get_font(_flow_font_id), buf);

  }

  if ( _bigenemy_state != 0 ) {
    Sint16 x1, y1;
    Uint16 w1 = _bigenemy_surface->width();
    Uint16 h1 = _bigenemy_surface->height();

    switch ( _bigenemy_state ) {
    case 1 : /* coming in from left */
      x1 = (MAIN_WIDTH - 20) * (current_time() - _bigenemy_last_time) / 1000;
      y1 = 20;
      if ( current_time() - _bigenemy_last_time > 1000 ) {
	_bigenemy_state = 2;
	x1 = MAIN_WIDTH - 20;
      }
      break;

    case 2 : /* end position reached */
      x1 = MAIN_WIDTH - 20;
      y1 = 20;
      break;

    case 3 : /* moving out to left */
      x1 = MAIN_WIDTH - 20;
      y1 = 20 - (20 + h1) * (current_time() - _bigenemy_last_time) / 1000;
      if ( current_time() - _bigenemy_last_time > 1000 ) {
	_bigenemy_state = 0;
      }
      break;
    }

    x1 -= w1;

    Sint16 x2 = x1 + 7;
    Sint16 y2 = y1 + 7;
    Uint16 w2 = w1 - 14;
    Uint16 h2 = h1 - 14;
    
    copy(x1, y1, _bigenemy_surface, 0, rectangle(0, 0, w1, h1));
    draw_rect(rectangle(x2, y2, w2 * _bigenemy_current / _bigenemy_max, h2), 0xff, 0x80, 0x00);
  }

  for ( Uint16 i = 0 ; i < MESSAGE_COUNT ; ++i ) {
    if ( _messages[i].text ) {
      message *m = &_messages[i];

      switch ( m->phase ) {
      case 0 : {
	Sint32 dest_x = (MAIN_XPOS + (MAIN_WIDTH - get_font(_flow_font_id)->strwidth((Uint8 *)m->text)) / 2) << 8;
	Sint32 dt = current_time() - m->time;

	m->x -= dt * m->s / 1000;
	if ( m->x < dest_x ) {
	  m->x = dest_x;
	  m->phase = 1;
	}

	m->time = current_time();
	break;
      }

      case 1 :
	if ( current_time() - m->time > 2000 ) {
	  m->phase = 2;
	  m->time = current_time();
	}
	break;

      case 2 : {
	Sint32 dest_x = (MAIN_XPOS - get_font(_flow_font_id)->strwidth((Uint8 *)m->text)) << 8;
	Sint32 dt = current_time() - m->time;

	m->x -= dt * m->s / 1000;
	if ( m->x < dest_x ) {
	  delete m->text;
	  m->text = NULL;
	}

	m->time = current_time();
	break;
      }
      }

      if ( m->text )
	draw_text(m->x >> 8, m->y, false, get_font(_flow_font_id), m->text);
    }
  }

  
  if ( _paused ) {
    font *fnt = get_font(_flow_font_id);
    draw_text(MAIN_XPOS + (MAIN_WIDTH - fnt->strwidth((Uint8 *)"PAUSED")) / 2, MAIN_YPOS + (MAIN_HEIGHT - fnt->height()) / 2, false, fnt, "PAUSED");
  }

  update(rectangle(MAIN_XPOS, MAIN_YPOS, MAIN_WIDTH, MAIN_HEIGHT));

  clip_box(old_clip);
}

void game::update_panel() {
  rectangle old_clip = clip_box();
  clip_box(rectangle(PANEL_XPOS, PANEL_YPOS, PANEL_WIDTH, PANEL_HEIGHT));

  for ( Uint16 s_num = 0 ; s_num < 4 ; s_num++ ) {
    static Uint16 s_left_tab[4] = { 24, 85, 24, 85 };
    static Uint16 s_top_tab[4] = { 205, 205, 270, 270 };
    static Uint16 s_right_tab[4] = { 76, 137, 76, 137 };
    static Uint16 s_bottom_tab[4] = { 265, 265, 330, 330 };
    static Uint16 s_gun_tab[4] = { PLAYER_GUN_LASER, PLAYER_GUN_PLASMA, PLAYER_GUN_BALL, PLAYER_GUN_SEEKER };

    Uint16 s_left = PANEL_XPOS + s_left_tab[s_num];
    Uint16 s_top = PANEL_YPOS + s_top_tab[s_num];
    Uint16 s_right = PANEL_XPOS + s_right_tab[s_num];
    Uint16 s_bottom = PANEL_YPOS + s_bottom_tab[s_num];
    Uint16 s_gun = s_gun_tab[s_num];
    surface *s_upper_img = get_image(_slot_upper_id[s_num]);
    surface *s_lower_img = get_image(_slot_lower_id[s_num]);

    if ( (_player->gun_power(s_gun) > 0) && (_slot_status[s_num] == 0) ) {
      _slot_status[s_num] = 1;
      _slot_time[s_num] = current_time();
    }
    else if ( (_player->gun_power(s_gun) == 0) && (_slot_status[s_num] == 2) ) {
      _slot_status[s_num] = 3;
      _slot_time[s_num] = current_time();
    }

    switch ( _slot_status[s_num] ) {
    case 0 : /* closed */
      copy(s_left, s_top, s_upper_img, 0, rectangle(0, 0, s_upper_img->width(), s_upper_img->height()));
      break;

    case 1 : /* opening */
      if ( current_time() - _slot_time[s_num] < 1400 ) {
	copy(s_left, s_top, s_upper_img, (s_upper_img->frames() - 1) * (current_time() - _slot_time[s_num]) / 1400, rectangle(0, 0, s_upper_img->width(), s_upper_img->height()));
      }
      else {
	copy(s_left, s_top, s_upper_img, s_upper_img->frames() - 2, rectangle(0, 0, s_upper_img->width(), s_upper_img->height()));
	_slot_status[s_num] = 2;
      }
      break;

    case 2 : /* opened */
      if ( _player->gun_type() == s_gun )
	copy(s_left, s_top, s_upper_img, s_upper_img->frames() - 1, rectangle(0, 0, s_upper_img->width(), s_upper_img->height()));
      else
	copy(s_left, s_top, s_upper_img, s_upper_img->frames() - 2, rectangle(0, 0, s_upper_img->width(), s_upper_img->height()));
      break;

    case 3 : /* closing */
      if ( current_time() - _slot_time[s_num] < 1400 ) {
	copy(s_left, s_top, s_upper_img, (s_upper_img->frames() - 1) * (1399 - current_time() + _slot_time[s_num]) / 1400, rectangle(0, 0, s_upper_img->width(), s_upper_img->height()));
      }
      else {
	copy(s_left, s_top, s_upper_img, 0, rectangle(0, 0, s_upper_img->width(), s_upper_img->height()));
	_slot_status[s_num] = 0;
      }
      break;
    }

    copy(s_left + s_upper_img->width(), s_top, s_lower_img, _player->gun_power(s_gun), rectangle(0, 0, s_lower_img->width(), s_lower_img->height()));
    
    if ( s_gun == PLAYER_GUN_SEEKER ) {
      Uint16 l = _player->seeker_load();
      if ( l > 0 ) {
	draw_rect(rectangle(s_left + 1, s_top + 1, (s_right - s_left - 1) * l / 256, 7), 0xff, 0xff, 0x00);
      }
    }

    update(rectangle(s_left, s_top, s_upper_img->width() + s_lower_img->width(), s_upper_img->height()));
  }

  if ( _player->score() != _old_score ) {
    Uint16 left = 14;
    Uint16 top = 61;
    Uint16 right = 146;
    Uint16 bottom = 97;
    Uint16 width = right - left + 1;
    Uint16 height = bottom - top + 1;
    font *fnt = get_font(_panel_font_id);

    copy(left + PANEL_XPOS, top + PANEL_YPOS, get_image(_panel_id), 0, rectangle(left, top, width, height));

    draw_text(PANEL_XPOS + left + (width - fnt->strwidth((Uint8 *)"SCORE")) / 2, PANEL_YPOS + top + height / 2 - fnt->height(), false, fnt, "SCORE");

    char buf[16];
    sprintf(buf, "%lu", (unsigned long)_player->score());
    draw_text(PANEL_XPOS + left + (width - fnt->strwidth((Uint8 *)buf)) / 2, PANEL_YPOS + top + height / 2, false, fnt, buf);

    update(rectangle(PANEL_XPOS + left, PANEL_YPOS + top, width, height));

    _old_score = _player->score();
  }

  if ( _player->energy() != _old_energy ) {
    Uint16 left = 14;
    Uint16 top = 102;
    Uint16 right = 146;
    Uint16 bottom = 138;
    Uint16 width = right - left + 1;
    Uint16 height = bottom - top + 1;
    font *fnt = get_font(_panel_font_id);

    copy(left + PANEL_XPOS, top + PANEL_YPOS, get_image(_panel_id), 0, rectangle(left, top, width, height));

    draw_text(PANEL_XPOS + left + (width - fnt->strwidth((Uint8 *)"ENERGY")) / 2, PANEL_YPOS + top + height / 2 - fnt->height(), false, fnt, "ENERGY");

    Uint16 x1 = left + 11;
    Uint16 x2 = right - 11;
    Uint16 y1 = top + height / 2 + 3;
    Uint16 y2 = bottom - 5;

    Uint16 w = (x2 - x1 + 1) * _player->energy() / PLAYER_MAX_ENERGY;

    draw_rect(rectangle(PANEL_XPOS + x1, PANEL_YPOS + y1, w, y2 - y1 + 1),
			0xff - 0xff * _player->energy() / PLAYER_MAX_ENERGY, 
			0xff * _player->energy() / PLAYER_MAX_ENERGY, 
			0);

    update(rectangle(PANEL_XPOS + left, PANEL_YPOS + top, width, height));

    _old_energy = _player->energy();
  }

  if ( _player->lifes() != _old_lifes ) {
    Uint16 left = 14;
    Uint16 top = 143;
    Uint16 right = 146;
    Uint16 bottom = 178;
    Uint16 width = right - left + 1;
    Uint16 height = bottom - top + 1;
    font *fnt = get_font(_panel_font_id);

    copy(left + PANEL_XPOS, top + PANEL_YPOS, get_image(_panel_id), 0, rectangle(left, top, width, height));

    draw_text(PANEL_XPOS + left + (width - fnt->strwidth((Uint8 *)"LIFES")) / 2, PANEL_YPOS + top + height / 2 - fnt->height(), false, fnt, "LIFES");

    char buf[16];
    sprintf(buf, "%lu", (unsigned long)_player->lifes());
    draw_text(PANEL_XPOS + left + (width - fnt->strwidth((Uint8 *)buf)) / 2, PANEL_YPOS + top + height / 2, false, fnt, buf);

    update(rectangle(PANEL_XPOS + left, PANEL_YPOS + top, width, height));

    _old_lifes = _player->lifes();
  }

  {
    Sint32 left = 23;
    Sint32 top = 364;
    Sint32 right = 138;
    Sint32 bottom = 456;
    Sint32 width = right - left + 1;
    Sint32 height = bottom - top + 1;

    copy(left + PANEL_XPOS, top + PANEL_YPOS, get_image(_panel_id), 0, rectangle(left, top, width, height));

    Sint32 x_min = _player->xpos() - 500;
    Sint32 x_max = _player->xpos() + 500;
    Sint32 y_min = _player->ypos() - 900;
    Sint32 y_max = _player->ypos() + 100;

    for ( _object_iterator o = _objects.begin() ; o != _objects.end() ; ++o ) {
      if ( *o && (*o)->visible() &&
	   ((*o)->xpos() > x_min) &&
	   ((*o)->xpos() < x_max) &&
	   ((*o)->ypos() > y_min) &&
	   ((*o)->ypos() < y_max) ) {
	Sint32 x = PANEL_XPOS + left + width * ((*o)->xpos() - x_min) / (x_max - x_min);
	Sint32 y = PANEL_YPOS + top + height * ((*o)->ypos() - y_min) / (y_max - y_min);

	if ( (*o)->type_mask() & OBJECT_ENEMY_SHIP )
	  draw_pixel(x, y, 0xff, 0xff, 0xff);
	/*else if ( (*o)->type_mask() & (OBJECT_ENEMY_PROJECTILE | OBJECT_PLAYER_PROJECTILE) )
	  draw_pixel(x, y, 0x77, 0x77, 0x77);*/
	else if ( (*o)->type_mask() & OBJECT_BONUS )
	  draw_pixel(x, y, 0xff, 0x00, 0x00);
	else if ( (*o)->type_mask() & OBJECT_PLAYER_SHIP ) {
	  draw_pixel(x, y, 0xff, 0xff, 0x00);
	  draw_pixel(x + 1, y, 0xaa, 0xaa, 0x00);
	  draw_pixel(x - 1, y, 0xaa, 0xaa, 0x00);
	  draw_pixel(x, y + 1, 0xaa, 0xaa, 0x00);
	  draw_pixel(x, y - 1, 0xaa, 0xaa, 0x00);
	  draw_pixel(x + 1, y + 1, 0x66, 0x66, 0x00);
	  draw_pixel(x - 1, y + 1, 0x66, 0x66, 0x00);
	  draw_pixel(x + 1, y - 1, 0x66, 0x66, 0x00);
	  draw_pixel(x - 1, y - 1, 0x66, 0x66, 0x00);
	}
      }
    }

    update(rectangle(PANEL_XPOS + left, PANEL_YPOS + top, width, height));
  }

  clip_box(old_clip);
}

class group *game::obtain_group(const char *name, Uint32 id) {
  class group *res = NULL;

  for ( _const_group_iterator g = _groups.begin() ; g != _groups.end() ; ++g ) {
    if ( *g && !strcmp((*g)->name(), name) && ((*g)->id() == id) ) {
      res = *g;
      break;
    }
  }

  if ( !res ) {
    struct group_table *gt = group_table;
    while ( gt->name ) {
      if ( !strcmp(name, gt->name) )
	break;
      ++gt;
    }

    if ( gt->name ) {
      res = gt->constructor(id);

      _groups += res;
    }
    else {
      show_message("Fatal error: unknown group requested.\n");
      exit(0);
    }

    res->initialize();
  }

  return( res );
}

void game::bigenemy_init(Uint32 p) {
  _bigenemy_state = 1;
  _bigenemy_last_time = current_time();
  _bigenemy_max = p;
  _bigenemy_current = p;
}

object **game::first_object() {
  object **o = _objects.begin();

  while ( (o != _objects.end()) && (!(*o) || !(*o)->visible()) )
    ++o;

  if ( o != _objects.end() )
    return( o );

  return( NULL );
}

object **game::next_object(object **o) {
  do {
    ++o;
  } while ( (o != _objects.end()) && (!(*o) || !(*o)->visible()) );

  if ( o != _objects.end() )
    return( o );

  return( NULL );
}

void game::bigenemy_update(Uint32 p) {
  _bigenemy_current = p;
}

void game::bigenemy_cleanup() {
  _bigenemy_state = 3;
  _bigenemy_last_time = current_time();
}

void game::add_message(const char *text, Sint32 speed = 350) {
  Sint32 ypos = MAIN_YPOS + MAIN_HEIGHT / 4;

  message *m = _messages;
  for ( Uint16 i = 0 ; i < MESSAGE_COUNT ; ++i ) {
    if ( !m->text )
      break;

    ypos = m->y + get_font(_flow_font_id)->height() + 4;
    ++m;
  }

  if ( m < _messages + MESSAGE_COUNT ) {
    m->text = new char[strlen(text) + 1];
    strcpy(m->text, text);
    m->x = MAIN_WIDTH << 8;
    m->y = ypos;
    m->s = speed << 8;
    m->phase = 0;
    m->time = current_time();
  }
}
