
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: group.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include "main.h"
#include "game.h"
#include "group.h"

group::group(const char *n, Uint32 id) {
  _name = new char[strlen(n) + 1];
  strcpy(_name, n);
  _id = id;

  _xpos = 0;
  _ypos = 0;
  _xvel = 0;
  _yvel = 0;
  _xacc = 0;
  _yacc = 0;

  _flags = 0;

  _objects_count = 4;
  _objects = new object *[_objects_count];
  for ( Uint16 i = 0 ; i < _objects_count ; ++i )
    _objects[i] = NULL;

  _last_time = g->current_time();

  if ( enable_debug )
    printf("new group '%s', id %lu created.\n", _name, _id);
}

group::~group() {
  if ( enable_debug )
    printf("group '%s', id %lu deleted.\n", _name, _id);

  delete _objects;
  delete _name;
}

void group::add_object(class object *obj) {
  // find free part slot
  class object **o = _objects;
  class object **oend = _objects + _objects_count;
  while ( o < oend ) {
    if ( *o == NULL ) {
      *o = obj;
      return;
    }
    ++o;
  }

  // expand list, if no free slot was found
  Uint16 nc = _objects_count + 4;
  object **no = new object *[nc];
  for ( Uint16 i = 0 ; i < _objects_count ; ++i )
    no[i] = _objects[i];
  no[_objects_count] = obj;
  for ( Uint16 i = _objects_count + 1 ; i < nc ; ++i )
    no[i] = NULL;

  delete _objects; 
  _objects = no;
  _objects_count = nc;
}

void group::rem_object(class object *obj) {
  Uint16 cnt = 0;

  for ( class object **oend = _objects + _objects_count, **o = _objects ; o < oend ; ++o )
    if ( *o == obj )
      *o = NULL;
    else if ( *o )
      cnt++;

  if ( cnt == 0 )
    destroy();
}

void group::initialize() {
  _last_time = g->current_time();
}

void group::move() {
  Sint32 dt = g->current_time() - _last_time;
  if ( dt > 0 ) {
    _xpos += _xacc * dt * dt / 1000000 + _xvel * dt / 1000;
    _ypos += _yacc * dt * dt / 1000000 + _yvel * dt / 1000;
    _xvel += _xacc * dt / 1000;
    _yvel += _yacc * dt / 1000;

    _last_time = g->current_time();
  }
}

void group::destroy() {
  _flags |= GROUP_DELETED;
}

bool group::setup() {
  return( true );
}

void group::cleanup() {
}
