
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: level.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include "level.h"
#include "game.h"
#include "object_table.h"
#include "object.h"
#include "object_bonus.h"
#include "object_player.h"
#include "main.h"
#include "sstring.h"
#include "svector.h"

/**********************************************************************
 *                                                                    *
 * ldl_node classes                                                   *
 *                                                                    *
 **********************************************************************/

/* class ldl_node */

Sint32 ldl_node::eval(class level *) const {
}

void ldl_node::class_scan(class level *l) {
  if ( next() )
    next()->class_scan(l);

  if ( child() )
    child()->class_scan(l);
}

void ldl_node::execute(class level *) const {
}

void ldl_node::dump() const {
  printf("[Unknown]");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
};

/* class ldl_node_string */

void ldl_node_string::dump() const {
  printf("\"%s\"", _str);

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_int */

void ldl_node_int::dump() const {
  printf("%ld", _val);

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_id */

Sint32 ldl_node_id::eval(class level *l) const {
  Sint32 r = 0;

  ldl_node_string *name = dynamic_cast<ldl_node_string *>(child());
  if ( name ) {
    r = l->get_variable(name->str());
  }

  return(r);
}

void ldl_node_id::dump() const {
  printf("(IDENTIFER");

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_op */

Sint32 ldl_node_op::eval(class level *l) const {
  ldl_node *a1 = child();
  ldl_node *a2 = a1 ? a1->next() : NULL;

  Sint32 v1 = a1 ? a1->eval(l) : 0;
  Sint32 v2 = a2 ? a2->eval(l) : 0;
  Sint32 r = 0;

  switch ( _op ) {
  case LDL_OP_ADD : r = v1 + v2; break;
  case LDL_OP_SUB : r = v1 - v2; break;
  case LDL_OP_MUL : r = v1 * v2; break;
  case LDL_OP_DIV : r = v1 / v2; break;
  case LDL_OP_MOD : r = v1 % v2; break;
  case LDL_OP_NEG : r = -v1; break;
  case LDL_OP_GT :  r = (v1 > v2) ? 1 : 0; break;
  case LDL_OP_GE :  r = (v1 >= v2) ? 1 : 0; break;
  case LDL_OP_LT :  r = (v1 < v2) ? 1 : 0; break;
  case LDL_OP_LE :  r = (v1 <= v2) ? 1 : 0; break;
  case LDL_OP_EQ :  r = (v1 == v2) ? 1 : 0; break;
  case LDL_OP_NE :  r = (v1 != v2) ? 1 : 0; break;
  }

  return(r);
}

void ldl_node_op::dump() const {
  printf("(");

  switch ( _op ) {
  case LDL_OP_ADD : printf("OP_ADD"); break;
  case LDL_OP_SUB : printf("OP_SUB"); break;
  case LDL_OP_MUL : printf("OP_MUL"); break;
  case LDL_OP_DIV : printf("OP_DIV"); break;
  case LDL_OP_MOD : printf("OP_MOD"); break;
  case LDL_OP_NEG : printf("OP_NEG"); break;
  case LDL_OP_GT : printf("OP_GT"); break;
  case LDL_OP_GE : printf("OP_GE"); break;
  case LDL_OP_LT : printf("OP_LT"); break;
  case LDL_OP_LE : printf("OP_LE"); break;
  case LDL_OP_EQ : printf("OP_EQ"); break;
  case LDL_OP_NE : printf("OP_NE"); break;
  default :         printf("OP_???"); break;
  }

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_object */

void ldl_node_object::class_scan(class level *l) {
  ldl_node_string *name = dynamic_cast<ldl_node_string *>(child());
  if ( name ) {
    _id = l->register_class(name->str());
  }

  ldl_node::class_scan(l);
}

void ldl_node_object::execute(class level *l) const {
  ldl_node_string *name = dynamic_cast<ldl_node_string *>(child());
  if ( name ) {
    if ( enable_debug )
      printf("OBJECT \"%s\"\n", name->str());

    ldl_node_modifier *mod = child() ? dynamic_cast<ldl_node_modifier *>(child()->next()) : NULL;
    if ( mod ) {
      svector<object_modifier> mod_tab;

      mod->build_modifier_table(l, mod_tab);

      if ( enable_debug ) {
	for ( svector<object_modifier>::iterator i = mod_tab.begin() ; i != mod_tab.end() ; ++i ) {
	  printf("  %s=%ld\n", (char *)i->name, i->val);
	}
      }

      object *o = object_table[_id].constructor();
      o->set_modifiers(mod_tab);
      g->add_object(o);
    }
  }

  if ( next() )
    next()->execute(l);
}

void ldl_node_object::dump() const {
  printf("(OBJECT");

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_bonus */

void ldl_node_bonus::execute(class level *l) const {
  ldl_node_string *name = dynamic_cast<ldl_node_string *>(child());
  if ( name ) {
    if ( enable_debug )
      printf("OBJECT \"%s\"\n", name->str());

    ldl_node_modifier *mod = child() ? dynamic_cast<ldl_node_modifier *>(child()->next()) : NULL;
    if ( mod ) {
      svector<object_modifier> mod_tab;

      mod->build_modifier_table(l, mod_tab);
    }
  }

  if ( next() )
    next()->execute(l);
}

void ldl_node_bonus::dump() const {
  printf("(BONUS");

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_modifier */

void ldl_node_modifier::build_modifier_table(class level *l, svector<object_modifier> &tab) const {
  ldl_node_string *name = dynamic_cast<ldl_node_string *>(child());
  if ( name ) {
    Sint32 val = name->next() ? name->next()->eval(l) : 0;

    object_modifier mod;
    mod.name = name->str();
    mod.val = val;
    tab += mod;
  }

  ldl_node_modifier *n = dynamic_cast<ldl_node_modifier *>(next());
  if ( n )
    n->build_modifier_table(l, tab);
}

void ldl_node_modifier::dump() const {
  printf("(MODIFIER");

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_trigger */

void ldl_node_trigger::execute(class level *l) const {
  ldl_node_trigger_type *trig_type = dynamic_cast<ldl_node_trigger_type *>(child());
  if ( trig_type ) {
    if ( enable_debug ) {
      printf("TRIGGER \"");
      switch ( trig_type->trig() ) {
      case LDL_TRIGGER_POSITION : printf("POSITION"); break;
      case LDL_TRIGGER_RELATIVE : printf("RELATIVE"); break;
      case LDL_TRIGGER_SIGNAL :   printf("SIGNAL"); break;
      case LDL_TRIGGER_TIME :     printf("TIME"); break;
      default:                    printf("???"); break;
      }
      printf("\"\n");
    }

    Uint16 type = trig_type->trig();
    ldl_node *code = trig_type->next() ? trig_type->next()->clone() : NULL;

    ldl_trigger *trig = new ldl_trigger(type, code);

    switch ( type ) {
    case LDL_TRIGGER_POSITION :
      if ( trig_type->child() )
	trig->ypos( trig_type->child()->eval(l) );
      break;

    case LDL_TRIGGER_RELATIVE :
      if ( trig_type->child() ) {
	trig->xpos( trig_type->child()->eval(l) );

	if ( trig_type->child()->next() ) {
	  trig->xpos( trig_type->child()->next()->eval(l) );

	  if ( trig_type->child()->next()->next() )
	    trig->xpos( trig_type->child()->next()->next()->eval(l) );
	}
      }
      break;

    case LDL_TRIGGER_SIGNAL :
      if ( trig_type->child() )
	trig->signal( trig_type->child()->eval(l) );
      break;

    case LDL_TRIGGER_TIME :
      if ( trig_type->child() )
	trig->time( trig_type->child()->eval(l) );
      break;
    }

    l->add_trigger(trig);
  }

  if ( next() )
    next()->execute(l);
}

void ldl_node_trigger::dump() const {
  printf("(TRIGGER");

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_trigger_type */

void ldl_node_trigger_type::dump() const {
  printf("(TRIGGERTYPE ");

  switch ( _trig ) {
  case LDL_TRIGGER_POSITION : printf("POSITION"); break;
  case LDL_TRIGGER_RELATIVE : printf("RELATIVE"); break;
  case LDL_TRIGGER_SIGNAL :   printf("SIGNAL"); break;
  case LDL_TRIGGER_TIME :   printf("TIME"); break;
  default:                    printf("???"); break;
  }

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_while */

void ldl_node_while::execute(class level *l) const {
  while ( true ) {
    Sint32 val = child() ? child()->eval(l) : 0;

    if ( enable_debug ) 
      printf("WHILE ( %d )\n", val);

    if ( !val )
      break;

    if ( child() && child()->next() )
      child()->next()->execute(l);
  }

  if ( next() )
    next()->execute(l);
}

void ldl_node_while::dump() const {
  printf("(WHILE");

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_repeat */

void ldl_node_repeat::execute(class level *l) const {
  Sint32 val = child() ? child()->eval(l) : 0;

  if ( enable_debug ) 
    printf("REPEAT ( %d )\n", val);

  while ( val > 0 ) {
    if ( child() && child()->next() )
      child()->next()->execute(l);

    --val;
  }

  if ( next() )
    next()->execute(l);
}

void ldl_node_repeat::dump() const {
  printf("(REPEAT");

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_if */

void ldl_node_if::execute(class level *l) const {
  Sint32 val = child() ? child()->eval(l) : 0;

  if ( enable_debug )
    printf("IF ( %d )\n", val);

  if ( val ) {
    if ( child() && child()->next() )
      child()->next()->execute(l);
  }

  if ( next() )
    next()->execute(l);
}

void ldl_node_if::dump() const {
  printf("(IF");

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_define */

void ldl_node_define::execute(class level *l) const {
  ldl_node_string *name = dynamic_cast<ldl_node_string *>(child());
  if ( name ) {
    Sint32 val = name->next() ? name->next()->eval(l) : 0;

    if ( enable_debug )
      printf("DEFINE %s=%d\n", name->str(), val);

    l->set_variable(name->str(), val);
  }

  if ( next() )
    next()->execute(l);
}

void ldl_node_define::dump() const {
  printf("(DEFINE");

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_call */

void ldl_node_call::execute(class level *l) const {
  ldl_node_string *name = dynamic_cast<ldl_node_string *>(child());
  if ( name ) {
    if ( enable_debug )
      printf("CALL %s\n", name->str());

    ldl_node *sub = l->get_sub(name->str());
    if ( sub )
      sub->execute(l);
  }

  if ( next() )
    next()->execute(l);
}

void ldl_node_call::dump() const {
  printf("(CALL");

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}

/* class ldl_node_sub */

void ldl_node_sub::execute(class level *l) const {
  ldl_node_string *name = dynamic_cast<ldl_node_string *>(child());
  if ( name ) {
    if ( enable_debug )
      printf("SUB %s\n", name->str());

    ldl_node *sub = name->next() ? name->next()->clone() : NULL;

    l->set_sub(name->str(), sub);
  }

  if ( next() )
    next()->execute(l);
}

void ldl_node_sub::dump() const {
  printf("(SUB");

  if ( child() ) {
    printf(", ");
    child()->dump();
  }

  printf(")");

  if ( next() ) {
    printf(", ");
    next()->dump();
  }
}


/**********************************************************************
 *                                                                    *
 * class level                                                        *
 *                                                                    *
 **********************************************************************/

extern FILE *yyin;
extern int lexer_line_number;
extern char *lexer_file_name;
int yyparse(void *);
void yyrestart(FILE *);
void lexer_setup();
void lexer_cleanup();

level::level() {
  _song = 0xffff;

  _width = 2 * MAIN_WIDTH;
  _height = 20 * MAIN_HEIGHT;
  _yvel = 50;

  _root = NULL;
}

level::~level() {
  delete _root;

  cleanup_variables();
  cleanup_subs();
}

bool level::load(char *dat) {
  FILE *fp = fopen(dat, "r");
  if ( fp ) {
    lexer_setup();

    yyrestart(fp);
    lexer_line_number = 1;
    lexer_file_name = dat;

    delete _root;
    _root = NULL;
    yyparse((void *)&_root);

    lexer_cleanup();
    fclose(fp);
  }

  return( true );
}

Uint16 level::register_class(char *name) {
  int id = 0;

  struct object_table *o = object_table;
  while ( o->name ) {
    if ( !strcmp(name, o->name) )
      break;
    ++o;
    ++id;
  }

  if ( o->name ) {
    for ( svector<Uint16>::const_iterator i = _classes.begin() ; i != _classes.end() ; ++i )
      if ( !strcmp(name, object_table[*i].name) )
	return( id );

    _classes += id;

    return( id );
  }
  else
    printf("Unknown class '%s' requested.\n", name);

  return( 0 );
}

void level::cleanup_variables() {
  for ( sarray<var>::iterator i = _vars.begin() ; i != _vars.end() ; ++i ) {
    if ( *i ) {
      delete (*i)->name;
      delete *i;
      *i = NULL;
    }
  }
}

Sint32 level::get_variable(const char *name) {
  Sint32 r = 0;

  if ( !strcmp(name, "scr_height") )
    r = MAIN_HEIGHT;
  else if ( !strcmp(name, "scr_width") )
    r = MAIN_WIDTH;
  else if ( !strcmp(name, "map_height") )
    r = height();
  else if ( !strcmp(name, "map_width") )
    r = width();
  else if ( !strcmp(name, "map_vel") )
    r = yvel();
  else if ( !strcmp(name, "time") )
    r = g->current_time();
  else if ( !strcmp(name, "scr_x") )
    r = g->xpos();
  else if ( !strcmp(name, "scr_y") )
    r = g->ypos();
  else if ( !strcmp(name, "top") )
    r = g->ypos() - 900;
  else if ( !strcmp(name, "rand") )
    r = rand();
  else {
    for ( sarray<var>::iterator i = _vars.begin() ; i != _vars.end() ; ++i ) {
      if ( *i && !strcmp((*i)->name, name) ) {
	r = (*i)->val;
	break;
      }
    }
  }

  return(r);
}

void level::set_variable(const char *name, Sint32 val) {
  if ( !strcmp(name, "map_height") )
    _height = val;
  else if ( !strcmp(name, "map_width") )
    _width = val;
  else if ( !strcmp(name, "map_vel") )
    _yvel = val;
  else {
    for ( sarray<var>::iterator i = _vars.begin() ; i != _vars.end() ; ++i ) {
      if ( *i && !strcmp((*i)->name, name) ) {
	(*i)->val = val;
	return;
      }
    }

    var *v = new var;
    v->name = new char[strlen(name) + 1];
    strcpy(v->name, name);
    v->val = val;

    _vars += v;
  }
}

void level::cleanup_subs() {
  for ( sarray<sub>::iterator i = _subs.begin() ; i != _subs.end() ; ++i ) {
    if ( *i ) {
      delete (*i)->name;
      delete (*i)->root;
      delete *i;
      *i = NULL;
    }
  }
}

ldl_node *level::get_sub(const char *name) {
  ldl_node *r = NULL;

  for ( sarray<sub>::iterator i = _subs.begin() ; i != _subs.end() ; ++i ) {
    if ( *i && !strcmp((*i)->name, name) ) {
      r = (*i)->root;
      break;
    }
  }

  return(r);
}

void level::set_sub(const char *name, ldl_node *root) {
  for ( sarray<sub>::iterator i = _subs.begin() ; i != _subs.end() ; ++i ) {
    if ( *i && !strcmp((*i)->name, name) ) {
      delete (*i)->root;
      (*i)->root = root;
      return;
    }
  }

  sub *s = new sub;
  s->name = new char[strlen(name) + 1];
  strcpy(s->name, name);
  s->root = root;

  _subs += s;
}

bool level::setup() {
  bool ok = true;

  _root->class_scan(this);

  for ( svector<Uint16>::const_iterator i = _classes.begin() ; i != _classes.end() ; ++i ) {
    if ( !object_table[*i].setup() )
      ok = false;
  }

  if ( ok ) {
    cleanup_variables();
    cleanup_subs();
    _root->execute(this);
  }

  return( ok );
}

void level::cleanup() {
  for ( sarray<ldl_trigger>::iterator i = _triggers.begin() ; i != _triggers.end() ; ++i )
    delete *i;
  _triggers.clear();
  
  for ( sarray<ldl_trigger>::iterator i = _added_triggers.begin() ; i != _added_triggers.end() ; ++i )
    delete *i;
  _triggers.clear();
  
  for ( svector<Uint16>::const_iterator i = _classes.begin() ; i != _classes.end() ; ++i )
    object_table[*i].cleanup();
  _classes.clear();

  delete _root;
  _root = NULL;
}

void level::check_triggers() {
  for ( sarray<ldl_trigger>::iterator i = _added_triggers.begin() ; i != _added_triggers.end() ; ++i ) {
    if ( *i ) {
      _triggers += *i;
      *i = NULL;
    }
  }

  for ( sarray<ldl_trigger>::iterator i = _triggers.begin() ; i != _triggers.end() ; ++i ) {
    if ( *i ) {
      bool hit = false;
      switch ( (*i)->type() ) {
      case LDL_TRIGGER_POSITION :
	if ( g->ypos() <= (*i)->ypos() ) {
	  if ( enable_debug )
	    printf("Position trigger toggled at y=%ld.\n", g->ypos());
	  hit = true;
	}
	break;

      case LDL_TRIGGER_RELATIVE : {
	Sint32 dx = g->player()->xpos() - (*i)->xpos();
	Sint32 dy = g->player()->ypos() - (*i)->ypos();

	if ( dx * dx + dy * dy <= (*i)->dist() * (*i)->dist() ) {
	  if ( enable_debug )
	    printf("Relative trigger toggled at x=%ld, y=%ld.\n", g->player()->xpos(), g->player()->ypos());
	  hit = true;
	}
	break;
      }

      case LDL_TRIGGER_SIGNAL :
	if ( g->signal_mask() & (*i)->signal() ) {
	  if ( enable_debug )
	    printf("Signal trigger toggled.\n");
	  g->rem_signal((*i)->signal());
	  hit = true;
	}
	break;

      case LDL_TRIGGER_TIME :
	if ( g->current_time() >= (*i)->time() ) {
	  if ( enable_debug )
	    printf("Time trigger toggled.\n");
	  hit = true;
	}
	break;
      }

      if ( hit ) {
	if ( (*i)->code() )
	  (*i)->code()->execute(this);
	
	delete *i;
	*i = NULL;
      }
    }
  }
}
