%{
#include <stdlib.h>
#include <stdio.h>
#include "level.h"

#define YYSTYPE ldl_node *
#define YYPARSE_PARAM root

void yyerror(char *);

extern char *yytext;
extern int lexer_line_number;
extern char *lexer_file_name;
int yylex();
char *lexer_get_string(int);
%}
 
%token NUMBER
%left OP_LE OP_LT OP_GE OP_GT OP_NE OP_EQ
%left OP_ADD OP_SUB
%left OP_MUL OP_DIV OP_MOD
%token IDENTIFIER
%token FINISH
%token OBJECT
%token WHILE
%token REPEAT
%token IF
%token BONUS
%token TRIGGER
%token POSITION
%token RELATIVE
%token SIGNAL
%token TIME
%token DEFINE
%token SUB
%token CALL

%% /* Grammar rules and actions follow */

input:    
	  statement_list { *((ldl_node **)root) = $1; }
	  ;

statement_list:
	  /* empty */ { $$ = NULL; }
	  | statement statement_list { $$ = $1; $1->next($2); }
	  ;

statement:
	  DEFINE IDENTIFIER '=' expression ';' 
		 { $$ = new ldl_node_define(); $$->child($2); $2->next($4); }
	  | WHILE '(' expression ')' '{' statement_list '}' 
		 { $$ = new ldl_node_while(); $$->child($3); $3->next($6); }
	  | REPEAT '(' expression ')' '{' statement_list '}' 
		 { $$ = new ldl_node_repeat(); $$->child($3); $3->next($6); }
	  | IF '(' expression ')' '{' statement_list '}' 
		 { $$ = new ldl_node_if(); $$->child($3); $3->next($6); }
	  | OBJECT IDENTIFIER '{' object_modifiers '}'
		 { $$ = new ldl_node_object(); $$->child($2); $2->next($4); }
	  | BONUS IDENTIFIER '{' object_modifiers '}'
		 { $$ = new ldl_node_bonus(); $$->child($2); $2->next($4); }
	  | TRIGGER trigger_type '{' statement_list '}'
	         { $$ = new ldl_node_trigger(); $$->child($2); $2->next($4); }
	  | SUB IDENTIFIER '{' statement_list '}'
	         { $$ = new ldl_node_sub(); $$->child($2); $2->next($4); }
	  | CALL IDENTIFIER
	         { $$ = new ldl_node_call(); $$->child($2); }
	  ;

trigger_type:
	  SIGNAL expression
	         { $$ = new ldl_node_trigger_type(LDL_TRIGGER_SIGNAL); $$->child($2); }
	  | TIME expression
	         { $$ = new ldl_node_trigger_type(LDL_TRIGGER_TIME); $$->child($2); }
	  | POSITION expression
	         { $$ = new ldl_node_trigger_type(LDL_TRIGGER_POSITION); $$->child($2); }		 
	  | RELATIVE expression ',' expression ',' expression
	         { $$ = new ldl_node_trigger_type(LDL_TRIGGER_RELATIVE); $$->child($2); $2->next($4); $2->next($6); }
	  ;

object_modifiers:
	  /* empty */ { $$ = NULL; }
	  | object_modifier ';' object_modifiers { $$ = $1; $1->next($3); }
	  ;

object_modifier:
	  IDENTIFIER '=' expression { $$ = new ldl_node_modifier(); $$->child($1); $1->next($3); }
	  ;
	  
expression:
	  NUMBER		      { $$ = $1; }
	  | IDENTIFIER		      { $$ = new ldl_node_id(); $$->child($1); }
	  | OP_SUB expression            { $$ = new ldl_node_op(LDL_OP_NEG); $$->child($2); }
	  | expression OP_ADD expression { $$ = new ldl_node_op(LDL_OP_ADD); $$->child($1); $1->next($3); }
	  | expression OP_SUB expression { $$ = new ldl_node_op(LDL_OP_SUB); $$->child($1); $1->next($3) }
	  | expression OP_MUL expression { $$ = new ldl_node_op(LDL_OP_MUL); $$->child($1); $1->next($3) }
	  | expression OP_DIV expression { $$ = new ldl_node_op(LDL_OP_DIV); $$->child($1); $1->next($3) }
	  | expression OP_MOD expression { $$ = new ldl_node_op(LDL_OP_MOD); $$->child($1); $1->next($3) }
	  | expression OP_LT expression { $$ = new ldl_node_op(LDL_OP_LT); $$->child($1); $1->next($3) }
	  | expression OP_LE expression { $$ = new ldl_node_op(LDL_OP_LE); $$->child($1); $1->next($3) }
	  | expression OP_GT expression { $$ = new ldl_node_op(LDL_OP_GT); $$->child($1); $1->next($3) }
	  | expression OP_GE expression { $$ = new ldl_node_op(LDL_OP_GE); $$->child($1); $1->next($3) }
	  | expression OP_EQ expression { $$ = new ldl_node_op(LDL_OP_EQ); $$->child($1); $1->next($3) }
	  | expression OP_NE expression { $$ = new ldl_node_op(LDL_OP_NE); $$->child($1); $1->next($3) }
	  | '(' expression ')'        { $$ = $2; }
	  ;

%%

void yyerror(char *s) {
  fprintf(stderr, "%s, line %d: %s\n", lexer_file_name, lexer_line_number, s);
}
