
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: level_starfield.cc,v 1.1 2000/10/15 02:55:28 pink Exp pink $ */

#include "level_starfield.h"
#include "main.h"
#include "game.h"

bool level_starfield::setup() {
  if ( level::setup() ) {
    for ( struct star *send = _stars + STAR_COUNT, *s = _stars ; s < send ; ++s ) {
      s->x = (rand() % width()) << 8;
      s->y = ((rand() % MAIN_HEIGHT) + MAIN_YPOS) << 8;
      s->z = rand() % (STAR_DEPTH << 8);
    }

    _last_time = 0x7fffffff;

    return( true );
  }

  return( false );
}

void level_starfield::render_background() {
  if ( _last_time > g->current_time() )
    _last_time = g->current_time();

  if ( g->current_time() > _last_time ) {
    Sint32 dt = g->current_time() - _last_time;

    for ( struct star *send = _stars + STAR_COUNT, *s = _stars ; s < send ; ++s ) {
      if ( s->y >> 8 >= MAIN_YPOS + MAIN_HEIGHT ) {
	s->x = (rand() % width()) << 8;
	s->y = MAIN_YPOS << 8;
	s->z = rand() % (STAR_DEPTH << 8);
      }

      Sint32 yvel = (s->z + 1) * g->yvel() / STAR_DEPTH;
    
      s->y += dt * yvel / 1000;
    }

    _last_time = g->current_time();
  }

  rectangle old_clip = g->clip_box();
  g->clip_box(rectangle(MAIN_XPOS, MAIN_YPOS, MAIN_WIDTH, MAIN_HEIGHT));

  g->draw_rect(rectangle(MAIN_XPOS, MAIN_YPOS, MAIN_WIDTH, MAIN_HEIGHT), 0, 0, 0);

  for ( struct star *send = _stars + STAR_COUNT, *s = _stars ; s < send ; ++s ) {
    Uint16 x = (s->x >> 8) - g->xpos();
    Uint16 y = s->y >> 8;
    Uint16 z = s->z >> 8;
    Uint8 c = 192 * z / STAR_DEPTH + 32;
    g->draw_pixel(x, y, c, c, c);
    if ( z == STAR_DEPTH - 1 ) {
      Uint8 c2 = c / 2;
      Uint8 c3 = c / 5;
      g->draw_pixel(x + 3, y, c3, c3, c3);
      g->draw_pixel(x - 3, y, c3, c3, c3);
      g->draw_pixel(x, y + 3, c3, c3, c3);
      g->draw_pixel(x, y - 3, c3, c3, c3);
      g->draw_pixel(x + 3, y, c2, c2, c2);
      g->draw_pixel(x - 3, y, c2, c2, c2);
      g->draw_pixel(x, y + 3, c2, c2, c2);
      g->draw_pixel(x, y - 3, c2, c2, c2);
      g->draw_pixel(x + 2, y, c, c, c);
      g->draw_pixel(x - 2, y, c, c, c);
      g->draw_pixel(x, y + 2, c, c, c);
      g->draw_pixel(x, y - 2, c, c, c);
      g->draw_pixel(x + 1, y, c, c, c);
      g->draw_pixel(x - 1, y, c, c, c);
      g->draw_pixel(x, y + 1, c, c, c);
      g->draw_pixel(x, y - 1, c, c, c);
      g->draw_pixel(x + 1, y + 1, c3, c3, c3);
      g->draw_pixel(x + 1, y - 1, c3, c3, c3);
      g->draw_pixel(x - 1, y + 1, c3, c3, c3);
      g->draw_pixel(x - 1, y - 1, c3, c3, c3);
    }
    else if ( z == STAR_DEPTH - 2 ) {
      Uint8 c2 = c / 2;
      Uint8 c3 = c / 5;
      g->draw_pixel(x + 2, y, c2, c2, c2);
      g->draw_pixel(x - 2, y, c2, c2, c2);
      g->draw_pixel(x, y + 2, c2, c2, c2);
      g->draw_pixel(x, y - 2, c2, c2, c2);
      g->draw_pixel(x + 1, y, c, c, c);
      g->draw_pixel(x - 1, y, c, c, c);
      g->draw_pixel(x, y + 1, c, c, c);
      g->draw_pixel(x, y - 1, c, c, c);
      g->draw_pixel(x + 1, y + 1, c3, c3, c3);
      g->draw_pixel(x + 1, y - 1, c3, c3, c3);
      g->draw_pixel(x - 1, y + 1, c3, c3, c3);
      g->draw_pixel(x - 1, y - 1, c3, c3, c3);
    }
    else if ( z == STAR_DEPTH - 3 ) {
      Uint8 c2 = c / 2;
      Uint8 c3 = c / 5;
      g->draw_pixel(x + 1, y, c2, c2, c2);
      g->draw_pixel(x - 1, y, c2, c2, c2);
      g->draw_pixel(x, y + 1, c2, c2, c2);
      g->draw_pixel(x, y - 1, c2, c2, c2);
      g->draw_pixel(x + 1, y + 1, c3, c3, c3);
      g->draw_pixel(x + 1, y - 1, c3, c3, c3);
      g->draw_pixel(x - 1, y + 1, c3, c3, c3);
      g->draw_pixel(x - 1, y - 1, c3, c3, c3);
    }
  }

  g->clip_box(old_clip);
}
