
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: leveldemo.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include <memory.h>
#include "leveldemo.h"
#include "game.h"

leveldemo::leveldemo() {
  _width = 100;
  _height = 100;
  _yvel = 70;
}

leveldemo::~leveldemo() {
}

bool leveldemo::setup() {
  bool rc = false;
  Sint16 x = 32;
  Sint16 y = 8;
  
  Uint16 bg_id = g->obtain_image("gfx/level.png");
  if ( bg_id != 0xffff ) {
    g->copy(0, 0, g->get_image(bg_id), rectangle(0, 0, 640, 480));
    g->update();

    Sint32 start_time = SDL_GetTicks();

    Uint16 flow_id = g->obtain_font("flow");
    if ( flow_id != 0xffff ) {
      font *flow = g->get_font(flow_id);

      Uint16 small_id = g->obtain_font("panel");
      if ( small_id != 0xffff ) {
	font *small = g->get_font(small_id);

	if ( load("levels/leveldemo.dat") ) {
	  bool skip = false;

	  Uint16 pic_id = g->obtain_image("gfx/level1.png");
	  if ( pic_id != 0xffff ) {
	    surface *pic = g->get_image(pic_id);

	    while ( !skip && (SDL_GetTicks() - start_time < 1500) ) {
	      SDL_Event event;
	      while(SDL_PollEvent(&event)) {
		switch (event.type) {
		case SDL_KEYDOWN:
		  skip = true;
		  break;
		}
	      }

	      SDL_Delay(25);
	    }

	    Uint16 aw = (pic->width() + 9) / 16;
	    Uint16 ah = (pic->height() + 9) / 16;
	    Uint8 *a = new Uint8[aw * ah];
	    memset(a, 0, sizeof(Uint8) * aw * ah);

	    for ( Sint16 i = aw * ah - 1 ; (i >= 0) && !skip ; --i ) {
	      start_time = SDL_GetTicks();

	      Uint16 ax = rand() % aw;
	      Uint16 ay = rand() % ah;

	      while ( a[ay * aw + ax] ) {
		ax++;
		if ( ax >= aw ) {
		  ax = 0;
		  ay++;
		  if ( ay >= ah )
		    ay = 0;
		}
	      }

	      a[ay * aw + ax] = 1;

	      g->copy((640 - pic->width()) / 2 + ax * 16, y + ay * 16, pic, rectangle(ax * 16, ay * 16, 16, 16));
	      g->update(rectangle((640 - pic->width()) / 2 + ax * 16, y + ay * 16, 16, 16));
	
	      Sint32 dt = SDL_GetTicks() - start_time;
	      if ( dt < 10 )
		SDL_Delay(10 - dt);

	      SDL_Event event;
	      while(SDL_PollEvent(&event)) {
		switch (event.type) {
		case SDL_KEYDOWN:
		  skip = true;
		  break;
		}
	      }
	    }

	    delete a;

	    if ( skip ) {
	      g->copy((640 - pic->width()) / 2, y, pic, rectangle(0, 0, pic->width(), pic->height()));
	      g->update(rectangle((640 - pic->width()) / 2, y, pic->width(), pic->height()));
	    }

	    y += pic->height() + 8;

	    g->release_image(pic_id);
	  }

	  start_time = SDL_GetTicks();
	  while ( !skip && (SDL_GetTicks() - start_time < 1000) ) {
	    SDL_Event event;
	    while(SDL_PollEvent(&event)) {
	      switch (event.type) {
	      case SDL_KEYDOWN:
		skip = true;
		break;
	      }
	    }

	    SDL_Delay(25);
	  }

	  char *txt =
	    "This level is just an example. It will not appear in the final release.\n"
	    "Don't forget to send your comments or suggestions to pink@odahoda.de";
	  char *s = txt;

	  while ( *s ) {
	    if ( *s == '\n' ) {
	      x = 32;
	      y += flow->height() + 2;
	      if ( !skip )
		SDL_Delay(1000/10);
	      ++s;
	    }
	    else {
	      Sint32 tx = x;
	      char *ts = s;
	    
	      while ( *ts && (*ts != ' ') && (*ts != '\n') )
		tx += flow->width(*ts++);

	      if ( tx >= 640 - 32 ) {
		x = 32;
		y += flow->height() + 2;
		if ( !skip )
		  SDL_Delay(1000/10);
	      }

	      if ( *s != ' ' ) {
		char t[2] = { *s, 0 };
	      
		g->draw_text(x, y, false, flow, t);
		g->update(rectangle(x, y, flow->width(*s), flow->height()));
		if ( !skip ) {
		  if ( *s == '.' )
		    SDL_Delay(150 + (rand() % 50));
		  else if ( *s == '.' )
		    SDL_Delay(100 + (rand() % 50));
		  else
		    SDL_Delay(50 + (rand() % 50));
		}
	      }

	      x += flow->width(*s);
	      ++s;
	    }

	    SDL_Event event;
	    while(SDL_PollEvent(&event)) {
	      switch (event.type) {
	      case SDL_KEYDOWN:
		skip = true;
		break;
	      }
	    }
	  }

	  if ( level_starfield::setup() ) {
	    _song = g->obtain_song("mus/level1.mod");
	    if ( _song != 0xffff ) {
	      char buf[80];
	      sprintf(buf, "Get ready!");
	      y = 480 - small->height();
	      x = (640 - small->strwidth((Uint8 *)buf)) / 2;
    
	      char *s = buf;

	      SDL_Event event;
	      while(SDL_PollEvent(&event));
      
	      bool wait = true;
	      while ( !skip ) {
		if ( *s ) {
		  char t[2] = { *s, 0 };
	      
		  g->draw_text(x, y, false, small, t);
		  g->update(rectangle(x, y, small->width(*s), small->height()));
		  SDL_Delay(50);
	    
		  x += small->width(*s);
		  ++s;
		}
	  
		SDL_Event event;
		while(SDL_PollEvent(&event)) {
		  switch (event.type) {
		  case SDL_KEYDOWN:
		    skip = true;
		    break;
		  }
		}
	      }

	      rc = true;
	    }
	  }
	}

	g->release_font(small_id);
      }

      g->release_font(flow_id);
    }

    g->release_image(bg_id);
  }

  return( rc );
}

void leveldemo::cleanup() {
  g->release_song(_song);

  level_starfield::cleanup();
}
