
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: main.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

/* $Id: main.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <time.h>

#ifdef WIN32
#include <windows.h>
#endif

#include "main.h"
#include "game.h"
#include "SDL_mixer.h"

class game *g = NULL;

bool enable_debug = false;
int debug_indention = 0;

bool sdl_initialized = false;
bool mix_opened = false;

#ifdef WIN32
char *msg_buf = NULL;
#endif

void vdprintf(const char *fmt, va_list args) {
  if ( enable_debug ) {
    for ( int i = 2 * debug_indention ; i > 0 ; --i )
      fputc(' ', stderr);

    vfprintf(stderr, fmt, args);
    fputc('\n', stderr);
  }
}

void dprintf(const char *buf, ...) {
  va_list v;
  va_start(v, buf);
  vdprintf(buf, v);
  va_end(v);
}

void denter() {
  debug_indention++;
}

void dleave() {
  debug_indention--;
}

void vshow_message(const char *fmt, va_list args) {
#ifdef WIN32
  char buf[1000];
  vsprintf(buf, fmt, args);
  if ( !msg_buf ) {
    msg_buf = (char *)malloc(strlen(buf) + 1);
    strcpy(msg_buf, buf);
  }
  else {
    char *tmp = (char *)malloc(strlen(msg_buf) + 1 + strlen(buf) + 1);
    strcpy(tmp, msg_buf);
    strcat(tmp, "\n");
    strcat(tmp, buf);
    free(msg_buf);
    msg_buf = tmp;
  }

#else
  vprintf(fmt, args);
#endif
}

void show_message(const char *buf, ...) {
  va_list v;
  va_start(v, buf);
  vshow_message(buf, v);
  va_end(v);
}

void cleanup() {
  if ( g ) {
    delete g;
    g = NULL;
  }

  if ( mix_opened ) {
    Mix_CloseAudio();
    mix_opened = false;
  }

  if ( sdl_initialized ) {
    SDL_Quit();
    sdl_initialized = false;
  }

#ifdef WIN32
  if ( msg_buf ) {
    int len = strlen(msg_buf) + 1;
    for ( char *s = msg_buf ; *s ; ++s )
      if ( *s == '\n' )
	len++;

    char *tmp = (char *)malloc(len);
    char *d = tmp;

    for ( char *s = msg_buf ; *s ; ++s ) {
      if ( *s == '\n' ) 
	*d++ = '\r';
      *d++ = *s;
    }	

    *d++ = 0;

    MessageBox(NULL, tmp, "Gamma Patrol", MB_ICONEXCLAMATION|MB_OK);
    free(tmp);

    free(msg_buf);
  }
#endif
}

void print_usage() {
  printf("GammaPatrol V"VERSION"\n"
	 "\n"
	 "Command line options:\n"
	 "\n"
	 "  --help          : print this help and exit\n"
	 "  --no-audio      : disable audio\n"
	 "  --audio         : enable audio\n"
	 "  --no-fullscreen : disable fullscreen display\n"
	 "  --fullscreen    : enable fullscreen display\n"
	 "  --no-joystick   : disable joystick support\n"
	 "  --joystick      : enable joystick support\n"
	 "  --no-debug      : disable debugging information\n"
	 "  --debug         : enable debugging information\n"
	 "\n");
}

int main(int argc, char **argv) {
  int start_level = -1;

  srand(time(NULL));

  atexit( cleanup );

  g = new game();
  g->read_settings();

  for ( int i = 1 ; i < argc ; ++i ) {
    if ( !strcmp(argv[i], "--no-audio") )
      g->enable_audio(false);
    else if ( !strcmp(argv[i], "--audio") )
      g->enable_audio(true);
    else if ( !strcmp(argv[i], "--fullscreen") )
      g->enable_fullscreen(true);
    else if ( !strcmp(argv[i], "--no-fullscreen") )
      g->enable_fullscreen(false);
    else if ( !strcmp(argv[i], "--no-debug") )
      enable_debug = false;
    else if ( !strcmp(argv[i], "--debug") )
      enable_debug = true;
    else if ( !strcmp(argv[i], "--no-joystick") )
      g->enable_joystick(false);
    else if ( !strcmp(argv[i], "--joystick") )
      g->enable_joystick(true);
    else if ( !strcmp(argv[i], "--help") ) {
      print_usage();
      exit(0);
    }
    else {
      fprintf(stderr, "Unknown options '%s'.\n\n", argv[i]);
      print_usage();
      exit(20);
    }
  }

  g->go(start_level);

  dprintf("game engine finished.");

  dprintf("cleanup:");
  denter();

  dprintf("deleting game...");
  if ( g ) {
    g->write_settings();
    delete g;
    g = NULL;
  }

  dprintf("done.");
  dleave();

  exit(0);
}
