
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include "main.h"
#include "game.h"
#include "object.h"
#include "object_bonus.h"
#include "object_player.h"

object::object() {
  _xpos = 0;
  _ypos = 0;
  _xvel = 0;
  _yvel = 0;
  _xacc = 0;
  _yacc = 0;

  _width = 0;
  _height = 0;

  _parts_count = 1;
  _parts = new part[_parts_count];
  _parts[0]._id = 0;
  _parts[0]._surface = NULL;

  for ( Uint16 t = 0 ; t < TRIGGER_COUNT ; ++t )
    _trigger[t]._delay = 0;

  _last_time = g->current_time();

  _layer = LAYER_ENEMIES;

  _hit_mask = 0;
  _type_mask = 0;
  
  _flags = 0;

  _delay = 0;
  _energy = 0;
  _bonus = BONUS_NONE;
  _score = 0;

  _auto_delete = true;
}

object::~object() {
  delete _parts;
}

void object::set_modifiers(svector<object_modifier> &tab) {
  for ( svector<object_modifier>::const_iterator i = tab.begin() ; i != tab.end() ; ++i ) {
    if ( i->name == "xpos" )
      xpos(i->val);
    else if ( i->name == "ypos" )
      ypos(i->val);
    else if ( i->name == "xvel" )
      xvel(i->val);
    else if ( i->name == "yvel" )
      yvel(i->val);
    else if ( i->name == "xacc" )
      xacc(i->val);
    else if ( i->name == "yacc" )
      yacc(i->val);
    else if ( i->name == "energy" )
      energy(i->val);
    else if ( i->name == "score" )
      score(i->val);
  }
}

object::part *object::add_part(Uint32 id) {
  // find free part slot
  part *p = _parts;
  part *pend = _parts + _parts_count;
  while ( p < pend ) {
    if ( p->_id == 0 )
      break;
    ++p;
  }

  // expand list, if no free slot was found
  if ( p >= pend ) {
    Uint16 nc = _parts_count + 1;
    part *np = new part[nc];
    for ( Uint16 i = 0 ; i < _parts_count ; ++i )
      np[i] = _parts[i];
    for ( Uint16 i = _parts_count ; i < nc ; ++i ) {
      np[i]._id = 0;
      np[i]._surface = NULL;
    }

    delete _parts; 
    _parts = np;
    p = _parts + nc - 1;
    _parts_count = nc;
  }

  // calculate unique id, if not geiven explicitly
  if ( id == 0 ) {
    for ( part *pend = _parts + _parts_count, *p = _parts ; p < pend ; ++p )
      if ( p->_id >= id )
	id = p->_id;

    ++id;
  }

  // initialize part structure
  p->_id = id;

  p->_surface = NULL;
  p->_xoffset = 0;
  p->_yoffset = 0;
  p->_layer = 0;

  p->_enable_collision = true;
  p->_enable_paint = true;

  p->_frames_per_second = 15;
  p->_frames_reverse = false;
  p->_frames_done = 0;
  p->_frames_offset = 0;
  p->_frames_count = 1;
  p->_frames_loops = -1;
  p->_frames_start = g->current_time();

  p->_phase_id = 0;

  return( p );
}

object::part *object::rem_part(Uint32 id) {
  part *p = get_part(id);
  if ( p ) {
    //delete p->_surface;
    p->_surface = NULL;
    p->_id = 0;

    update_size();
  }

  return( p );
}

object::part *object::get_part(Uint32 id) {
  for ( part *pend = _parts + _parts_count, *p = _parts ; p < pend ; ++p )
    if ( p->_id == id )
      return( p );

  return( NULL );
}

void object::finish_phase(part *p) {
  p->_frames_start = g->current_time();
  p->_frames_done = 0;
}

void object::update_size() {
  // recalculate width & height
  _lbound = 0, _rbound = 0, _tbound = 0, _bbound = 0;
  for ( part *pend = _parts + _parts_count, *p = _parts ; p < pend ; ++p )
    if ( p->_id != 0 ) {
      if ( p->_xoffset < _lbound )
	_lbound = p->_xoffset;
      if ( p->_yoffset < _tbound )
	_tbound = p->_yoffset;
      if ( p->_xoffset + p->_surface->width() > _rbound )
	_rbound = p->_xoffset + p->_surface->width();
      if ( p->_yoffset + p->_surface->height() > _bbound )
	_bbound = p->_yoffset + p->_surface->height();
    }

  _width = _rbound - _lbound;
  _height = _bbound - _tbound;
}

Uint16 object::get_trigger() const {
  for ( Uint16 i = 0 ; i < TRIGGER_COUNT ; ++i )
    if ( _trigger[i]._delay == 0 ) 
      return( i );

  return(0xffff);
}

void object::set_trigger(Uint16 id, Uint32 delay) {
  _trigger[id]._start = g->current_time();
  _trigger[id]._delay = delay;
}

void object::unset_trigger(Uint16 id) {
  _trigger[id]._delay = 0;
}

void object::handle_trigger(Uint16 id) {
  _trigger[id]._delay = 0;
}

Uint16 object::current_frame(part *p) const {
  return( p->_frames_offset + p->_frames_done % p->_frames_count );
}

void object::initialize() {
  _last_time = g->current_time();

  for ( part *pend = _parts + _parts_count, *p = _parts ; p < pend ; ++p )
    if ( p->_id != 0 ) 
      p->_frames_start = g->current_time();
}

void object::move() {
  Sint32 dt = g->current_time() - _last_time;
  if ( dt > 0 ) {
    _xpos += (Sint32)_xacc * dt * dt / 1000000 + (Sint32)_xvel * dt / 1000;
    _ypos += (Sint32)_yacc * dt * dt / 1000000 + (Sint32)_yvel * dt / 1000;
    _xvel += (Sint32)_xacc * dt / 1000;
    _yvel += (Sint32)_yacc * dt / 1000;

    if ( _flags & OBJECT_INVISIBLE ) {
      if ( _delay == 0 )
	_flags &= ~OBJECT_INVISIBLE;
      else {
	if ( _delay >= dt )
	  _delay -= dt;
	else
	  _delay = 0;
      }
    }

    _last_time = g->current_time();
  }

  for ( part *pend = _parts + _parts_count, *p = _parts ; p < pend ; ++p )
    if ( p->_id != 0 ) {
      p->_frames_done = p->_frames_per_second * (g->current_time() - p->_frames_start) / 1000;
      if ( (p->_frames_loops != -1) && (p->_frames_done >= p->_frames_loops * p->_frames_count) ) {
	finish_phase(p);
      }
    }

  for ( Uint16 t = 0 ; t < TRIGGER_COUNT ; ++t ) {
    if ( (_trigger[t]._delay != 0) && (g->current_time() - _trigger[t]._start >= _trigger[t]._delay) )
      handle_trigger(t);
  }
}

void object::paint() {
  Sint16 max_layer = -10000;
  Sint16 min_layer = 10000;

  for ( part *pend = _parts + _parts_count, *p = _parts ; p < pend ; ++p ) {
    if ( (p->_id != 0) && p->_enable_paint ) {
      if ( p->_layer > max_layer )
	max_layer = p->_layer;

      if ( p->_layer < min_layer )
	min_layer = p->_layer;
    }
  }
  
  for ( Sint16 l = min_layer ; l <= max_layer ; ++l ) {
    for ( part *pend = _parts + _parts_count, *p = _parts ; p < pend ; ++p )
      if ( (p->_id != 0) && p->_enable_paint && (p->_layer == l) )
	g->draw_bob(((fxpos() - g->fxpos()) >> 8) + p->_xoffset, ((fypos() - g->fypos()) >> 8) + p->_yoffset, current_frame(p), p->_surface);
  }
}

bool object::check_collision(object *o) {
  Sint16 max_layer1 = -10000;
  Sint16 min_layer1 = 10000;

  for ( part *p1end = _parts + _parts_count, *p1 = _parts ; p1 < p1end ; ++p1 ) {
    if ( (p1->_id != 0) && p1->_enable_collision ) {
      if ( p1->_layer > max_layer1 )
	max_layer1 = p1->_layer;
      if ( p1->_layer < min_layer1 )
	min_layer1 = p1->_layer;
    }
  }

  Sint16 max_layer2 = -10000;
  Sint16 min_layer2 = 10000;

  for ( part *p2end = o->_parts + o->_parts_count, *p2 = o->_parts ; p2 < p2end ; ++p2 ) {
    if ( (p2->_id != 0) && p2->_enable_collision ) {
      if ( p2->_layer > max_layer2 )
	max_layer2 = p2->_layer;
      if ( p2->_layer < min_layer2 )
	min_layer2 = p2->_layer;
    }
  }

  for ( Sint16 l1 = max_layer1 ; l1 >= min_layer1 ; --l1 ) {
    for ( part *p1end = _parts + _parts_count, *p1 = _parts ; p1 < p1end ; ++p1 ) {
      if ( (p1->_id != 0) && p1->_enable_collision && (p1->_layer == l1) ) {
	for ( Sint16 l2 = max_layer2 ; l2 >= min_layer2 ; --l2 ) {
	  for ( part *p2end = o->_parts + o->_parts_count, *p2 = o->_parts ; p2 < p2end ; ++p2 ) {
	    if ( (p2->_id != 0) && p2->_enable_collision && (p2->_layer == l2) ) {
	      Sint16 l1 = xpos() + p1->_xoffset;
	      Sint16 r1 = l1 + p1->_surface->width() - 1;
	      Sint16 t1 = ypos() + p1->_yoffset;
	      Sint16 b1 = t1 + p1->_surface->height() - 1;

	      Sint16 l2 = o->xpos() + p2->_xoffset;
	      Sint16 r2 = l2 + p2->_surface->width() - 1;
	      Sint16 t2 = o->ypos() + p2->_yoffset;
	      Sint16 b2 = t2 + p2->_surface->height() - 1;

	      if ( (t2 <= b1) && (b2 >= t1) && (l2 <= r1) && (r2 >= l1) && 
		   p1->_surface->check_intersection(p2->_surface, current_frame(p1), o->current_frame(p2), l2 - l1, t2 - t1) ) {
		Uint32 e1 = energy();
		Uint32 e2 = o->energy();

		hit_by(o);
		o->hit_by(this);
		hit(o);
		o->hit(this);

		damage(p1, e2);
		o->damage(p2, e1);

		return( true );
	      }
	    }
	  }
	}
      }
    }
  }

  return( false );
}

bool object::damage(part *, Uint32 e) {
  bool destroyed = false;

  if ( e >= energy() ) {
    energy(0);
    destroy();
    destroyed = true;
  }
  else
    energy(energy() - e);

  return( destroyed );
}

void object::hit_by(object *) {
}

void object::hit(object *) {
}

void object::destroy() {
  _flags |= OBJECT_DELETED;

  if ( this != (object *)g->player() ) {
    object *o = NULL;

    switch ( _bonus ) {
    case BONUS_ENERGY :
      o = new object_bonus_energy();
      break;
    case BONUS_SHIELD :
      o = new object_bonus_shield();
      break;
    case BONUS_LIFE :
      o = new object_bonus_life();
      break;
    case BONUS_PLASMA :
      o = new object_bonus_plasma();
      break;
    case BONUS_LASER :
      o = new object_bonus_laser();
      break;
    case BONUS_BALL :
      o = new object_bonus_ball();
      break;
    case BONUS_SEEKER :
      o = new object_bonus_seeker();
      break;
    }

    if ( o ) {
      o->xpos(xpos());
      o->ypos(ypos());
      o->yvel(30);
      g->add_object(o);
    }

    g->player()->add_score(_score);
  }
}

bool object::setup() {
  if ( object_bonus_energy::setup() &&
       object_bonus_life::setup() &&
       object_bonus_shield::setup() &&
       object_bonus_ball::setup() &&
       object_bonus_plasma::setup() &&
       object_bonus_laser::setup() &&
       object_bonus_seeker::setup() ) {
    return( true );
  }

  return( false );
}

void object::cleanup() {
  object_bonus_energy::cleanup();
  object_bonus_life::cleanup();
  object_bonus_shield::cleanup();
  object_bonus_ball::cleanup();
  object_bonus_plasma::cleanup();
  object_bonus_laser::cleanup();
  object_bonus_seeker::cleanup();
}
