
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object.h,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#ifndef _OBJECT_H_
#define _OBJECT_H_

#include "main.h"
#include "sstring.h"
#include "svector.h"

#define OBJECT_PLAYER_SHIP         0x00000001
#define OBJECT_PLAYER_PROJECTILE   0x00000002
#define OBJECT_ENEMY_SHIP          0x00000004
#define OBJECT_ENEMY_PROJECTILE    0x00000008
#define OBJECT_BONUS               0x00000010
#define OBJECT_NEUTRAL             0x00000020

#define OBJECT_DELETED             0x00000001
#define OBJECT_INVISIBLE           0x00000002

#define LAYER_ENEMIES              0
#define LAYER_PROJECTILES          1
#define LAYER_PLAYER               2
#define LAYER_EXPLOSIONS           3

#define TRIGGER_COUNT              5

struct object_modifier {
  sstring name;
  Sint32 val;
};


class object {
 protected:
  Sint32 _xpos, _ypos;
  Sint32 _xvel, _yvel;
  Sint32 _xacc, _yacc;
  Sint16 _width, _height;
  Sint16 _lbound, _rbound, _tbound, _bbound;

  Uint32 _last_time;

  struct part {
    Uint32 _id;

    class surface *_surface;
    Sint16 _xoffset, _yoffset;
    Sint16 _layer;

    bool _enable_collision;
    bool _enable_paint;

    Uint16 _frames_offset;
    Uint16 _frames_count;
    Uint16 _frames_per_second;
    bool _frames_reverse;
    Sint16 _frames_loops;
    Uint16 _frames_done;
    Sint32 _frames_start;

    Sint16 _phase_id;
  };

  struct part *_parts;
  Uint16 _parts_count;

  part *add_part(Uint32);
  part *rem_part(Uint32);
  part *get_part(Uint32);
  virtual void finish_phase(part *);

  struct trigger {
    Uint32 _start;
    Uint32 _delay;
  };

  struct trigger _trigger[TRIGGER_COUNT];
  
  Uint16 get_trigger() const;
  void set_trigger(Uint16, Uint32);
  void unset_trigger(Uint16);
  virtual void handle_trigger(Uint16);

  Uint16 _layer;

  bool _auto_delete;

  Uint32 _hit_mask;
  Uint32 _type_mask;

  Uint32 _bonus;
  Uint32 _energy;
  Uint32 _score;

  Uint32 _flags;
  Sint32 _delay;

 public:
  object();
  virtual ~object();

  virtual void set_modifiers(svector<object_modifier> &);

  Sint32 fxpos() const { return( _xpos ); }
  Sint32 xpos() const { return( _xpos >> 8 ); }
  void xpos(Sint32 x) { _xpos = x << 8; }
  Sint32 fypos() const { return( _ypos ); }
  Sint32 ypos() const { return( _ypos >> 8 ); }
  void ypos(Sint32 y) { _ypos = y << 8; }
  Sint32 fxvel() const { return( _xvel ); }
  Sint32 xvel() const { return( _xvel >> 8 ); }
  void xvel(Sint32 x) { _xvel = x << 8; }
  Sint32 fyvel() const { return( _yvel ); }
  Sint32 yvel() const { return( _yvel >> 8 ); }
  void yvel(Sint32 y) { _yvel = y << 8; }
  Sint32 fxacc() const { return( _xacc ); }
  Sint32 xacc() const { return( _xacc >> 8 ); }
  void xacc(Sint32 x) { _xacc = x << 8; }
  Sint32 fyacc() const { return( _yacc ); }
  Sint32 yacc() const { return( _yacc >> 8 ); }
  void yacc(Sint32 y) { _yacc = y << 8; }

  Sint16 width() const { return( _width ); }
  Sint16 height() const { return( _height ); }
  Sint16 lbound() const { return( _lbound ); }
  Sint16 rbound() const { return( _rbound ); }
  Sint16 tbound() const { return( _tbound ); }
  Sint16 bbound() const { return( _bbound ); }
  void update_size();

  bool auto_delete() const { return( _auto_delete ); }
  void auto_delete(bool a) { _auto_delete = a; }

  Uint16 layer() const { return( _layer ); }
  Uint32 hit_mask() const { return( _hit_mask ); }
  void hit_mask(Uint32 m) { _hit_mask = m; }
  Uint32 type_mask() const { return( _type_mask ); }
  void type_mask(Uint32 m) { _type_mask = m; }

  Uint32 energy() const { return( _energy ); }
  void energy(Uint32 e) { _energy = e; }
  Uint32 bonus() const { return( _bonus ); }
  void bonus(Uint32 e) { _bonus = e; }
  Uint32 score() const { return( _score ); }
  void score(Uint32 e) { _score = e; }

  Uint32 flags() const { return( _flags ); }
  void flags(Uint32 f) { _flags = f; }
  bool visible() const { return( (_flags & (OBJECT_DELETED | OBJECT_INVISIBLE)) ? false : true); }

  Uint32 delay() const { return( _delay ); }
  void delay(Uint32 d) { _delay = d; if ( d > 0 ) _flags |= OBJECT_INVISIBLE; }

  virtual void initialize();
  virtual Uint16 current_frame(part *) const;
  virtual void move();
  virtual bool check_collision(object *);
  virtual void paint();

  virtual bool damage(part *, Uint32);
  virtual void hit_by(object *);
  virtual void hit(object *);

  virtual void destroy();

  static bool setup();
  static void cleanup();
};

#endif





