
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_aster.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include "main.h"
#include "game.h"
#include "object_aster.h"
#include "object_expl.h"

/***********************************************************
 *                                                         *
 * class object_aster1                                     *
 *                                                         *
 *                                                         *
 ***********************************************************/

//$TAB { "aster1", object_aster1::constructor, object_aster1::setup, object_aster1::cleanup },

Uint16 object_aster1::_image_id = 0xffff;

object_aster1::object_aster1() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  p->_frames_per_second = (rand() % 5) + 16;
  p->_frames_reverse = (rand() & 16) ? true : false;
  update_size();

  xpos( rand() % g->map_width() );
  ypos( 0 );
  xvel( 0 );
  yvel( 45 );
  xacc( 0 );
  yacc( 0 );

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP | OBJECT_PLAYER_PROJECTILE;
  _type_mask = OBJECT_ENEMY_SHIP;

  _energy = 300;
  _score = 300;
}

void object_aster1::destroy() {
  object::destroy();

  g->play_expl_sound(SOUND_EXPL_4, SOUND_LAYER_ENEMY);

  for ( int i = 0 ; i < 2 ; ++i ) {
    object *o = new object_aster2();
    o->xpos(xpos());
    o->ypos(ypos());
    o->xvel((30 + (rand() % 45)) * ((rand() & 16) ? -1 : 1));
    o->yvel(20 + (rand() % 45));
    g->add_object(o);
  }

  object *o = new object_expl1();
  o->xpos(xpos());
  o->ypos(ypos());
  o->xvel(xvel());
  o->yvel(yvel());
  g->add_object(o);
}

object *object_aster1::constructor() {
  return( new object_aster1() );
}

bool object_aster1::setup() {
  if ( object::setup() &&
       object_expl1::setup() &&
       object_aster2::setup() ) {
    _image_id = g->obtain_bob("aster1");
    if ( _image_id != 0xffff ) {
      return( true );
    }
  }

  return( false );
}

void object_aster1::cleanup() {
  g->release_bob(_image_id);

  object_aster2::cleanup();
  object_expl1::cleanup();
  object::cleanup();
}

/***********************************************************
 *                                                         *
 * class object_aster2                                     *
 *                                                         *
 ***********************************************************/

//$TAB { "aster2", object_aster2::constructor, object_aster2::setup, object_aster2::cleanup },

Uint16 object_aster2::_image_id = 0xffff;

object_aster2::object_aster2() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  p->_frames_per_second = (rand() % 5) + 12;
  p->_frames_reverse = (rand() & 16) ? true : false;
  update_size();

  xpos( rand() % g->map_width() );
  ypos( 0 );
  xvel( 0 );
  yvel( 45 );
  xacc( 0 );
  yacc( 0 );

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP | OBJECT_PLAYER_PROJECTILE;
  _type_mask = OBJECT_ENEMY_SHIP;

  _energy = 120;
  _score = 150;
}

void object_aster2::destroy() {
  object::destroy();

  g->play_expl_sound(SOUND_EXPL_2, SOUND_LAYER_ENEMY, 64);

  for ( int i = 0 ; i < 3 ; ++i ) {
    object *o = new object_aster3();
    o->xpos(xpos());
    o->ypos(ypos());
    o->xvel((60 + (rand() % 30)) * ((rand() & 16) ? -1 : 1));
    o->yvel(60 + (rand() % 30));
    g->add_object(o);
  }

  object *o = new object_expl2();
  o->xpos(xpos());
  o->ypos(ypos());
  o->xvel(xvel());
  o->yvel(yvel());
  g->add_object(o);
}

object *object_aster2::constructor() {
  return( new object_aster2() );
}

bool object_aster2::setup() {
  if ( object::setup() &&
       object_expl2::setup() && 
       object_aster3::setup() ) {
    _image_id = g->obtain_bob("aster2");
    if ( _image_id != 0xffff ) {
      return( true );
    }
  }

  return( false );
}

void object_aster2::cleanup() {
  g->release_bob(_image_id);
  
  object_aster3::cleanup();
  object_expl2::cleanup();
}

/***********************************************************
 *                                                         *
 * class object_aster3                                     *
 *                                                         *
 ***********************************************************/

//$TAB { "aster3", object_aster3::constructor, object_aster3::setup, object_aster3::cleanup },

Uint16 object_aster3::_image_id = 0xffff;

object_aster3::object_aster3() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  p->_frames_per_second = (rand() % 3) + 9;
  p->_frames_reverse = (rand() & 16) ? true : false;
  update_size();

  xpos( rand() % g->map_width() );
  ypos( 0 );
  xvel( 0 );
  yvel( 45 );
  xacc( 0 );
  yacc( 0 );

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP | OBJECT_PLAYER_PROJECTILE;
  _type_mask = OBJECT_ENEMY_SHIP;

  _energy = 100;
  _score = 50;
}

void object_aster3::destroy() {
  object::destroy();

  g->play_expl_sound(SOUND_EXPL_3, SOUND_LAYER_ENEMY, 32);

  object *o = new object_expl3();
  o->xpos(xpos());
  o->ypos(ypos());
  o->xvel(xvel());
  o->yvel(yvel());
  g->add_object(o);
}

object *object_aster3::constructor() {
  return( new object_aster3() );
}

bool object_aster3::setup() {
  if ( object::setup() &&
       object_expl3::setup() ) {
    _image_id = g->obtain_bob("aster3");
    if ( _image_id != 0xffff ) {
      return( true );
    }
  }

  return( false );
}

void object_aster3::cleanup() {
  g->release_bob(_image_id);

  object_expl3::cleanup();
  object::cleanup();
}

