
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_bigsaucer.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include "main.h"
#include "game.h"
#include "object_bigsaucer.h"
#include "object_expl.h"
#include "object_smoke.h"
#include "object_missile.h"

//$TAB { "bigsaucer", object_bigsaucer::constructor, object_bigsaucer::setup, object_bigsaucer::cleanup },

Uint16 object_bigsaucer::_image_id = 0xffff;

Sint16 object_bigsaucer::_leg_pos_x[3] = { -8, 61, -47 };
Sint16 object_bigsaucer::_leg_pos_y[3] = { -15, -15, -16 };
Sint16 object_bigsaucer::_hole_pos_x[3] = { -2, 94, -79 };
Sint16 object_bigsaucer::_hole_pos_y[3] = { -16, -38, -43 };

object_bigsaucer::object_bigsaucer() : object() {
  surface *b = g->get_bob(_image_id);

  part *p = add_part(1);
  p->_surface = b;
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_layer = 0;
  p->_frames_count = 1;
  p->_frames_offset = 0;
  p->_frames_per_second = 10;

  p = add_part(10);
  p->_surface = b;
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_layer = 1;
  p->_frames_count = 1;
  p->_frames_offset = 1;
  p->_frames_per_second = 10;

  p = add_part(11);
  p->_surface = b;
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_layer = 1;
  p->_frames_count = 1;
  p->_frames_offset = 2;
  p->_frames_per_second = 10;

  p = add_part(12);
  p->_surface = b;
  p->_layer = 1;
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = 1;
  p->_frames_offset = 3;
  p->_frames_per_second = 10;
  
  update_size();

  _move_phase = 0;
  _leg_phase = 0;
  _attractor_x = g->map_width() / 2;
  _attractor_y = g->ypos() + MAIN_HEIGHT / 2;

  xpos( g->map_width() - lbound() );
  ypos( g->ypos() + 50 + b->height() / 2 );
  xvel( 0 );
  yvel( 0 );
  xacc( 0 );
  yacc( 0 );

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP | OBJECT_PLAYER_PROJECTILE;
  _type_mask = OBJECT_ENEMY_SHIP;

  _energy = 5000;
  _leg_energy[0] = 1400;
  _leg_energy[1] = 1400;
  _leg_energy[2] = 1400;

  _signal = 0x00000001;

  auto_delete(false);
}

void object_bigsaucer::set_modifiers(svector<object_modifier> &tab) {
  for ( svector<object_modifier>::const_iterator i = tab.begin() ; i != tab.end() ; ++i ) {
    if ( i->name == "signal" )
      _signal = i->val;
  }

  object::set_modifiers(tab);
}

void object_bigsaucer::initialize() {
  object::initialize();

  _next_missiles = g->current_time() + 8000;

  g->bigenemy_init(_energy + _leg_energy[0] + _leg_energy[1] + _leg_energy[2]);
}

void object_bigsaucer::finish_phase(part *p) {
  if ( (p->_id == 10) || (p->_id == 11) || (p->_id == 12) ) {
    switch ( p->_phase_id ) {
    case 0 :
      p->_frames_count = 16;
      p->_frames_offset = 7 + 16 * (p->_id - 10);
      p->_frames_loops = 1;

      p->_phase_id = 1;
      break;

    case 1 :
      p->_frames_count = 1;
      p->_frames_offset = 1 + p->_id - 10;
      p->_frames_loops = -1;

      p->_phase_id = 0;
      break;
    }
  }

  object::finish_phase(p);
}

void object_bigsaucer::move() {
  switch ( _leg_phase ) {
  case 0 :
    break;

  case 1 :
    if ( rand() % 100 == 0 ) {
      bool yes = true;

      for ( int i = 10 ; i <= 12 ; ++i ) {
	part *p = get_part(i);
	if ( p && (p->_phase_id != 0) )
	  yes = false;
      }

      if ( yes )
	_leg_phase = 2;
    }
    break;

  case 2 :
    for ( int i = 10 ; i <= 12 ; ++i ) {
      part *p = get_part(i);
      if ( p )
	finish_phase(p);
    }

    _leg_phase = 1;
    break;
  }

  switch ( _move_phase ) {
  case 0 :
    if ( xpos() > g->xpos() + MAIN_WIDTH / 2 ) {
      Sint16 dx = _attractor_x - xpos();
      Sint16 dy = _attractor_y - ypos();

      if ( xvel() < -100 ) {
	xacc(0);
	xvel(-100);
      }
      else if ( xvel() > 100 ) {
	xacc(0);
	xvel(100);
      }
      else
	xacc(-150 >? dx <? 150);

      if ( yvel() < -100 ) {
	yacc(0);
	yvel(-100);
      }
      else if ( yvel() > 100 ) {
	yacc(0);
	yvel(100);
      }
      else
	yacc(-150 >? dy <? 150);
    }
    else {
      _move_phase = 1;
      _leg_phase = 1;
      _attractor_x = g->xpos() + MAIN_WIDTH / 4 + (rand() % (MAIN_WIDTH / 2));
      _attractor_y = g->ypos() + MAIN_HEIGHT / 4 + (rand() % (5 * MAIN_HEIGHT / 8));
    }
    break;

  case 1 : {
    Sint16 dx = _attractor_x - xpos();
    Sint16 dy = _attractor_y - ypos();

    if ( dx * dx + dy * dy < 5000 ) {
      _attractor_x = g->xpos() + MAIN_WIDTH / 4 + (rand() % (MAIN_WIDTH / 2));
      _attractor_y = g->ypos() + MAIN_HEIGHT / 4 + (rand() % (5 * MAIN_HEIGHT / 8));
    }

    if ( xpos() < MAIN_WIDTH / 4 )
      xacc(180);
    else if ( xpos() > g->map_width() - MAIN_WIDTH / 4 )
      xacc(-180);
    else if ( xvel() < -100 ) {
      xacc(0);
      xvel(-100);
    }
    else if ( xvel() > 100 ) {
      xacc(0);
      xvel(100);
    }
    else
      xacc(-150 >? dx <? 150);

    if ( ypos() < g->ypos() + MAIN_HEIGHT / 4 )
      yacc(180);
    else if ( ypos() > g->ypos() + 7 * MAIN_HEIGHT / 8 )
      yacc(-180);
    else if ( yvel() < -100 ) {
      yacc(0);
      yvel(-100);
    }
    else if ( yvel() > 100 ) {
      yacc(0);
      yvel(100);
    }
    else
      yacc(-150 >? dy <? 150);
    break;
  }

  case 2 :
    break;

  case 1000 :
    if ( rand() % 5 == 0 ) {
      g->play_expl_sound(SOUND_EXPL_2, SOUND_LAYER_ENEMY);

      object *o = new object_expl1();
      o->xpos(xpos() + (rand() % 212) - 106);
      o->ypos(ypos() + (rand() % 102) - 112);
      o->xvel(0);
      o->yvel(0);
      g->add_object(o);
    }
    break;
  }

  for ( Uint16 i = 0 ; i < 3 ; ++i ) {
    if ( (_leg_energy[i] == 0) && (g->current_time() > _next_smoke[i]) ) {
      _next_smoke[i] = g->current_time() + 500 + rand() % 500;

      object *o = (rand() % 4 == 0) ? new object_smoke2() : new object_smoke3();
      o->xpos(xpos() + _hole_pos_x[i] + (rand() % 21) - 10);
      o->ypos(ypos() + _hole_pos_y[i] + (rand() % 21) - 10);
      o->xvel(0);
      o->yvel(0);
      g->add_object(o);
    }
  }

  if ( (_energy > 0) && (g->current_time() > _next_missiles) ) {
    _next_missiles = g->current_time() + 6000;

    object_missile *o = new object_missile();
    o->xpos(xpos());
    o->ypos(ypos());
    o->dir(0);
    g->add_object(o);

    o = new object_missile();
    o->xpos(xpos() - 30);
    o->ypos(ypos());
    o->dir(450);
    g->add_object(o);

    o = new object_missile();
    o->xpos(xpos() + 30);
    o->ypos(ypos());
    o->dir(315);
    g->add_object(o);
  }

  object::move();
}

bool object_bigsaucer::damage(part *p, Uint32 e) {
  switch ( p->_id ) {
  case 1 :
    if ( (_leg_energy[0] == 0) && (_leg_energy[1] == 0) && (_leg_energy[2] == 0) ) {
      if ( e >= _energy ) {
	_energy = 0;
	_hit_mask = 0;

	g->player()->add_score_to_depot(47435);

	g->bigenemy_cleanup();
	g->add_signal(_signal);

	yacc(150);
	//if ( xpos() > g->xpos() + MAIN_WIDTH / 2 )
	//xvel(-xpos() / 3);
	//else
	//xvel((g->xpos() + MAIN_WIDTH - xpos()) / 3);

	_move_phase = 1000;
	auto_delete(true);
      }
      else
	_energy -= e;
    }
    break;

  case 10 :
  case 11 :
  case 12 :
    if ( e >= _leg_energy[p->_id - 10] ) {
      _leg_energy[p->_id - 10] = 0;
      _next_smoke[p->_id - 10] = g->current_time() + 1000;

      g->play_expl_sound(SOUND_EXPL_1, SOUND_LAYER_ENEMY);

      Sint16 x = xpos() + _leg_pos_x[p->_id - 10];
      Sint16 y = ypos() + _leg_pos_y[p->_id - 10];

      for ( int i = 0 ; i < 3 ; ++i ) {
	object *o = new object_expl2();
	o->xpos(x);
	o->ypos(y);
	o->xvel((40 + (rand() % 100)) * ((rand() & 16) ? -1 : 1));
	o->yvel((40 + (rand() % 100)) * ((rand() & 16) ? -1 : 1));
	g->add_object(o);
      }

      for ( int i = 0 ; i < 8 ; ++i ) {
	object *o = new object_expl3();
	o->xpos(x);
	o->ypos(y);
	o->xvel((75 + (rand() % 150)) * ((rand() & 16) ? -1 : 1));
	o->yvel((75 + (rand() % 150)) * ((rand() & 16) ? -1 : 1));
	g->add_object(o);
      }

      p->_frames_count = 1;
      p->_frames_offset = 4 + p->_id - 10;
      p->_frames_per_second = 10;
      p->_frames_loops = -1;
      p->_enable_collision = false;
      p->_id += 10;
    }
    else
      _leg_energy[p->_id - 10] -= e;
    break;
  }

  g->bigenemy_update(_energy + _leg_energy[0] + _leg_energy[1] + _leg_energy[2]);

  return( false );
}

object *object_bigsaucer::constructor() {
  return( new object_bigsaucer() );
}

bool object_bigsaucer::setup() {
  if ( object::setup() &&
       object_missile::setup() &&
       object_smoke2::setup() &&
       object_smoke3::setup() &&
       object_expl1::setup() &&
       object_expl2::setup() &&
       object_expl3::setup() ) {
    _image_id = g->obtain_bob("bsau");
    if ( _image_id != 0xffff )
      return( true );
  }

  return( false );
}

void object_bigsaucer::cleanup() {
  g->release_bob(_image_id);

  object_missile::cleanup();
  object_smoke3::cleanup();
  object_smoke2::cleanup();
  object_expl3::cleanup();
  object_expl2::cleanup();
  object_expl1::cleanup();
  object::cleanup();
}
