
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_blocker.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "game.h"
#include "object_blocker.h"
#include "object_expl.h"
#include "object_player.h"
#include "object_shot.h"

/************************************************************************
 *                                                                      *
 * class object_blocker                                                 *
 *                                                                      *
 ************************************************************************/

//$TAB { "blocker", object_blocker::constructor, object_blocker::setup, object_blocker::cleanup },

Uint16 object_blocker::_image_id = 0xffff;

object_blocker::object_blocker() : object() {
  part *p = add_part(1);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _auto_delete = false;

  _dir = 0;
  _group_id = 0;
  _group_member = 0;

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP | OBJECT_PLAYER_PROJECTILE;
  _type_mask = OBJECT_ENEMY_SHIP;

  _energy = 2000;
  _score = 4000;

  _phase = 0;
  _group = NULL;
}

object_blocker::~object_blocker() {
  if ( _group )
    _group->rem_object(this);
}

void object_blocker::set_modifiers(svector<object_modifier> &tab) {
  for ( svector<object_modifier>::const_iterator i = tab.begin() ; i != tab.end() ; ++i ) {
    if ( i->name == "group" )
      _group_id = i->val;
    else if ( i->name == "member" )
      _group_member = i->val;
  }

  object::set_modifiers(tab);
}

void object_blocker::handle_trigger(Uint16 id) {
  if ( id == _shot_trigger ) {
    _shot_trigger = get_trigger();
    set_trigger(_shot_trigger, (4000 + rand() % 3000));

    if ( _shot_enabled ) {
      object *o = new object_shot(1);
      o->xpos(xpos());
      o->ypos(ypos());
      o->energy(50);
      g->add_object(o);
    }
  }

  object::handle_trigger(id);
}

Uint16 object_blocker::current_frame(part *p) const {
  Sint16 f;

  switch ( p->_id ) {
  case 1 :
    f = (p->_frames_count * (_dir >> 8) / 360) % p->_frames_count;
    break;

  default :
    f = object::current_frame(p);
    break;
  }

  return( f );
}

void object_blocker::move() {
  const Sint32 DVEL = 100 << 8;
  const Sint32 VEL = 200;

  _shot_enabled = false;

  switch ( _phase ) {
  case 0 : {
    double target_x = _group->xpos() + 7*g->map_width()/16 * sin(M_PI * (_group->angle() + (360<<8)/16 * _group_member) / (180.0 * 256));
    double target_y = _group->ypos() + MAIN_HEIGHT/3 * cos(M_PI * (_group->angle() + (360<<8)/16 * _group_member) / (180.0 * 256));

    double dx = target_x - xpos();
    double dy = target_y - ypos();
    if ( dx*dx + dy*dy < 4*4 )
      _phase = 1;

    double d;
  
    if ( dy > 0 )
      d = atan(-dx / dy) + 2 * M_PI;
    else if ( dy < 0 )
      d = atan(-dx / dy) + M_PI;
    else if ( dx > 0 )
      d = 1.5 * M_PI;
    else
      d = 0.5 * M_PI;

    _dir = ((Sint32)(180.0 * d / M_PI) << 8) % (360 << 8);

    xvel((Sint16)(-150 * sin(d)));
    yvel((Sint16)(150 * cos(d)));
    break;
  }

  case 1 : {
    double target_x = _group->xpos() + 7*g->map_width()/16 * sin(M_PI * (_group->angle() + (360<<8)/16 * _group_member) / (180.0 * 256));
    double target_y = _group->ypos() + MAIN_HEIGHT/3 * cos(M_PI * (_group->angle() + (360<<8)/16 * _group_member) / (180.0 * 256));

    if ( _group->ypos() > g->ypos() + 4*MAIN_HEIGHT/3 )
      _phase = 2;

    xpos(target_x);
    ypos(target_y);
    xvel(0);
    yvel(0);

    Sint32 dvel = 0;
    if ( g->player()->visible() ) {
      double dx = g->player()->xpos() - xpos();
      double dy = g->player()->ypos() - ypos();

      double d;
  
      if ( dy > 0 )
	d = atan(-dx / dy) + 2 * M_PI;
      else if ( dy < 0 )
	d = atan(-dx / dy) + M_PI;
      else if ( dx > 0 )
	d = 1.5 * M_PI;
      else
	d = 0.5 * M_PI;

      Sint16 nd = (Sint16)(180.0 * d / M_PI) % 360;
      Sint16 od = (_dir >> 8) % 360;

      if ( abs(nd - od) > 5 ) {
	if ( nd > od ) {
	  if ( nd - od < 180 )
	    dvel = DVEL;
	  else
	    dvel = -DVEL;
	}
	else if ( nd < od ) {
	  if ( od - nd < 180 )
	    dvel = -DVEL;
	  else
	    dvel = DVEL;
	}
      }
      else
	_shot_enabled = true;
    }
    else
      dvel = DVEL;

    _dir += (Sint32)(g->current_time() - _last_time) * dvel / 1000;
    while ( _dir < 0 )
      _dir += 360 << 8;
    break;
  }

  case 2 : {
    Sint32 target_x = (_group_member & 0x1) ? -200 : (Sint32)(g->map_width() + 200);
    Sint32 target_y = g->ypos() + MAIN_HEIGHT - MAIN_HEIGHT * _group_member/2 / 8;

    double dx = target_x - xpos();
    double dy = target_y - ypos();
    double d;
  
    if ( dx * dx + dy * dy < 16*16 )
      _auto_delete = true;

    if ( dy > 0 )
      d = atan(-dx / dy) + 2 * M_PI;
    else if ( dy < 0 )
      d = atan(-dx / dy) + M_PI;
    else if ( dx > 0 )
      d = 1.5 * M_PI;
    else
      d = 0.5 * M_PI;

    Sint16 nd = (Sint16)(180.0 * d / M_PI) % 360;
    Sint16 od = (_dir >> 8) % 360;

    Sint32 dvel = 0;
    if ( abs(nd - od) > 10 ) {
      if ( nd > od ) {
	if ( nd - od < 180 )
	  dvel = DVEL;
	else
	  dvel = -DVEL;
      }
      else if ( nd < od ) {
	if ( od - nd < 180 )
	  dvel = -DVEL;
	else
	  dvel = DVEL;
      }
    }

    _dir += (Sint32)(g->current_time() - _last_time) * dvel / 1000;
    while ( _dir < 0 )
      _dir += 360 << 8;

    d = M_PI * (_dir >> 8) / 180.0;
    
    xvel((Sint16)(-100 * sin(d)));
    yvel((Sint16)(100 * cos(d)));
    break;
  }
  }

  object::move();
}

void object_blocker::initialize() {
  _group = dynamic_cast<group_blocker *>(g->obtain_group("blocker", _group_id));
  if ( !_group ) {
    printf("Ooops, got wrong group class returned...\n");
    exit(0);
  }

  _group->add_object(this);
  
  _shot_trigger = get_trigger();
  set_trigger(_shot_trigger, (4000 + rand() % 3000));

  object::initialize();
}

void object_blocker::destroy() {
  object::destroy();

  g->play_expl_sound(SOUND_EXPL_2, SOUND_LAYER_ENEMY, 48);

  for ( int i = 0 ; i < 4 ; ++i ) {
    object *o = new object_expl2();
    o->xpos(xpos());
    o->ypos(ypos());
    o->xvel((75 + (rand() % 150)) * ((rand() & 16) ? -1 : 1));
    o->yvel((75 + (rand() % 150)) * ((rand() & 16) ? -1 : 1));
    g->add_object(o);
  }

  object *o = new object_expl1();
  o->xpos(xpos());
  o->ypos(ypos());
  o->xvel(xvel());
  o->yvel(yvel());
  g->add_object(o);
}

object *object_blocker::constructor() {
  return( new object_blocker() );
}

bool object_blocker::setup() {
  if ( object::setup() &&
       object_shot::setup() &&
       object_expl2::setup() ) {
    _image_id = g->obtain_bob("blocker");
    if ( _image_id != 0xffff )
      return( true );
  }

  return( false );
}

void object_blocker::cleanup() {
  g->release_bob(_image_id);

  object_expl2::cleanup();
  object_shot::cleanup();
  object::cleanup();
}

/************************************************************************
 *                                                                      *
 * class group_blocker                                                  *
 *                                                                      *
 ************************************************************************/

//$GROUP { "blocker", group_blocker::constructor, group_blocker::setup, group_blocker::cleanup },

group_blocker::group_blocker(Uint32 id) : group("blocker", id) {
  xpos(g->map_width() / 2);
  ypos(g->ypos() - 500);

  yvel(0);
  _angle = 0;
}

void group_blocker::move() {
  Sint32 dt = g->current_time() - _last_time;
  _angle += (10 << 8) * dt / 1000;

  group::move();
}

void group_blocker::initialize() {
  group::initialize();
}

void group_blocker::destroy() {
  group::destroy();

  g->add_signal(0x00000001);
}

group *group_blocker::constructor(Uint32 id) {
  return( new group_blocker(id) );
}

bool group_blocker::setup() {
  if ( group::setup() )
    return( true );

  return( false );
}

void group_blocker::cleanup() {
  group::cleanup();
}
