
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_bomber.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "game.h"
#include "object_expl.h"
#include "object_bomber.h"
#include "object_player.h"
#include "object_shot.h"

//$TAB { "bomber", object_bomber::constructor, object_bomber::setup, object_bomber::cleanup },

Uint16 object_bomber::_image_id = 0xffff;

object_bomber::object_bomber() : object() {
  part *p = add_part(1);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _dir = 0;
  _vel = 200;
  _dvel = 200 << 8;
  xvel(1);
  yvel(1);

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP | OBJECT_PLAYER_PROJECTILE;
  _type_mask = OBJECT_ENEMY_SHIP;

  _energy = 500;
  _score = 4000;
}

void object_bomber::set_modifiers(svector<object_modifier> &tab) {
  for ( svector<object_modifier>::const_iterator i = tab.begin() ; i != tab.end() ; ++i ) {
    if ( i->name == "dvel" )
      _dvel = i->val << 8;
    else if ( i->name == "vel" )
      _vel = i->val;
  }

  object::set_modifiers(tab);
}

void object_bomber::handle_trigger(Uint16 id) {
  if ( id == _shot_trigger ) {
    _shot_trigger = get_trigger();
    set_trigger(_shot_trigger, (200 + rand() % 50));

    if ( (_dir > (60 << 8)) && (_dir < (180 << 8)) ) {
      object *o = new object_shot(0);
      o->xpos(xpos());
      o->ypos(ypos());
      o->energy(50);
      g->add_object(o);
    }
  }

  object::handle_trigger(id);
}

Uint16 object_bomber::current_frame(part *p) const {
  Sint16 f;

  switch ( p->_id ) {
  case 1 :
    f = (p->_frames_count - 1) * (_dir >> 8) / 256;
    break;

  default :
    f = object::current_frame(p);
    break;
  }

  return( f );
}

void object_bomber::move() {
  Sint32 dt = g->current_time() - _last_time;

  _dir += _dvel * dt / 1000;
  if ( _dir > (256 << 8) )
    _dir = 256 << 8;
  
  double d = M_PI * (60.0 * (double)((_dir >> 8) - 128) / 255) / 180;

  xvel((Sint16)(_vel * cos(d)));
  yvel((Sint16)(_vel * -sin(d)));

  object::move();
}

void object_bomber::initialize() {
  _shot_trigger = get_trigger();
  set_trigger(_shot_trigger, (500 + rand() % 100));

  object::initialize();
}

void object_bomber::destroy() {
  object::destroy();

  g->play_expl_sound(SOUND_EXPL_2, SOUND_LAYER_ENEMY, 48);

  object *o = new object_expl3();
  o->xpos(xpos());
  o->ypos(ypos());
  o->xvel(xvel());
  o->yvel(yvel());
  g->add_object(o);
}

object *object_bomber::constructor() {
  return( new object_bomber() );
}

bool object_bomber::setup() {
  if ( object::setup() &&
       object_shot::setup() &&
       object_expl2::setup() ) {
    _image_id = g->obtain_bob("bomber");
    if ( _image_id != 0xffff )
      return( true );
  }

  return( false );
}

void object_bomber::cleanup() {
  g->release_bob(_image_id);

  object_expl2::cleanup();
  object_shot::cleanup();
  object::cleanup();
}
