
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_bonus.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include "main.h"
#include "game.h"
#include "object_bonus.h"
#include "object_player.h"

/****************************************************************
 *                                                              *
 * class object_bonus_energy                                    *
 *                                                              *
 ****************************************************************/

Uint16 object_bonus_energy::_image_id = 0xffff;
Uint16 object_bonus_energy::_snd_id = 0xffff;

object_bonus_energy::object_bonus_energy() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP;
  _type_mask = OBJECT_BONUS;
}

void object_bonus_energy::hit_by(object *o) {
  if ( o == (object *)g->player() ) {
    g->add_message("Energy");
    g->player()->add_energy(250);
    g->play_sound(_snd_id, SOUND_LAYER_MISC);
  }
}

bool object_bonus_energy::setup() {
  _image_id = g->obtain_bob("bEnergy");
  if ( _image_id != 0xffff ) {
    _snd_id = g->obtain_sound("bonus");
    if ( _snd_id != 0xff )
      return( true );
  }

  return( false );
}

void object_bonus_energy::cleanup() {
  g->release_sound(_snd_id);
  g->release_bob(_image_id);
}

/****************************************************************
 *                                                              *
 * class object_bonus_shield                                    *
 *                                                              *
 ****************************************************************/

Uint16 object_bonus_shield::_image_id = 0xffff;
Uint16 object_bonus_shield::_snd_id = 0xffff;

object_bonus_shield::object_bonus_shield() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP;
  _type_mask = OBJECT_BONUS;
}

void object_bonus_shield::hit_by(object *o) {
  if ( o == (object *)g->player() ) {
    g->add_message("Shield");
    g->player()->set_shield();
    g->play_sound(_snd_id, SOUND_LAYER_MISC);
  }
}

bool object_bonus_shield::setup() {
  _image_id = g->obtain_bob("bShield");
  if ( _image_id != 0xffff ) {
    _snd_id = g->obtain_sound("bonus");
    if ( _snd_id != 0xff )
      return( true );
  }

  return( false );
}

void object_bonus_shield::cleanup() {
  g->release_sound(_snd_id);
  g->release_bob(_image_id);
}

/****************************************************************
 *                                                              *
 * class object_bonus_life                                    *
 *                                                              *
 ****************************************************************/

Uint16 object_bonus_life::_image_id = 0xffff;
Uint16 object_bonus_life::_snd_id = 0xffff;

object_bonus_life::object_bonus_life() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP;
  _type_mask = OBJECT_BONUS;
}

void object_bonus_life::hit_by(object *o) {
  if ( o == (object *)g->player() ) {
    g->add_message("Extra life");
    g->player()->add_life();
    g->play_sound(_snd_id, SOUND_LAYER_MISC);
  }
}

bool object_bonus_life::setup() {
  _image_id = g->obtain_bob("bLife");
  if ( _image_id != 0xffff ) {
    _snd_id = g->obtain_sound("bonus");
    if ( _snd_id != 0xff )
      return( true );
  }

  return( false );
}

void object_bonus_life::cleanup() {
  g->release_sound(_snd_id);
  g->release_bob(_image_id);
}

/****************************************************************
 *                                                              *
 * class object_bonus_ball                                    *
 *                                                              *
 ****************************************************************/

Uint16 object_bonus_ball::_image_id = 0xffff;
Uint16 object_bonus_ball::_snd_id = 0xffff;

object_bonus_ball::object_bonus_ball() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP;
  _type_mask = OBJECT_BONUS;
}

void object_bonus_ball::hit_by(object *o) {
  if ( o == (object *)g->player() ) {
    if ( g->player()->gun_power(PLAYER_GUN_BALL) > 0 )
      g->add_message("Enhance weapon: ball");
    else
      g->add_message("New weapon: ball");

    g->player()->set_gun(PLAYER_GUN_BALL);
    g->play_sound(_snd_id, SOUND_LAYER_MISC);
  }
}

bool object_bonus_ball::setup() {
  _image_id = g->obtain_bob("bBall");
  if ( _image_id != 0xffff ) {
    _snd_id = g->obtain_sound("bonus");
    if ( _snd_id != 0xff )
      return( true );
  }

  return( false );
}

void object_bonus_ball::cleanup() {
  g->release_sound(_snd_id);
  g->release_bob(_image_id);
}

/****************************************************************
 *                                                              *
 * class object_bonus_seeker                                    *
 *                                                              *
 ****************************************************************/

Uint16 object_bonus_seeker::_image_id = 0xffff;
Uint16 object_bonus_seeker::_snd_id = 0xffff;

object_bonus_seeker::object_bonus_seeker() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP;
  _type_mask = OBJECT_BONUS;
}

void object_bonus_seeker::hit_by(object *o) {
  if ( o == (object *)g->player() ) {
    if ( g->player()->gun_power(PLAYER_GUN_SEEKER) > 0 )
      g->add_message("Enhance weapon: missile");
    else
      g->add_message("New weapon: missile");

    g->player()->set_gun(PLAYER_GUN_SEEKER);
    g->play_sound(_snd_id, SOUND_LAYER_MISC);
  }
}

bool object_bonus_seeker::setup() {
  _image_id = g->obtain_bob("bSeeker");
  if ( _image_id != 0xffff ) {
    _snd_id = g->obtain_sound("bonus");
    if ( _snd_id != 0xff )
      return( true );
  }

  return( false );
}

void object_bonus_seeker::cleanup() {
  g->release_sound(_snd_id);
  g->release_bob(_image_id);
}

/****************************************************************
 *                                                              *
 * class object_bonus_laser                                    *
 *                                                              *
 ****************************************************************/

Uint16 object_bonus_laser::_image_id = 0xffff;
Uint16 object_bonus_laser::_snd_id = 0xffff;

object_bonus_laser::object_bonus_laser() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP;
  _type_mask = OBJECT_BONUS;
}

void object_bonus_laser::hit_by(object *o) {
  if ( o == (object *)g->player() ) {
    if ( g->player()->gun_power(PLAYER_GUN_LASER) > 0 )
      g->add_message("Enhance weapon: laser");
    else
      g->add_message("New weapon: laser");

    g->player()->set_gun(PLAYER_GUN_LASER);
    g->play_sound(_snd_id, SOUND_LAYER_MISC);
  }
}

bool object_bonus_laser::setup() {
  _image_id = g->obtain_bob("bLaser");
  if ( _image_id != 0xffff ) {
    _snd_id = g->obtain_sound("bonus");
    if ( _snd_id != 0xff )
      return( true );
  }

  return( false );
}

void object_bonus_laser::cleanup() {
  g->release_sound(_snd_id);
  g->release_bob(_image_id);
}

/****************************************************************
 *                                                              *
 * class object_bonus_plasma                                    *
 *                                                              *
 ****************************************************************/

Uint16 object_bonus_plasma::_image_id = 0xffff;
Uint16 object_bonus_plasma::_snd_id = 0xffff;

object_bonus_plasma::object_bonus_plasma() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP;
  _type_mask = OBJECT_BONUS;
}

void object_bonus_plasma::hit_by(object *o) {
  if ( o == (object *)g->player() ) {
    if ( g->player()->gun_power(PLAYER_GUN_PLASMA) > 0 )
      g->add_message("Enhance weapon: plasma");
    else
      g->add_message("New weapon: plasma");
    g->player()->set_gun(PLAYER_GUN_PLASMA);
    g->play_sound(_snd_id, SOUND_LAYER_MISC);
  }
}

bool object_bonus_plasma::setup() {
  _image_id = g->obtain_bob("bPlasma");
  if ( _image_id != 0xffff ) {
    _snd_id = g->obtain_sound("bonus");
    if ( _snd_id != 0xff )
      return( true );
  }

  return( false );
}

void object_bonus_plasma::cleanup() {
  g->release_sound(_snd_id);
  g->release_bob(_image_id);
}


