
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_expl.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include "main.h"
#include "game.h"
#include "object_expl.h"

/*******************************************************************
 *                                                                 *
 * class object_expl1                                              *
 *                                                                 *
 *******************************************************************/

Uint16 object_expl1::_image_id = 0xffff;

object_expl1::object_expl1() : object() {
  part *p = add_part(1);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _layer = LAYER_EXPLOSIONS;

  _hit_mask = 0;
  _type_mask = OBJECT_NEUTRAL;
}

void object_expl1::initialize() {
  object::initialize();

  _pix_last_time = g->current_time();

  for ( Uint16 i = 0 ; i < _pix_cnt ; ++i ) {
    _pix[i].x = xpos() << 8;
    _pix[i].y = ypos() << 8;
    _pix[i].xs = (xvel() + (rand() % 501) - 250) << 8;
    _pix[i].ys = (yvel() + (rand() % 501) - 250) << 8;
    _pix[i].r = 236 + (rand() % 20);
    _pix[i].g = 220 + (rand() % 20);
    _pix[i].b = 10 + (rand() % 40);
  }
}

void object_expl1::move() {
  object::move();

  if ( g->current_time() > _pix_last_time ) {
    Sint32 dt = g->current_time() - _pix_last_time;
    
    for ( struct pix *send = _pix + _pix_cnt, *s = _pix ; s < send ; ++s ) {
      s->x += dt * s->xs / 1000;
      s->y += dt * s->ys / 1000;
    }

    _pix_last_time = g->current_time();
  }

  part *p = get_part(1);
  if ( !p || (p->_frames_done >= p->_frames_count) )
    _flags |= OBJECT_DELETED;
}

void object_expl1::paint() {
  part *p = get_part(1);
  if ( p ) {
    for ( struct pix *send = _pix + _pix_cnt, *s = _pix ; s < send ; ++s ) {
      g->draw_pixel((s->x - g->fxpos()) >> 8, (s->y - g->fypos()) >> 8, 
		    (p->_frames_count - p->_frames_done) * s->r / p->_frames_count, 
		    (p->_frames_count - p->_frames_done) * s->g / p->_frames_count, 
		    (p->_frames_count - p->_frames_done) * s->b / p->_frames_count);
    }
  }

  object::paint();
}

bool object_expl1::setup() {
  _image_id = g->obtain_bob("expl1");

  if ( _image_id != 0xffff )
    return( true );

  return( false );
}

void object_expl1::cleanup() {
  g->release_bob(_image_id);
}

/*******************************************************************
 *                                                                 *
 * class object_expl2                                              *
 *                                                                 *
 *******************************************************************/

Uint16 object_expl2::_image_id = 0xffff;

object_expl2::object_expl2() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _layer = LAYER_EXPLOSIONS;

  _hit_mask = 0;
  _type_mask = OBJECT_NEUTRAL;
}

void object_expl2::initialize() {
  object::initialize();

  _pix_last_time = g->current_time();

  for ( Uint16 i = 0 ; i < _pix_cnt ; ++i ) {
    _pix[i].x = xpos() << 8;
    _pix[i].y = ypos() << 8;
    _pix[i].xs = (xvel() + (rand() % 401) - 200) << 8;
    _pix[i].ys = (yvel() + (rand() % 401) - 200) << 8;
    _pix[i].r = 200 + (rand() % 15);
    _pix[i].g = 180 + (rand() % 15);
    _pix[i].b = 15 + (rand() % 30);
  }
}

void object_expl2::move() {
  object::move();

  if ( g->current_time() > _pix_last_time ) {
    Sint32 dt = g->current_time() - _pix_last_time;
    
    for ( struct pix *send = _pix + _pix_cnt, *s = _pix ; s < send ; ++s ) {
      s->x += dt * s->xs / 1000;
      s->y += dt * s->ys / 1000;
    }

    _pix_last_time = g->current_time();
  }


  part *p = get_part(1);
  if ( !p || (p->_frames_done >= p->_frames_count) )
    _flags |= OBJECT_DELETED;
}

void object_expl2::paint() {
  part *p = get_part(1);
  if ( p ) {
    for ( struct pix *send = _pix + _pix_cnt, *s = _pix ; s < send ; ++s ) {
      g->draw_pixel((s->x - g->fxpos()) >> 8, (s->y - g->fypos()) >> 8, 
		    (p->_frames_count - p->_frames_done) * s->r / p->_frames_count, 
		    (p->_frames_count - p->_frames_done) * s->g / p->_frames_count, 
		    (p->_frames_count - p->_frames_done) * s->b / p->_frames_count);
    }
  }

  object::paint();
}

bool object_expl2::setup() {
  _image_id = g->obtain_bob("expl2");

  if ( _image_id != 0xffff )
    return( true );

  return( false );
}

void object_expl2::cleanup() {
  g->release_bob(_image_id);
}

/*******************************************************************
 *                                                                 *
 * class object_expl3                                              *
 *                                                                 *
 *******************************************************************/

Uint16 object_expl3::_image_id = 0xffff;

object_expl3::object_expl3() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _layer = LAYER_EXPLOSIONS;

  _hit_mask = 0;
  _type_mask = OBJECT_NEUTRAL;
}

void object_expl3::initialize() {
  object::initialize();

  _pix_last_time = g->current_time();

  for ( Uint16 i = 0 ; i < _pix_cnt ; ++i ) {
    _pix[i].x = xpos() << 8;
    _pix[i].y = ypos() << 8;
    _pix[i].xs = (xvel() + (rand() % 301) - 150) << 8;
    _pix[i].ys = (yvel() + (rand() % 301) - 150) << 8;
    _pix[i].r = 180 + (rand() % 10);
    _pix[i].g = 150 + (rand() % 10);
    _pix[i].b = 5 + (rand() % 20);
  }
}

void object_expl3::move() {
  object::move();

  if ( g->current_time() > _pix_last_time ) {
    Sint32 dt = g->current_time() - _pix_last_time;
    
    for ( struct pix *send = _pix + _pix_cnt, *s = _pix ; s < send ; ++s ) {
      s->x += dt * s->xs / 1000;
      s->y += dt * s->ys / 1000;
    }

    _pix_last_time = g->current_time();
  }


  part *p = get_part(1);
  if ( !p || (p->_frames_done >= p->_frames_count) )
    _flags |= OBJECT_DELETED;
}

void object_expl3::paint() { 
  part *p = get_part(1);
  if ( p ) {
    for ( struct pix *send = _pix + _pix_cnt, *s = _pix ; s < send ; ++s ) {
      g->draw_pixel((s->x - g->fxpos()) >> 8, (s->y - g->fypos()) >> 8, 
		    (p->_frames_count - p->_frames_done) * s->r / p->_frames_count, 
		    (p->_frames_count - p->_frames_done) * s->g / p->_frames_count, 
		    (p->_frames_count - p->_frames_done) * s->b / p->_frames_count);
    }
  }

  object::paint();
}

bool object_expl3::setup() {
  _image_id = g->obtain_bob("expl3");

  if ( _image_id != 0xffff )
    return( true );

  return( false );
}

void object_expl3::cleanup() {
  g->release_bob(_image_id);
}

