
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_glider.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "game.h"
#include "object_glider.h"
#include "object_expl.h"
#include "object_shot.h"

//$TAB { "glider", object_glider::constructor, object_glider::setup, object_glider::cleanup },

Uint16 object_glider::_image_id = 0xffff;

object_glider::object_glider() : object() {
  part *p = add_part(1);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _auto_delete = false;

  _dir = 0;
  _group_id = 0;
  _group_member = 0;

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP | OBJECT_PLAYER_PROJECTILE;
  _type_mask = OBJECT_ENEMY_SHIP;

  _energy = 150;
  _score = 500;
}

void object_glider::set_modifiers(svector<object_modifier> &tab) {
  for ( svector<object_modifier>::const_iterator i = tab.begin() ; i != tab.end() ; ++i ) {
    if ( i->name == "group" )
      _group_id = i->val;
    else if ( i->name == "member" )
      _group_member = i->val;
  }

  object::set_modifiers(tab);
}

void object_glider::handle_trigger(Uint16 id) {
  if ( id == _shot_trigger ) {
    _shot_trigger = get_trigger();
    set_trigger(_shot_trigger, (2000 + rand() % 3000));

    object *o = new object_shot(0);
    o->xpos(xpos());
    o->ypos(ypos());
    g->add_object(o);
  }

  object::handle_trigger(id);
}

Uint16 object_glider::current_frame(part *p) const {
  Sint16 f;

  switch ( p->_id ) {
  case 1 :
    f = (p->_frames_count * (_dir >> 8) / 360) % p->_frames_count;
    break;

  default :
    f = object::current_frame(p);
    break;
  }

  return( f );
}

void object_glider::move() {
  const Sint32 DVEL = 200 << 8;
  const Sint32 VEL = 200;

  double dx = _target_x[_target] - xpos();
  double dy = _target_y[_target] - ypos();
  double d;
  
  if ( dx * dx + dy * dy < 16*16 ) {
    _target++;
    if ( (_target_x[_target] < 0) || (_target_x[_target] >= g->map_width()) ||
	 (_target_y[_target] < 0) || (_target_y[_target] >= g->map_height()) )
      _auto_delete = true;
  }

  if ( dy > 0 )
    d = atan(-dx / dy) + 2 * M_PI;
  else if ( dy < 0 )
    d = atan(-dx / dy) + M_PI;
  else if ( dx > 0 )
    d = 1.5 * M_PI;
  else
    d = 0.5 * M_PI;

  Sint16 nd = (Sint16)(180.0 * d / M_PI) % 360;
  Sint16 od = (_dir >> 8) % 360;

  Sint32 dvel = 0;
  if ( abs(nd - od) > 10 ) {
    if ( nd > od ) {
      if ( nd - od < 180 )
	dvel = DVEL;
      else
	dvel = -DVEL;
    }
    else if ( nd < od ) {
      if ( od - nd < 180 )
	dvel = -DVEL;
      else
	dvel = DVEL;
    }
  }

  _dir += (Sint32)(g->current_time() - _last_time) * dvel / 1000;
  while ( _dir < 0 )
    _dir += 360 << 8;

  d = M_PI * (_dir >> 8) / 180.0;

  xvel((Sint16)(-VEL * sin(d)));
  yvel((Sint16)(VEL * cos(d)));

  object::move();
}

void object_glider::initialize() {
  _target = 0;

  switch ( _group_id ) {
  default :
  case 0 :
    _target_x[0] = g->xpos() + 5 * MAIN_WIDTH / 8;
    _target_y[0] = g->ypos() + 3 * MAIN_HEIGHT / 4 - (9 - _group_member) * 16;

    _target_x[1] = g->xpos() + 1 * MAIN_WIDTH / 4 + _group_member * 20;
    _target_y[1] = g->ypos() + 1 * MAIN_HEIGHT / 4;

    _target_x[2] = g->xpos() + 3 * MAIN_WIDTH / 8 + _group_member * 10;
    _target_y[2] = g->ypos() + 2 * MAIN_HEIGHT / 4;

    _target_x[3] = g->xpos() + MAIN_WIDTH / 2;
    _target_y[3] = g->ypos() + MAIN_HEIGHT / 4 - _group_member * 20;

    _target_x[4] = g->xpos() + MAIN_WIDTH / 2;
    _target_y[4] = -100;
    break;

  case 1 :
    _target_x[0] = g->xpos() + 3 * MAIN_WIDTH / 8;
    _target_y[0] = g->ypos() + 3 * MAIN_HEIGHT / 4 - (9 - _group_member) * 16;

    _target_x[1] = g->xpos() + 3 * MAIN_WIDTH / 4 - _group_member * 20;
    _target_y[1] = g->ypos() + 1 * MAIN_HEIGHT / 4;

    _target_x[2] = g->xpos() + 5 * MAIN_WIDTH / 8 - _group_member * 10;
    _target_y[2] = g->ypos() + 2 * MAIN_HEIGHT / 4;

    _target_x[3] = g->xpos() + MAIN_WIDTH / 2;
    _target_y[3] = g->ypos() + MAIN_HEIGHT / 4 - _group_member * 20;

    _target_x[4] = g->xpos() + MAIN_WIDTH / 2;
    _target_y[4] = -100;
    break;
  }

  _shot_trigger = get_trigger();
  set_trigger(_shot_trigger, (2000 + rand() % 3000));

  object::initialize();
}

void object_glider::destroy() {
  object::destroy();

  g->play_expl_sound(SOUND_EXPL_2, SOUND_LAYER_ENEMY, 48);

  object *o = new object_expl3();
  o->xpos(xpos());
  o->ypos(ypos());
  o->xvel(xvel());
  o->yvel(yvel());
  g->add_object(o);
}

object *object_glider::constructor() {
  return( new object_glider() );
}

bool object_glider::setup() {
  if ( object::setup() &&
       object_shot::setup() &&
       object_expl2::setup() ) {
    _image_id = g->obtain_bob("glider");
    if ( _image_id != 0xffff )
      return( true );
  }

  return( false );
}

void object_glider::cleanup() {
  g->release_bob(_image_id);

  object_expl2::cleanup();
  object_shot::cleanup();
  object::cleanup();
}
