
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_missile.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "game.h"
#include "object_missile.h"
#include "object_player.h"
#include "object_expl.h"
#include "object_smoke.h"

//$TAB { "missile", object_missile::constructor, object_missile::setup, object_missile::cleanup },

Uint16 object_missile::_image_id = 0xffff;

object_missile::object_missile() : object() {
  part *p = add_part(1);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _auto_delete = false;

  _dir = 0;

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP | OBJECT_PLAYER_PROJECTILE;
  _type_mask = OBJECT_ENEMY_SHIP;

  _energy = 150;
  _score = 500;
}

Uint16 object_missile::current_frame(part *p) const {
  Sint16 f;

  switch ( p->_id ) {
  case 1 :
    f = (p->_frames_count * (_dir >> 8) / 360) % p->_frames_count;
    break;

  default :
    f = object::current_frame(p);
    break;
  }

  return( f );
}

void object_missile::move() {
  const Sint32 DVEL = 160 << 8;
  const Sint32 VEL = 200;

  object *pl = g->player();
  double dx = pl->xpos() - xpos();
  double dy = pl->ypos() - ypos();
  double d;
  
  if ( dy > 0 )
    d = atan(-dx / dy) + 2 * M_PI;
  else if ( dy < 0 )
    d = atan(-dx / dy) + M_PI;
  else if ( dx > 0 )
    d = 1.5 * M_PI;
  else
    d = 0.5 * M_PI;

  Sint16 nd = (Sint16)(180.0 * d / M_PI) % 360;
  Sint16 od = (_dir >> 8) % 360;

  Sint32 dvel = 0;
  if ( abs(nd - od) > 10 ) {
    if ( nd > od ) {
      if ( nd - od < 180 )
	dvel = DVEL;
      else
	dvel = -DVEL;
    }
    else if ( nd < od ) {
      if ( od - nd < 180 )
	dvel = -DVEL;
      else
	dvel = DVEL;
    }
  }

  _dir += (Sint32)(g->current_time() - _last_time) * dvel / 1000;
  while ( _dir < 0 )
    _dir += 360 << 8;

  d = M_PI * (_dir >> 8) / 180.0;

  xvel((Sint16)(-VEL * sin(d)));
  yvel((Sint16)(VEL * cos(d)));

  if ( g->current_time() > _next_smoke ) {
    _next_smoke = g->current_time() + rand() % 250 + 375;

    object *o = new object_smoke3();
    o->xpos(xpos());
    o->ypos(ypos());
    o->xvel(xvel() / 5);
    o->yvel(yvel() / 5);
    g->add_object(o);
  }

  if ( g->current_time() - _start_time > 7000 )
    destroy();
  else
    object::move();
}

void object_missile::initialize() {
  _next_smoke = g->current_time() + rand() % 250 + 375;
  _start_time = g->current_time();

  object::initialize();
}

void object_missile::destroy() {
  object::destroy();

  g->play_expl_sound(SOUND_EXPL_3, SOUND_LAYER_ENEMY, 48);

  object *o = new object_expl3();
  o->xpos(xpos());
  o->ypos(ypos());
  o->xvel(xvel());
  o->yvel(yvel());
  g->add_object(o);
}

object *object_missile::constructor() {
  return( new object_missile() );
}

bool object_missile::setup() {
  if ( object::setup() &&
       object_smoke1::setup() &&
       object_expl3::setup() ) {
    _image_id = g->obtain_bob("missile");
    if ( _image_id != 0xffff )
      return( true );
  }

  return( false );
}

void object_missile::cleanup() {
  g->release_bob(_image_id);

  object_expl3::cleanup();
  object_smoke3::cleanup();
  object::cleanup();
}
