
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_player.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include "main.h"
#include "game.h"
#include "object_player.h"
#include "object_projectile.h"
#include "object_expl.h"

#define MAX_XVEL    (300 << 8)
#define XACC        (4000 << 8)
#define MAX_FVEL    (300 << 8)
#define FACC        (4000 << 8)
#define MAX_RVEL    (200 << 8)
#define RACC        (3000 << 8)

Uint16 object_player::_image_id = 0xffff;
Uint16 object_player::_shield_id = 0xffff;
Uint16 object_player::_snd_laser_id = 0xffff;
Uint16 object_player::_snd_plasma_id = 0xffff;
Uint16 object_player::_snd_ball_id = 0xffff;
Uint16 object_player::_snd_seeker_id = 0xffff;
Uint16 object_player::_snd_score_id = 0xffff;

object_player::object_player() : object() {
  part *p = add_part(1);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = 7;
  p->_frames_offset = 0;
  p->_layer = 1;

  p = add_part(2);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_per_second = 20;
  p->_frames_count = 14;
  p->_frames_offset = 7;
  p->_layer = 0;

  update_size();

  auto_delete(false);

  _shield = g->get_bob(_shield_id);

  _layer = LAYER_PLAYER;

  _shield_cnt = 0;
  _invul_cnt = 0;

  update_hit_mask();
  _type_mask = OBJECT_PLAYER_SHIP;

  _lifes = 3;
  _energy_depot = PLAYER_MAX_ENERGY;
  _energy = 0;
  _score_depot = 0;
  _smart_bombs = 0;

  _enable_steering = true;

  _gun_power[PLAYER_GUN_LASER] = 0;
  _gun_power[PLAYER_GUN_BALL] = 0;
  _gun_power[PLAYER_GUN_PLASMA] = 0;
  _gun_power[PLAYER_GUN_SEEKER] = 0;
  _gun_type = PLAYER_GUN_LASER;
  _gun_power[_gun_type] = 1;
}

object_player::~object_player() {
}

void object_player::startup() {
  unset_gun(PLAYER_GUN_LASER);
  unset_gun(PLAYER_GUN_PLASMA);
  unset_gun(PLAYER_GUN_BALL);
  unset_gun(PLAYER_GUN_SEEKER);
  unset_shield();

  add_energy(PLAYER_MAX_ENERGY);
  set_invul();
  enable_steering();
  xpos(g->xpos() + MAIN_WIDTH / 2);
  xvel(0);
  xacc(0);
  ypos(g->ypos() + MAIN_HEIGHT - bbound() - 20);
  yvel(0);
  yacc(0);
  flags(flags() & ~(OBJECT_INVISIBLE | PLAYER_KILLED | PLAYER_KILLED_DONE));
  update_hit_mask();

  _seeker_enabled = false;
}

void object_player::update_hit_mask() {
  _hit_mask = OBJECT_ENEMY_SHIP | OBJECT_ENEMY_PROJECTILE | OBJECT_BONUS;

  if ( _shield_cnt > 0 )
    _hit_mask &= ~OBJECT_ENEMY_PROJECTILE;

  if ( _invul_cnt > 0 )
    _hit_mask &= ~(OBJECT_ENEMY_PROJECTILE | OBJECT_ENEMY_SHIP);
}

void object_player::initialize() {
  _score_depot_last_time = g->current_time();
  _fire_last_time = g->current_time();
  _flame_current = 7;
  _flame_last_time = g->current_time();
  _seeker_enabled = false;

  object::initialize();
}

void object_player::move() {
  _req_xvel = 0;
  _req_yvel = 0;

  if ( flags() & PLAYER_KILLED ) {
    // UUAAAAAHH, EXPLOOOODING!

    _seeker_enabled = false;

    if ( g->current_time() - _killed_time > 10000 ) {
      flags(flags() | PLAYER_KILLED_DONE);
    }
    else if ( g->current_time() - _killed_time > 5000 ) {
    }
    else if ( g->current_time() - _expl_last_time > _expl_delay ) {
      _expl_delay = rand() % 200 + (g->current_time() - _killed_time) / 20 + 100;
      _expl_last_time = g->current_time();

      switch ( rand() % 4 ) {
      case 0 : {
	g->play_expl_sound(SOUND_EXPL_2, SOUND_LAYER_ENEMY);

	object *o = new object_expl1();
	o->xpos(xpos() + (rand() % 101) - 50);
	o->ypos(ypos() + (rand() % 101) - 50);
	g->add_object(o);
	break;
      }

      case 1 :
      case 2 :
      case 3 : {
	g->play_expl_sound(SOUND_EXPL_4, SOUND_LAYER_ENEMY);

	object *o = new object_expl2();
	o->xpos(xpos() + (rand() % 151) - 75);
	o->ypos(ypos() + (rand() % 151) - 75);
	g->add_object(o);
	break;
      }
      }
    }
  }
  else {
    if ( _enable_steering ) {
      switch ( g->input_device() ) {
      case INPUT_KEYBOARD : {
	Uint8 *key_table = SDL_GetKeyState(NULL);
	if ( key_table[SDLK_UP] == SDL_PRESSED )
	  _req_yvel = -MAX_FVEL;
	else if ( key_table[SDLK_DOWN] == SDL_PRESSED )
	  _req_yvel = MAX_RVEL;

	if ( key_table[SDLK_RIGHT] == SDL_PRESSED )
	  _req_xvel = MAX_XVEL;
	else if ( key_table[SDLK_LEFT] == SDL_PRESSED )
	  _req_xvel = -MAX_XVEL;

	if ( key_table[SDLK_SPACE] != SDL_PRESSED ) {
	  _seeker_start = g->current_time();
	  _seeker_enabled = true;
	}
	break;
      }

      case INPUT_JOYSTICK : {
	SDL_Joystick *joy = g->joystick_handle();

	Sint16 x_axis = SDL_JoystickGetAxis(joy, 0);
	Sint16 y_axis = SDL_JoystickGetAxis(joy, 1);
	Uint8 button = SDL_JoystickGetButton(joy, 0);

	_req_xvel = MAX_XVEL * (Sint32)(x_axis >> 4) / (Sint32)(10000 >> 4);
	if ( _req_xvel > MAX_XVEL )
	  _req_xvel = MAX_XVEL;
	else if ( _req_xvel < -MAX_XVEL )
	  _req_xvel = -MAX_XVEL;

	if ( y_axis < 0 ) {
	  _req_yvel = -MAX_FVEL * (Sint32)-(y_axis >> 4) / (Sint32)(10000 >> 4);
	  if ( _req_yvel < -MAX_FVEL )
	    _req_yvel = -MAX_FVEL;
	}
	else if ( y_axis > 0 ) {
	  _req_yvel = MAX_RVEL * (Sint32)(y_axis >> 4) / (Sint32)(10000 >> 4);
	  if ( _req_yvel > MAX_RVEL )
	    _req_yvel = MAX_RVEL;
	}

	if ( button == 0 ) {
	  _seeker_start = g->current_time();
	  _seeker_enabled = true;
	}
	break;
      }
      }

      if ( _req_xvel < _xvel )
	_xacc = -XACC;
      else if ( _req_xvel > _xvel )
	_xacc = XACC;
      else
	_xacc = 0;

      if ( _req_yvel < _yvel )
	_yacc = -FACC;
      else if ( _req_yvel > _yvel )
	_yacc = RACC;
      else
	_yacc = 0;
    }
    else {
      _fire_flag = false;
      _seeker_enabled = false;
    }

    Sint32 dt = g->current_time() - _last_time;
    if ( dt > 0 ) {
      _xpos += _xacc * dt * dt / 1000000 + _xvel * dt / 1000;
      if ( _enable_steering && (xpos() < width() / 4) )
	xpos( width() / 4 );
      else if ( _enable_steering && (xpos() > g->map_width() - width() / 4) )
	xpos( g->map_width() - width() / 4 );

      if ( _enable_steering ) {
	Sint32 nv = _xvel + (Sint32)_xacc * dt / 1000;

	if ( ((_xvel < _req_xvel) && (nv > _req_xvel)) ||
	     ((_xvel > _req_xvel) && (nv < _req_xvel)) )
	  nv = _req_xvel;
	
	_xvel = nv;
      }
      else
	_xvel += (Sint32)_xacc * dt / 1000;

      _ypos += _yacc * dt * dt / 1000000 + (_yvel - (Sint32)(g->level()->yvel() << 8)) * dt / 1000;
      if ( _enable_steering && (ypos() < g->ypos() + height() / 4) )
	ypos( g->ypos() + height() / 4 );
      else if ( _enable_steering && (ypos() > g->ypos() + MAIN_HEIGHT - height() / 4) )
	ypos( g->ypos() + MAIN_HEIGHT - height() / 4 );

      if ( _enable_steering ) {
	Sint32 nv = _yvel + (Sint32)_yacc * dt / 1000;

	if ( ((_yvel < _req_yvel) && (nv > _req_yvel)) ||
	     ((_yvel > _req_yvel) && (nv < _req_yvel)) )
	  nv = _req_yvel;
	
	_yvel = nv;
      }
      else
	_yvel += (Sint32)_yacc * dt / 1000;
      
      if ( _seeker_enabled && (_gun_power[PLAYER_GUN_SEEKER] > 0) && (g->current_time() - _seeker_start >= 1200) ) {
	_seeker_start = g->current_time();
	_seeker_enabled = false;

	g->play_sound(_snd_seeker_id, SOUND_LAYER_PLAYER, 32 + _gun_power[PLAYER_GUN_SEEKER] * 96 / 6);

	object *o = NULL;

	switch ( _gun_power[PLAYER_GUN_SEEKER] ) {
	case 1 :
	  o = new object_seeker(0);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  g->add_object(o);
	  break;
	case 2 :
	  o = new object_seeker(1);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  g->add_object(o);
	  break;
	case 3 :
	  o = new object_seeker(0);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  g->add_object(o);
      
	  o = new object_seeker(1);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  o->xvel(-23);
	  g->add_object(o);
      
	  o = new object_seeker(1);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  o->xvel(23);
	  g->add_object(o);
	  break;
	case 4 :
	  o = new object_seeker(1);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  g->add_object(o);
      
	  o = new object_seeker(1);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  o->xvel(-23);
	  g->add_object(o);
      
	  o = new object_seeker(1);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  o->xvel(23);
	  g->add_object(o);
	  break;
	case 5 :
	  o = new object_seeker(1);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  g->add_object(o);
      
	  o = new object_seeker(2);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  o->xvel(-28);
	  g->add_object(o);
      
	  o = new object_seeker(2);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  o->xvel(28);
	  g->add_object(o);
	  break;
	case 6 :
	  o = new object_seeker(2);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  g->add_object(o);
      
	  o = new object_seeker(2);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  o->xvel(-28);
	  g->add_object(o);
      
	  o = new object_seeker(2);
	  o->xpos(xpos());
	  o->ypos(ypos() + tbound());
	  o->xvel(28);
	  g->add_object(o);
	  break;
	}
      }

      if ( _fire_flag ) {
	if ( g->current_time() - _fire_last_time > 100 ) {
	  object *o;

	  switch ( _gun_type ) {
	  case PLAYER_GUN_PLASMA : 
	    g->play_sound(_snd_plasma_id, SOUND_LAYER_PLAYER, 32 + _gun_power[PLAYER_GUN_PLASMA] * 96 / 6);

	    switch ( _gun_power[PLAYER_GUN_PLASMA] ) {
	    case 1 :
	      o = new object_plasma1();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;

	    case 2 :
	      o = new object_plasma2();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;

	    case 3 :
	      o = new object_plasma3();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;

	    case 4 :
	      o = new object_plasma3();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_plasma1l();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_plasma1r();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;
	
	    case 5 :
	      o = new object_plasma3();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);

	      o = new object_plasma2l();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	
	      o = new object_plasma2r();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;
	
	    case 6 :
	      o = new object_plasma3();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_plasma3l();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_plasma3r();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;
	    }
	    break;
    
	  case PLAYER_GUN_LASER : 
	    g->play_sound(_snd_laser_id, SOUND_LAYER_PLAYER, 32 + _gun_power[PLAYER_GUN_LASER] * 96 / 6);

	    switch ( _gun_power[PLAYER_GUN_LASER] ) {
	    case 1 :
	      o = new object_laser1();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;
      
	    case 2 :
	      o = new object_laser2();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;
      
	    case 3 :
	      o = new object_laser3();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;
      
	    case 4 :
	      o = new object_laser3();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_laser1();
	      o->xpos(xpos() - 10);
	      o->ypos(ypos() + tbound() + 2);
	      g->add_object(o);
      
	      o = new object_laser1();
	      o->xpos(xpos() + 10);
	      o->ypos(ypos() + tbound() + 2);
	      g->add_object(o);
	      break;
      
	    case 5 :
	      o = new object_laser3();
	      o->xpos(xpos() - 6);
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_laser3();
	      o->xpos(xpos() + 6);
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;
      
	    case 6 :
	      o = new object_laser3();
	      o->xpos(xpos() - 5);
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_laser3();
	      o->xpos(xpos() + 5);
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_laser1();
	      o->xpos(xpos() - 12);
	      o->ypos(ypos() + tbound() + 2);
	      g->add_object(o);
      
	      o = new object_laser1();
	      o->xpos(xpos() + 12);
	      o->ypos(ypos() + tbound() + 2);
	      g->add_object(o);
	      break;
	    }
	    break;
    
	  case PLAYER_GUN_BALL : 
	    g->play_sound(_snd_ball_id, SOUND_LAYER_PLAYER, 32 + _gun_power[PLAYER_GUN_BALL] * 96 / 6);

	    switch ( _gun_power[PLAYER_GUN_BALL] ) {
	    case 1 :
	      o = new object_ball1();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;
	    case 2 :
	      o = new object_ball2();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
	      break;
	    case 3 :
	      o = new object_ball1();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_ball2();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      o->xvel(-23);
	      g->add_object(o);
      
	      o = new object_ball2();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      o->xvel(23);
	      g->add_object(o);
	      break;
	    case 4 :
	      o = new object_ball2();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_ball2();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      o->xvel(-23);
	      g->add_object(o);
      
	      o = new object_ball2();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      o->xvel(23);
	      g->add_object(o);
	      break;
	    case 5 :
	      o = new object_ball2();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_ball3();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      o->xvel(-28);
	      g->add_object(o);
      
	      o = new object_ball3();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      o->xvel(28);
	      g->add_object(o);
	      break;
	    case 6 :
	      o = new object_ball3();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      g->add_object(o);
      
	      o = new object_ball3();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      o->xvel(-28);
	      g->add_object(o);
      
	      o = new object_ball3();
	      o->xpos(xpos());
	      o->ypos(ypos() + tbound());
	      o->xvel(28);
	      g->add_object(o);
	      break;
	    }
	    break;
	  }

	  _fire_last_time = g->current_time();
	}
      }

      if ( _energy_depot > 0 ) {
	Uint32 e = _energy_depot;
      
	if ( e > dt * PLAYER_MAX_ENERGY / 2000 )
	  e = dt * PLAYER_MAX_ENERGY / 2000;

	_energy += e;
	_energy_depot -= e;

	if ( _energy > PLAYER_MAX_ENERGY ) {
	  _energy = PLAYER_MAX_ENERGY;
	  _energy_depot = 0;
	}
      }

      if ( (_score_depot > 0) && (g->current_time() - _score_depot_last_time > 200) ) {
	Uint32 shift = 1000000000;
	while ( shift > 0 ) {
	  if ( _score_depot / shift != 0 ) {
	    g->play_sound(_snd_score_id, SOUND_LAYER_MISC);

	    _score += shift;
	    _score_depot -= shift;
	    _score_depot_last_time = g->current_time();
	    break;
	  }

	  shift /= 10;
	}
      }

      if ( _shield_cnt > 0 ) {
	if ( dt < _shield_cnt ) {
	  _shield_cnt -= dt;

	  part *p = get_part(10);
	  if ( p ) 
	    p->_enable_paint = (_shield_cnt > 1000) || (_shield_cnt % 100 < 50);
	}
	else {
	  unset_shield();
	}
      }

      if ( _invul_cnt > 0 ) {
	if ( dt < _invul_cnt )
	  _invul_cnt -= dt;
	else {
	  _invul_cnt = 0;
	  update_hit_mask();
	}
      }

      if ( g->current_time() - _flame_last_time > 1000 / 20 ) {
	_flame_current = 7 + (rand() % 14);
	_flame_last_time = g->current_time();
      }
    }
  }

  _fire_flag = false;
  _last_time = g->current_time();

  object::move();
}

Uint16 object_player::current_frame(part *p) const {
  Sint16 f;

  switch ( p->_id ) {
  case 1 : /* body */
    f = (3 * _req_xvel + MAX_XVEL - 1) / MAX_XVEL + 3;
    if ( f > 6 )
      f = 6;
    else if ( f < 0 )
      f = 0;
    break;

  case 2 : /* turbine */
    f = _flame_current;
    break;

  default :
    f = object::current_frame(p);
    break;
  }

  return( f );
}

void object_player::paint() {
  if ( (_invul_cnt == 0) || (_invul_cnt % 70 < 35) ) {
    object::paint();
  }
}

bool object_player::damage(part *p, Uint32 e) {
  bool destroyed = false;

  destroyed = object::damage(p, e);

  return( destroyed );
}

void object_player::hit_by(object *) {
}

void object_player::hit(object *) {
}

void object_player::destroy() {
  flags(flags() | OBJECT_INVISIBLE | PLAYER_KILLED);

  _killed_time = g->current_time();
  _expl_delay = 500;

  g->play_expl_sound(SOUND_EXPL_1, SOUND_LAYER_PLAYER);

  for ( int i = 0 ; i < 12 ; ++i ) {
    object *o = new object_expl3();
    o->xpos(xpos());
    o->ypos(ypos());
    o->xvel((75 + (rand() % 150)) * ((rand() & 16) ? -1 : 1));
    o->yvel((75 + (rand() % 150)) * ((rand() & 16) ? -1 : 1));
    g->add_object(o);
  }
}

void object_player::add_smart_bomb() {
  // TODO
  _smart_bombs++;
}

void object_player::drop_smart_bomb() {
  // TODO
}

void object_player::set_gun(Uint16 t) {
  if ( t != PLAYER_GUN_SEEKER )
    _gun_type = t;

  if ( _gun_power[t] < 6 )
    _gun_power[t]++;
}

void object_player::select_gun(Uint16 t) {
  _gun_type = t;
}

void object_player::unset_gun(Uint16 t) {
  if ( _gun_power[t] > 0 )
    _gun_power[t]--;

  if ( _gun_power[t] == 0 ) {
    if ( t == PLAYER_GUN_LASER )
      _gun_power[t] = 1;

    select_gun(PLAYER_GUN_LASER);
  }
}

Uint16 object_player::seeker_load() const {
  Uint16 res = 0;
  if ( _seeker_enabled && (_gun_power[PLAYER_GUN_SEEKER] > 0) && (g->current_time() > _seeker_start + 200) ) {
    res = 256 * (g->current_time() - _seeker_start - 200) / 1000;
    if ( res > 256 )
      res = 256;
  }
  return( res );
}

void object_player::set_shield(Uint32 s) {
  _shield_cnt = s;

  part *p = add_part(10);
  p->_surface = g->get_bob(_shield_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_per_second = 20;
  p->_frames_count = p->_surface->frames();
  p->_layer = 2;
  p->_enable_collision = false;

  update_hit_mask();
}

void object_player::unset_shield() {
  _shield_cnt = 0;

  rem_part(10);

  update_hit_mask();
}

void object_player::set_invul(Uint32 s) {
  _invul_cnt = s;
  update_hit_mask();
}

void object_player::unset_invul() {
  _invul_cnt = 0;
  update_hit_mask();
}

bool object_player::setup() {
  if ( object_plasma1::setup() &&
       object_plasma1l::setup() &&
       object_plasma1r::setup() &&
       object_plasma2::setup() &&
       object_plasma2l::setup() &&
       object_plasma2r::setup() &&
       object_plasma3::setup() &&
       object_plasma3l::setup() &&
       object_plasma3r::setup() &&
       object_laser1::setup() &&
       object_laser2::setup() &&
       object_laser3::setup() &&
       object_ball1::setup() &&
       object_ball2::setup() &&
       object_ball3::setup() &&
       object_seeker::setup() &&
       object_expl1::setup() &&
       object_expl2::setup() &&
       object_expl3::setup() ) {
    _image_id = g->obtain_bob("pship");
    _shield_id = g->obtain_bob("shield");
    _snd_laser_id = g->obtain_sound("laser");
    _snd_plasma_id = g->obtain_sound("plasma");
    _snd_ball_id = g->obtain_sound("ball");
    _snd_seeker_id = g->obtain_sound("seeker");
    _snd_score_id = g->obtain_sound("boing");

    if ( (_image_id != 0xffff) && (_shield_id != 0xffff) &&
	 (_snd_laser_id != 0xffff) && (_snd_plasma_id != 0xffff) && (_snd_ball_id != 0xffff) && (_snd_seeker_id != 0xffff) &&
	 (_snd_score_id != 0xffff) )
      return( true );
  }

  return( false );
}

void object_player::cleanup() {
  g->release_sound(_snd_score_id);
  g->release_sound(_snd_seeker_id);
  g->release_sound(_snd_ball_id);
  g->release_sound(_snd_plasma_id);
  g->release_sound(_snd_laser_id);
  g->release_bob(_image_id);
  g->release_bob(_shield_id);

  object_expl3::cleanup();
  object_expl2::cleanup();
  object_expl1::cleanup();
  object_seeker::cleanup();
  object_ball1::cleanup();
  object_ball2::cleanup();
  object_ball3::cleanup();
  object_laser1::cleanup();
  object_laser2::cleanup();
  object_laser3::cleanup();
  object_plasma1::cleanup();
  object_plasma1r::cleanup();
  object_plasma1l::cleanup();
  object_plasma2::cleanup();
  object_plasma2r::cleanup();
  object_plasma2l::cleanup();
  object_plasma3::cleanup();
  object_plasma3r::cleanup();
  object_plasma3l::cleanup();
}
