
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_player.h,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#ifndef _OBJECT_PLAYER_H_
#define _OBJECT_PLAYER_H_

#include "object.h"

#define PLAYER_MAX_ENERGY 1000

#define PLAYER_GUN_LASER 0
#define PLAYER_GUN_PLASMA 1
#define PLAYER_GUN_BALL 2
#define PLAYER_GUN_SEEKER 3

#define PLAYER_KILLED 0x80000000
#define PLAYER_KILLED_DONE 0x40000000

class object_player : public object {
 protected:
  static Uint16 _image_id;
  static Uint16 _shield_id;
  static Uint16 _snd_laser_id;
  static Uint16 _snd_plasma_id;
  static Uint16 _snd_ball_id;
  static Uint16 _snd_seeker_id;
  static Uint16 _snd_score_id;

  Sint32 _req_xvel, _req_yvel;
  
  class surface *_shield;
  Sint32 _shield_cnt;
  Sint32 _invul_cnt;

  Uint32 _lifes;
  Uint32 _energy_depot;
  Uint32 _score_depot;
  Uint32 _score_depot_last_time;

  bool _enable_steering;
  bool _fire_flag;
  Uint32 _fire_last_time;

  Uint16 _flame_current;
  Uint32 _flame_last_time;

  Uint16 _gun_type;
  Uint16 _gun_power[4];
  Uint32 _smart_bombs;

  Uint32 _seeker_start;
  Uint32 _seeker_enabled;

  Uint32 _killed_time;
  Uint32 _expl_last_time;
  Uint32 _expl_delay;

  void update_hit_mask();

 public:
  object_player();
  ~object_player();

  void move();
  void paint();

  void initialize();
  Uint16 current_frame(part *) const;
  bool damage(part *, Uint32);
  void hit_by(object *);
  void hit(object *);
  void destroy();

  static bool setup();
  static void cleanup();

  void startup();
  void add_score(Uint32 s) { _score += s; }
  void add_score_to_depot(Uint32 s) { _score_depot += s; }

  Uint32 lifes() const { return( _lifes ); }
  void lifes(Uint32 l) { _lifes = l; }
  void add_life() { _lifes++; };
  void sub_life() { _lifes--; };

  Uint16 smart_bombs() const { return( _smart_bombs ); }
  void smart_bombs(Uint16 n) { _smart_bombs = n; }
  void add_smart_bomb();

  void set_fire_flag() { _fire_flag = true; }
  void drop_smart_bomb();

  void add_energy(Uint32 e) {
    _energy_depot += e;
  }

  void set_gun(Uint16);
  void unset_gun(Uint16);
  Uint16 gun_power(Uint16 i) const { return( _gun_power[i] ); }
  void select_gun(Uint16);
  Uint16 gun_type() const { return( _gun_type ); }

  Uint16 seeker_load() const;

  bool has_shield() const { return( _shield_cnt > 0 ); }
  void set_shield(Uint32 v);
  void set_shield() { set_shield(10000); }
  void unset_shield();

  void set_invul(Uint32 v);
  void set_invul() { set_invul(5000); }
  void unset_invul();

  void enable_steering() { _enable_steering = true; }
  void disable_steering() { _enable_steering = false; }
};

#endif
