
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_saucer.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include "main.h"
#include "game.h"
#include "object_saucer.h"
#include "object_expl.h"

//$TAB { "saucer", object_saucer::constructor, object_saucer::setup, object_saucer::cleanup },

Uint16 object_saucer::_image_id = 0xffff;

object_saucer::object_saucer() : object() {
  part *p = add_part(1);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames() / 4;
  p->_frames_loops = (rand() % 3) + 2;
  p->_phase_id = 0;
  update_size();

  xpos( rand() % g->map_width() );
  ypos( -height() );
  xvel( 0 );
  yvel( 90 );
  xacc( 0 );
  yacc( 0 );

  _layer = LAYER_ENEMIES;

  _hit_mask = OBJECT_PLAYER_SHIP | OBJECT_PLAYER_PROJECTILE;
  _type_mask = OBJECT_ENEMY_SHIP;

  _energy = 400;
  _score = 400;
}

void object_saucer::set_modifiers(svector<object_modifier> &tab) {
  for ( svector<object_modifier>::const_iterator i = tab.begin() ; i != tab.end() ; ++i ) {
    if ( i->name == "delay" ) {
      part *p = get_part(1);
      if ( p )
	p->_frames_loops = i->val;
    }
  }

  object::set_modifiers(tab);
}

void object_saucer::finish_phase(part *p) {
  if ( p->_id == 1 ){
    switch ( p->_phase_id ) {
    case 0 :
      yacc( -180 );

      p->_frames_loops = 1;
      p->_frames_offset = p->_surface->frames() / 4;
      p->_frames_count = 12 * p->_surface->frames() / 32;
      p->_phase_id = 1;
      break;

    case 1 :
      for ( Sint16 xv = -45 ; xv <= 45 ; xv += 45 ) {
	object *o = new object_saucer_projectile();
	o->xpos(xpos());
	o->ypos(ypos());
	o->xvel(xv);
	o->yvel(120);
	g->add_object(o);
      }

      p->_frames_loops = 1;
      p->_frames_offset += p->_frames_count;
      p->_frames_count = 4 * p->_surface->frames() / 32;
      p->_phase_id = 2;
      break;

    case 2 :
      p->_frames_loops = -1;
      p->_frames_offset += p->_frames_count;
      p->_frames_count = p->_surface->frames() / 4;
      p->_phase_id = 3;
      break;
    }
  }

  object::finish_phase(p);
}

void object_saucer::destroy() {
  object::destroy();

  g->play_expl_sound(SOUND_EXPL_1, SOUND_LAYER_ENEMY, 96);

  for ( int i = 0 ; i < 6 ; ++i ) {
    object *o = new object_expl3();
    o->xpos(xpos());
    o->ypos(ypos());
    o->xvel((75 + (rand() % 150)) * ((rand() & 16) ? -1 : 1));
    o->yvel((75 + (rand() % 150)) * ((rand() & 16) ? -1 : 1));
    g->add_object(o);
  }

  object *o = new object_expl1();
  o->xpos(xpos());
  o->ypos(ypos());
  o->xvel(xvel());
  o->yvel(yvel());
  g->add_object(o);
}

object *object_saucer::constructor() {
  return( new object_saucer() );
}

bool object_saucer::setup() {
  if ( object::setup() && 
       object_expl1::setup() &&
       object_expl3::setup() &&
       object_saucer_projectile::setup() ) {
    _image_id = g->obtain_bob("saucer");
    if ( _image_id != 0xffff )
      return( true );
  }

  return( false );
}

void object_saucer::cleanup() {
  g->release_bob(_image_id);

  object_saucer_projectile::cleanup();
  object_expl3::cleanup();
  object_expl1::cleanup();
  object::cleanup();
}

/****************************************************************
 *                                                              *
 * class object_saucer_projectile                               *
 *                                                              *
 ****************************************************************/

Uint16 object_saucer_projectile::_image_id = 0xffff;

object_saucer_projectile::object_saucer_projectile() : object() {
  part *p = add_part(0);
  p->_surface = g->get_bob(_image_id);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _layer = LAYER_PROJECTILES;

  _hit_mask = OBJECT_PLAYER_SHIP;
  _type_mask = OBJECT_ENEMY_PROJECTILE;

  _energy = 100;
}

void object_saucer_projectile::destroy() {
  object::destroy();

  object *o = new object_expl3();
  o->xpos(xpos());
  o->ypos(ypos());
  o->xvel(xvel());
  o->yvel(yvel());
  g->add_object(o);
}

bool object_saucer_projectile::setup() {
  if ( object_expl3::setup() ) {
    _image_id = g->obtain_bob("saucerp");
    if ( _image_id != 0xffff )
      return( true );
  }

  return( false );
}

void object_saucer_projectile::cleanup() {
  g->release_bob(_image_id);

  object_expl3::cleanup();
}
