
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: object_shot.cc,v 1.1 2000/10/11 20:04:05 pink Exp pink $ */

#include <stdlib.h>
#include <math.h>
#include "main.h"
#include "game.h"
#include "object_player.h"
#include "object_shot.h"

Uint16 object_shot::_image_id[SHOT_COUNT] = { 0xffff, 0xffff };

object_shot::object_shot(Uint16 t) : object() {
  part *p = add_part(1);
  p->_surface = g->get_bob(_image_id[t]);
  p->_xoffset = -p->_surface->width() / 2;
  p->_yoffset = -p->_surface->height() / 2;
  p->_frames_count = p->_surface->frames();
  update_size();

  _type = t;

  _layer = LAYER_PROJECTILES;

  _hit_mask = OBJECT_PLAYER_SHIP;
  _type_mask = OBJECT_ENEMY_PROJECTILE;

  switch ( t ) {
  case 0 :
    _energy = 100;
    break;
  case 1 :
    _energy = 120;
    break;
  }
}

Uint16 object_shot::current_frame(part *p) const {
  Sint16 f;

  switch ( p->_id ) {
  case 1 :
    switch ( _type ) {
    case 1 :
      f = (p->_frames_count * (_dir >> 8) / 180) % p->_frames_count;
      break;

    default :
      f = object::current_frame(p);
      break;
    }
    break;

  default :
    f = object::current_frame(p);
    break;
  }

  return( f );
}

void object_shot::initialize() {
  double vel;

  switch ( _type ) {
  case 0 :
    vel = 100;
    break;
  case 1 :
    vel = 200;
    break;
  }

  object *pl = g->player();
  double dx = pl->xpos() - xpos();
  double dy = pl->ypos() - ypos();
  double d;
  
  if ( dy > 0 )
    d = atan(-dx / dy) + 2 * M_PI;
  else if ( dy < 0 )
    d = atan(-dx / dy) + M_PI;
  else if ( dx > 0 )
    d = 1.5 * M_PI;
  else
    d = 0.5 * M_PI;

  _dir = ((Sint16)(180.0 * d / M_PI) % 360) << 8;

  xvel((Sint16)(-vel * sin(d)));
  yvel((Sint16)(vel * cos(d)));
}

bool object_shot::setup() {
  _image_id[0] = g->obtain_bob("shot1");
  _image_id[1] = g->obtain_bob("shot2");

  if ( (_image_id[0] != 0xffff) && (_image_id[1] != 0xffff)  )
    return( true );

  return( false );
}

void object_shot::cleanup() {
  g->release_bob(_image_id[0]);
  g->release_bob(_image_id[1]);
}
