
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id$ */

#ifndef _SSTRING_H_
#define _SSTRING_H_

#include <string.h>

class sstring {
 private:
  char *_buf;
  size_t _len;
  size_t _size;

 public:
  sstring() {
    _buf = new char[16];
    _len = 0;
    _size = 16;
  }

  sstring(const sstring &s) {
    _len = s._len;
    _size = _len + 1;
    _buf = new char[_size];
    strcpy(_buf, s._buf);
  }

  sstring(const char *s) {
    _len = strlen(s);
    _size = _len + 1;
    _buf = new char[_size];
    strcpy(_buf, s);
  }

  ~sstring() {
    delete _buf;
  }

  size_t size() const { return( _len ); }

  sstring& operator= (const sstring &s) {
    if ( s._len + 1 > _size ) {
      delete _buf;
      _size = s._len + 1;
      _buf = new char[_size];
    }

    strcpy(_buf, s._buf);
    _len = s._len;

    return(*this);
  }

  sstring& operator= (const char *s) {
    size_t len = strlen(s);

    if ( len + 1 > _size ) {
      delete _buf;
      _size = len + 1;
      _buf = new char[_size];
    }

    strcpy(_buf, s);
    _len = len;

    return(*this);
  }

  sstring& operator+= (const sstring &s) {
    size_t len = _len + s._len;

    if ( len + 1 > _size ) {
      _size = _len + 1;
      char *tmp = new char[_size];
      strcpy(tmp, _buf);
      delete _buf;
      _buf = tmp;
    }

    strcpy(_buf + _len, s._buf);
    _len = len;
  }

  sstring& operator+= (const char *s) {
    size_t len = _len + strlen(s);

    if ( len + 1 > _size ) {
      _size = _len + 1;
      char *tmp = new char[_size];
      strcpy(tmp, _buf);
      delete _buf;
      _buf = tmp;
    }

    strcpy(_buf + _len, s);
    _len = len;
  }

  operator char *() const {
    return(_buf);
  }
};

inline bool operator== (const sstring &a, const sstring &b) {
  return( strcmp((const char *)a, (const char *)b) ? false : true );
}

inline bool operator== (const sstring &a, const char *b) {
  return( strcmp((const char *)a, b) ? false : true );
}

inline bool operator== (const char *a, const sstring &b) {
  return( strcmp(a, (const char *)b) ? false : true );
}

inline bool operator!= (const sstring &a, const sstring &b) {
  return( strcmp((const char *)a, (const char *)b) ? true : false );
}

inline bool operator!= (const sstring &a, const char *b) {
  return( strcmp((const char *)a, b) ? true : false );
}

inline bool operator!= (const char *a, const sstring &b) {
  return( strcmp(a, (const char *)b) ? true : false );
}

inline bool operator< (const sstring &a, const sstring &b) {
  return( strcmp((const char *)a, (const char *)b) < 0 );
}

inline bool operator< (const sstring &a, const char *b) {
  return( strcmp((const char *)a, b) < 0 );
}

inline bool operator< (const char *a, const sstring &b) {
  return( strcmp(a, (const char *)b) < 0 );
}

inline bool operator<= (const sstring &a, const sstring &b) {
  return( strcmp((const char *)a, (const char *)b) <= 0 );
}

inline bool operator<= (const sstring &a, const char *b) {
  return( strcmp((const char *)a, b) <= 0 );
}

inline bool operator<= (const char *a, const sstring &b) {
  return( strcmp(a, (const char *)b) <= 0 );
}

inline bool operator> (const sstring &a, const sstring &b) {
  return( strcmp((const char *)a, (const char *)b) > 0 );
}

inline bool operator> (const sstring &a, const char *b) {
  return( strcmp((const char *)a, b) > 0 );
}

inline bool operator> (const char *a, const sstring &b) {
  return( strcmp(a, (const char *)b) > 0 );
}

inline bool operator>= (const sstring &a, const sstring &b) {
  return( strcmp((const char *)a, (const char *)b) >= 0 );
}

inline bool operator>= (const sstring &a, const char *b) {
  return( strcmp((const char *)a, b) >= 0 );
}

inline bool operator>= (const char *a, const sstring &b) {
  return( strcmp(a, (const char *)b) >= 0 );
}

inline sstring operator+ (const sstring &a, const sstring &b) {
  sstring r(a);
  r += b;
  return r;
}

inline sstring operator+ (const sstring &a, const char *b) {
  sstring r(a);
  r += b;
  return r;
}

inline sstring operator+ (const char *a, const sstring &b) {
  sstring r(a);
  r += b;
  return r;
}

#endif
