
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id: surface.h,v 1.1 2000/10/11 20:04:06 pink Exp pink $ */

#ifndef _SURFACE_H_
#define _SURFACE_H_

#include <stdio.h>
#include "SDL.h"
#include "font.h"
#include "rectangle.h"

class surface {
 protected:
  Uint16 _width, _height, _frames;

  SDL_Surface **_surface;
  rectangle *_bound;
  Uint16 _last_frame;

  rectangle _clip_box;

  Uint16 _current_frame;

 public:
  surface();
  surface(SDL_Surface *);
  surface(Uint16, Uint16, Uint16);
  ~surface();

  void set_video(SDL_Surface *);
  void add_sdl_surface(const rectangle &, SDL_Surface *);

  static surface *load_image(const char *);
  static surface *load_image(const char **, const char **, const char *);

  bool lock(Uint16 fr) const {
    if ( !SDL_MUSTLOCK(_surface[fr]) )
      return( true );

    if ( SDL_LockSurface(_surface[fr]) < 0 )
      return( false );

    return( true );
  }

  void unlock(Uint16 fr) const {
    if(SDL_MUSTLOCK(_surface[fr]))
      SDL_UnlockSurface(_surface[fr]);
  }

  void display_format();

  rectangle get_bound(const rectangle &, Uint16);
  rectangle get_bound() { return( get_bound(rectangle(0, 0, width(), height()), 0) ); }

  void clip_box(const rectangle &r) { _clip_box = r; }
  rectangle clip_box() const { return( _clip_box ); }

  Sint16 width() const { return( _width ); }
  Sint16 height() const { return( _height ); }
  Uint16 frames() const { return( _frames ); }
  rectangle bound(Uint16 f) const { return( _bound[f] ); }
  Uint32 memory() const;

  void frame(Uint16 f) { _current_frame = f; }
  Uint16 frame() const { return( _current_frame ); }

  void draw_pixel(Sint16, Sint16, Uint8, Uint8, Uint8);
  void draw_pixels(Uint16, Sint16 *, Sint16 *, Uint8 *, Uint8 *, Uint8 *);
  void draw_rect(const rectangle &, Uint8, Uint8, Uint8);
  void draw_text(Sint16, Sint16, bool, font *, const char *);
  void draw_mono_text(Sint16, Sint16, bool, Uint8, Uint8, Uint8, font *, const char *);
  void draw_bob(Sint16, Sint16, Sint16, surface *);

  void copy(Sint16, Sint16, const surface *, Uint16, const rectangle &);
  void copy(Sint16 x, Sint16 y, const surface *s, const rectangle &r) {
    copy(x, y, s, 0, r);
  }

  bool check_intersection(const surface *, Uint16, Uint16, Sint16, Sint16) const;
  bool check_intersection(const surface *s, Sint16 x, Sint16 y) const {
    return( check_intersection(s, x, y) );
  }
};

#endif
