
/*

   (c) 2000 by A Vision of Paradise


   This file is part of the game 'Gamma Patrol'.

   'Gamma Patrol' is SHAREWARE, please look at the files 'README' and
   'LICENCE' for further details.

   The source code is NOT OpenSource, GPL'd or what ever!
   You can use it for educational purposes, but you are not allowed
   to distribute modified or partial version of 'Gamma Patrol'.

*/

/* $Id:$ */

#include <SDL.h>
#include "surface_video.h"
#include "main.h"

surface_video::surface_video(Uint16 w, Uint16 h) : surface(w, h, 2) {
}

surface_video::~surface_video() {
  if ( _surface[0] && (_surface[0]->flags & SDL_FULLSCREEN) )
    SDL_WM_ToggleFullScreen(_surface[0]);

  _surface[0] = NULL; /* don't free video surface */
}

bool surface_video::initialize(Uint32 flags) {
  bool success = false;

  const SDL_VideoInfo *info = SDL_GetVideoInfo();
  if ( info->hw_available ) {
    flags |= SDL_HWSURFACE;
    printf("Requesting hardware surface\n");
  }

  if ( (flags & SDL_HWSURFACE) == SDL_HWSURFACE ) {
    if ( info->video_mem*1024 >= 4*_width*_height ) {
      flags |= SDL_DOUBLEBUF;
      printf("Requesting double buffering (got %ld bytes video memory, need %ld bytes)\n", info->video_mem*1024, 4*_width*_height);
    }
    else
      flags &= ~SDL_HWSURFACE;
  }

  Uint32 bpp = SDL_VideoModeOK(_width, _height, 16, flags);
  if ( bpp != 0 ) {
    _surface[0] = SDL_SetVideoMode(_width, _height, bpp, flags);
    if ( _surface[0] ) {

      if ( (flags & SDL_DOUBLEBUF) && !(_surface[0]->flags & SDL_DOUBLEBUF) && (_surface[0]->flags & SDL_HWSURFACE) ) {
	_surface[1] = SDL_CreateRGBSurface(SDL_HWSURFACE, _width, _height, bpp, 
					   _surface[0]->format->Rmask, _surface[0]->format->Gmask, _surface[0]->format->Bmask, _surface[0]->format->Amask);
	if ( _surface[1] ) {
	  _current_frame = 1;

	  success = true;
	}
      }
      else
	success = true;
    }
    else {
      show_message("Couldn't set display mode: %s\n", SDL_GetError());
    }
  }
  else {
    show_message("Requested mode (%dx%d) not available\n", _width, _height);
  }

  return( success );
}

void surface_video::enable_fullscreen(bool e) {
  if ( e && !(_surface[0]->flags & SDL_FULLSCREEN) ) 
    SDL_WM_ToggleFullScreen(_surface[0]);
  else if ( !e && (_surface[0]->flags & SDL_FULLSCREEN) ) 
    SDL_WM_ToggleFullScreen(_surface[0]);
}

bool surface_video::fullscreen_enabled() const {
  return( (_surface[0]->flags & SDL_FULLSCREEN) ? true : false );
}

void surface_video::update() {
  if ( _surface[1] )
    SDL_BlitSurface(_surface[1], NULL, _surface[0], NULL);
  else if ( _surface[0]->flags & SDL_DOUBLEBUF )
    SDL_Flip(_surface[0]);
  else
    SDL_UpdateRect(_surface[0], 0, 0, 0, 0);
}

void surface_video::update_flip() {
  if ( _surface[0]->flags & SDL_DOUBLEBUF )
    SDL_Flip(_surface[0]);
}

void surface_video::update(const rectangle &r) {
  if ( _surface[1] ) {
    SDL_Rect sr = *r.rect();

    SDL_BlitSurface(_surface[1], &sr, _surface[0], &sr);
  }
  else if ( !(_surface[0]->flags & SDL_DOUBLEBUF) )
    SDL_UpdateRect(_surface[0], r.x(), r.y(), r.w(), r.h());
}
